@@ -1,8 +1,8 @@
 /*
 cartlib.c - Flash Linker Advance support for uCON64
 
-Copyright (c) 2001        Jeff Frohwein
-Copyright (c) 2002 - 2004 dbjh
+Copyright (c) 2001              Jeff Frohwein
+Copyright (c) 2002 - 2004, 2020 dbjh
 
 
 This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "config.h"
 #endif
 
-
 #ifdef  USE_PARALLEL
 
 // *** GBA flash cart support routines in GCC ***
@@ -282,8 +281,8 @@ CartTypeDetect (void)
 
   WriteFlash (_CART_START, INTEL28F_RIC);       // Read Identifier codes from flash.
   // Works for intel 28F640J3A & Sharp LH28F320BJE.
-  Manuf = ReadFlash (_CART_START);
-  Device = ReadFlash (_CART_START + _MEM_INC);
+  Manuf = (unsigned short) ReadFlash (_CART_START);
+  Device = (unsigned short) ReadFlash (_CART_START + _MEM_INC);
 
   switch (Manuf)
     {
@@ -303,13 +302,13 @@ CartTypeDetect (void)
           break;
         default:
           // Check to see if this is a Visoly "Turbo" cart
-          Device = ReadFlash (_CART_START + _MEM_INC + _MEM_INC);
+          Device = (unsigned short) ReadFlash (_CART_START + _MEM_INC + _MEM_INC);
           switch (Device)
             {
             case 0x16:         // 2 x i28F320J3A
             case 0x17:         // 2 x i28F640J3A
             case 0x18:         // 2 x i28F128J3A
-              type = Device + 0x80;
+              type = (unsigned char) (Device + 0x80);
               break;
             }
         }
@@ -375,15 +374,14 @@ EraseNonTurboFABlocks (u32 StartAddr, u32 BlockCount)
   u16 k;
   u16 Ready = 1;
   u32 i = 0;
-  u32 Timeout;
   time_t starttime = time (NULL);
 
   for (k = 0; k < BlockCount; k++)
     {
-      i = StartAddr + (k * 65536 * _MEM_INC);
+      u32 Timeout = FP_TIMEOUT2;
 
+      i = StartAddr + (k * 65536 * _MEM_INC);
       Ready = 0;
-      Timeout = FP_TIMEOUT2;
 
       while ((Ready == 0) && (Timeout != 0))
         {
@@ -434,8 +432,7 @@ EraseNonTurboFABlocks (u32 StartAddr, u32 BlockCount)
         }
       else
         break;
-      ucon64_gauge (starttime, (k + 1) * 128 * 1024,
-                     BlockCount * 128 * 1024);
+      ucon64_gauge (starttime, (k + 1) * 128 * 1024, BlockCount * 128 * 1024);
     }
 
   if (!Ready)
@@ -460,18 +457,16 @@ u32
 EraseTurboFABlocks (u32 StartAddr, u32 BlockCount)
 {
   u16 j, k;
-  u16 done1, done2;
   u16 Ready = 1;
   u32 i = 0;
-  u32 Timeout;
   time_t starttime = time (NULL);
 
   for (k = 0; k < BlockCount; k++)
     {
-      i = StartAddr + (k * 131072 * _MEM_INC);
+      u32 Timeout = FP_TIMEOUT2;
 
+      i = StartAddr + (k * 131072 * _MEM_INC);
       Ready = 0;
-      Timeout = FP_TIMEOUT2;
 
       while ((!Ready) && (Timeout != 0))
         {
@@ -482,8 +477,9 @@ EraseTurboFABlocks (u32 StartAddr, u32 BlockCount)
 
       if (Ready)
         {
-          done1 = 0;
-          done2 = 0;
+          u16 done1 = 0;
+          u16 done2 = 0;
+
           Ready = 0;
           Timeout = FP_TIMEOUT3;
 
@@ -525,8 +521,7 @@ EraseTurboFABlocks (u32 StartAddr, u32 BlockCount)
         }
       else
         break;
-      ucon64_gauge (starttime, (k + 1) * 256 * 1024,
-                     BlockCount * 256 * 1024);
+      ucon64_gauge (starttime, (k + 1) * 256 * 1024, BlockCount * 256 * 1024);
     }
 
   if (!Ready)
@@ -544,7 +539,7 @@ EraseTurboFABlocks (u32 StartAddr, u32 BlockCount)
 }
 #endif
 
-#ifdef NOA_FLASH_CART_SUPPORT
+#if 0 // #ifdef NOA_FLASH_CART_SUPPORT
 // Write 2 x Length bytes to official Nintendo flash cart.
 // Function returns true if write was successful.
 
@@ -553,11 +548,7 @@ WriteNintendoFlashCart (u32 SrcAddr, u32 FlashAddr, u32 Length)
 {
   int j;
   int LoopCount = 0;
-  u16 *SrcAddr2 = (u16 *)
-#ifdef  __LP64__
-                  (u64)
-#endif
-                  SrcAddr;
+  u16 *SrcAddr2 = (u16 *) SrcAddr;
 
   while (LoopCount < (int) Length)
     {
@@ -586,7 +577,7 @@ WriteNintendoFlashCart (u32 SrcAddr, u32 FlashAddr, u32 Length)
 }
 #endif
 
-#ifdef NONTURBO_FA_SUPPORT
+#if 0 // #ifdef NONTURBO_FA_SUPPORT
 // Write 32 x Length bytes to older (non-Turbo) FA/Visoly flash cart.
 // Function returns true if write was successful.
 
@@ -594,18 +585,14 @@ u32
 WriteNonTurboFACart (u32 SrcAddr, u32 FlashAddr, u32 Length)
 {
   int Ready = 0;
-  int Timeout = 0;
   int LoopCount = 0;
-  u16 *SrcAddr2 = (u16 *)
-#ifdef  __LP64__
-                  (u64)
-#endif
-                  SrcAddr;
+  u16 *SrcAddr2 = (u16 *) SrcAddr;
 
   while (LoopCount < (int) Length)
     {
+      int Timeout = FP_TIMEOUT1;
+
       Ready = 0;
-      Timeout = FP_TIMEOUT1;
 
       while ((Ready == 0) && (Timeout != 0))
         {
@@ -675,7 +662,7 @@ WriteNonTurboFACart (u32 SrcAddr, u32 FlashAddr, u32 Length)
 }
 #endif
 
-#ifdef TURBO_FA_SUPPORT
+#if 0 // #ifdef TURBO_FA_SUPPORT
 // Write 64 x Length bytes to newer (Turbo) FA/Visoly flash cart.
 // Function returns true if write was successful.
 
@@ -683,22 +670,17 @@ u32
 WriteTurboFACart (u32 SrcAddr, u32 FlashAddr, u32 Length)
 {
   int i, k;
-  int done1, done2;
-  int Timeout;
   int Ready = 0;
   int LoopCount = 0;
-  u16 *SrcAddr2 = (u16 *)
-#ifdef  __LP64__
-                  (u64)
-#endif
-                  SrcAddr;
+  u16 *SrcAddr2 = (u16 *) SrcAddr;
 
   while (LoopCount < (int) Length)
     {
-      done1 = 0;
-      done2 = 0;
+      int done1 = 0;
+      int done2 = 0;
+      int Timeout = 0x4000;
+
       Ready = 0;
-      Timeout = 0x4000;
 
       while ((!Ready) && (Timeout != 0))
         {
