@@ -1,7 +1,8 @@
 /*
 cmc.c - Cyan's Megadrive ROM copier support for uCON64
 
-Copyright (c) 1999 - 2004 Cyan Helkaraxe
+Copyright (c) 1999 - 2004                      Cyan Helkaraxe
+Portions copyright (c) 2004, 2015 - 2017, 2021 dbjh
 
 Special thanks to dbjh for helping with the uCON64 integration
 of this software, and providing the wrapping code.
@@ -117,26 +118,29 @@ respective owners.
   confusion.
 */
 #define INTRO_TEXT "Cyan's Megadrive Copier             (c) 1999-2004 Cyan Helkaraxe\n" \
-                   "Software version 2.5 original, designed for hardware version 1.x\n\n"
+                   "Software version 2.5, designed for hardware version 1.x\n\n"
 
 
 #ifdef  HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdio.h>
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
+#endif
 #include <stdlib.h>
-#include <time.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
 #include <string.h>
+#include "misc/archive.h"
 #include "misc/bswap.h"
 #include "misc/misc.h"
-#include "misc/parallel.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
-#endif
-#include "misc/getopt2.h"                       // st_getopt2_t
+#include "misc/term.h"
 #include "ucon64.h"
 #include "ucon64_misc.h"
-#include "cmc.h"
+#include "backup/cmc.h"
+
 
 #ifdef  USE_PARALLEL
 
@@ -150,37 +154,32 @@ respective owners.
 #define MBYTE (1024 * 1024)
 #define DEFAULT_SPEED 3
 
-#ifdef  _MSC_VER
-// Visual C++ doesn't allow inline in C source code
-#define inline __inline
-#endif
-
 
 /************************
  *  Internal functions  *
  ************************/
 
 static inline void
-cyan_write_copier (unsigned char data, unsigned int parport)
+cyan_write_copier (unsigned char data, unsigned short parport)
 // write a value to the data register of the parallel port
 {
-  outportb ((unsigned short) (parport + PARPORT_DATA), data);
+  outportb (parport + PARPORT_DATA, data);
 }
 
 
 static inline unsigned char
-cyan_read_copier (unsigned int parport)
+cyan_read_copier (unsigned short parport)
 // read a value from the status register of the parallel port
 {
-  return inportb ((unsigned short) (parport + PARPORT_STATUS));
+  return inportb (parport + PARPORT_STATUS);
 }
 
 
 static inline unsigned char
-cyan_verify_copier (unsigned int parport)
+cyan_verify_copier (unsigned short parport)
 // read a value back from the data register for verification
 {
-  return inportb ((unsigned short) (parport + PARPORT_DATA));
+  return inportb (parport + PARPORT_DATA);
 }
 
 
@@ -326,32 +325,32 @@ cyan_get_address (unsigned long b)
 /**** non-hardware, indirectly accessing ****/
 
 static inline void
-cyan_delay (int speed, unsigned int parport)
+cyan_delay (int speed, unsigned short parport)
 // Delays a certain amount of time depending on speed selected. 0=long delay,
 //  used for reset and hi counter.
 {
-  int i, scritch = 0;
+  int i;
 
   switch (speed)
     {
     case 0:
       for (i = 0; i < 128; i++)
-        scritch += cyan_read_copier (parport);
+        cyan_read_copier (parport);
     case 1:                                     // falling through
       for (i = 0; i < 64; i++)
-        scritch += cyan_read_copier (parport);
+        cyan_read_copier (parport);
     case 2:                                     // falling through
       for (i = 0; i < 12; i++)
-        scritch += cyan_read_copier (parport);
+        cyan_read_copier (parport);
     case 3:                                     // falling through
-      scritch += cyan_read_copier (parport);
-      scritch += cyan_read_copier (parport);
+      cyan_read_copier (parport);
+      cyan_read_copier (parport);
     }
 }
 
 
 static void
-cyan_reset (unsigned int parport)
+cyan_reset (unsigned short parport)
 // resets the copier
 {
   cyan_delay (0, parport);
@@ -368,7 +367,7 @@ cyan_reset (unsigned int parport)
 
 
 static inline unsigned short
-cyan_get_word (int speed, unsigned int parport)
+cyan_get_word (int speed, unsigned short parport)
 // gets a byte pair from the ROM and return two bytes in big endian byte order
 {
   unsigned short value = 0;
@@ -444,7 +443,7 @@ check_exit (void)
 
 
 static unsigned char *
-cyan_read_rom (int speed, unsigned int parport, unsigned char *buffer)
+cyan_read_rom (int speed, unsigned short parport, unsigned char *buffer)
 /*
   Read the ROM and return a pointer to a 4 MB area of memory containing all ROM
   data. Designed to be used from inside cyan_copy_rom(), although it can be
@@ -509,7 +508,7 @@ cyan_read_rom (int speed, unsigned int parport, unsigned char *buffer)
 
 
 static void
-cyan_test_parport (unsigned int parport)
+cyan_test_parport (unsigned short parport)
 // Test the parallel port to see if it appears to be functioning correctly, and
 //  terminate if there's an error.
 {
@@ -611,7 +610,7 @@ cyan_test_parport (unsigned int parport)
 
 
 static int
-cyan_test_copier (int test, int speed, unsigned int parport)
+cyan_test_copier (int test, int speed, unsigned short parport)
 {
   unsigned char *buffer1, *buffer2 = NULL;
   int count = 1;
@@ -621,7 +620,7 @@ cyan_test_copier (int test, int speed, unsigned int parport)
 
   switch (test)
     {
-    // reliability test -- note: this test may be required to run for 8 hours or more
+    // reliability test -- NOTE: this test may be required to run for 8 hours or more
     case 1:
       printf ("Reliability test mode selected, speed %d\n", speed);
       cyan_test_parport (parport);
@@ -640,12 +639,12 @@ cyan_test_copier (int test, int speed, unsigned int parport)
         fputc ('\n', stdout);
       buffer1 = cyan_read_rom (speed, parport, NULL);
       if (!buffer1)                             // user abort
-        exit (0);
+        return 0;
 
       // detect if ROM is all 0x00 or 0xff and print an error if so
       cyan_calculate_rom_size (buffer1, 1);
 
-      while (1)
+      for (;;)
         {
           clear_line ();                        // remove last gauge
           printf ("   Pass %2d OK\n", count);
@@ -667,7 +666,7 @@ cyan_test_copier (int test, int speed, unsigned int parport)
           if (!buffer2)
             {
               free (buffer1);
-              exit (0);
+              return 0;
             }
           if (memcmp (buffer1, buffer2, 4 * MBYTE))
             {
@@ -694,15 +693,12 @@ cyan_test_copier (int test, int speed, unsigned int parport)
                       "You may wish to replace the battery or try another power supply, and use\n"
                       "shorter cables.\n"
                       "Make sure no electrical appliances turn on or off during the copy.\n"
-                      "Re-run the test to be sure; it's recommended that you use a lower speed\n");
+                      "Re-run the test to be sure; it is recommended that you use a lower speed\n");
               free (buffer1);
               free (buffer2);
-              exit (1);
+              return 1;
             }
         }
-
-      free (buffer1);
-      free (buffer2);
       break;
     // manual test
     case 2:
@@ -732,7 +728,7 @@ cyan_test_copier (int test, int speed, unsigned int parport)
             "If the above didn't make any sense to you, press escape or q and turn the ROM\n"
             "copier off immediately!\n"
             "This test is designed for advanced users only\n");
-      while (1)
+      for (;;)
         {
           const char *status[2] = {"* ", ". "};
 
@@ -794,7 +790,7 @@ cyan_test_copier (int test, int speed, unsigned int parport)
             {
               cyan_reset (parport);
               puts ("\nUser aborted test");
-              exit (0);
+              return 0;
             }
         }
       break;
@@ -802,12 +798,12 @@ cyan_test_copier (int test, int speed, unsigned int parport)
       fputs ("INTERNAL ERROR: Invalid test number passed to cyan_test_copier()\n", stderr);
       exit (1);
     }
-  return 0;
+//  return 0;
 }
 
 
 static int
-cyan_copy_rom (const char *filename, int speed, unsigned int parport)
+cyan_copy_rom (const char *filename, int speed, unsigned short parport)
 /*
   Copy a ROM file -- this assumes the filename is valid and the file does not
   already exist, since it will blindly try to write (overwrite) the filename you
@@ -904,6 +900,14 @@ cyan_copy_rom (const char *filename, int speed, unsigned int parport)
  * uCON64 wrapping *
  *******************/
 
+#ifdef  USE_PARALLEL
+static st_ucon64_obj_t cmc_obj[] =
+  {
+    {UCON64_GEN, WF_STOP | WF_NO_ROM},
+    {UCON64_GEN, WF_SWITCH}
+  };
+#endif
+
 const st_getopt2_t cmc_usage[] =
   {
     {
@@ -914,24 +918,24 @@ const st_getopt2_t cmc_usage[] =
 #ifdef  USE_PARALLEL
     {
       "xcmc", 0, 0, UCON64_XCMC,
-      NULL, "receive ROM from Cyan's Megadrive ROM copier; " OPTION_LONG_S "port=PORT",
-      &ucon64_wf[WF_OBJ_GEN_STOP_NO_ROM]
+      NULL, "receive ROM from Cyan's Megadrive ROM copier; " OPTION_LONG_S "port" OPTARG_S "PORT",
+      &cmc_obj[0]
     },
     {
       "xcmct", 1, 0, UCON64_XCMCT,
       "TEST", "run test TEST\n"
-      "TEST=1 burn-in reliability test (specify speed)\n"
-      "TEST=2 testbench mode (experts only)",
-      &ucon64_wf[WF_OBJ_GEN_STOP_NO_ROM]
+      "TEST" OPTARG_S "1 burn-in reliability test (specify speed)\n"
+      "TEST" OPTARG_S "2 testbench mode (experts only)",
+      &cmc_obj[0]
     },
     {
       "xcmcm", 1, 0, UCON64_XCMCM,
       "SPEED", "specify transfer speed\n"
-      "SPEED=1 slow (debug)\n"
-      "SPEED=2 medium\n"
-      "SPEED=3 fast (default)\n"                // verify with value of DEFAULT_SPEED
-      "SPEED=4 full speed (risky)",
-      &ucon64_wf[WF_OBJ_GEN_SWITCH]
+      "SPEED" OPTARG_S "1 slow (debug)\n"
+      "SPEED" OPTARG_S "2 medium\n"
+      "SPEED" OPTARG_S "3 fast (default)\n"     // verify with value of DEFAULT_SPEED
+      "SPEED" OPTARG_S "4 full speed (risky)",
+      &cmc_obj[1]
     },
 #endif // USE_PARALLEL
     {NULL, 0, 0, 0, NULL, NULL, NULL}
@@ -941,29 +945,35 @@ const st_getopt2_t cmc_usage[] =
 #ifdef  USE_PARALLEL
 
 int
-cmc_read_rom (const char *filename, unsigned int parport, int speed)
+cmc_read_rom (const char *filename, unsigned short parport, int speed)
 {
 #if     (defined __unix__ || defined __BEOS__) && !defined __MSDOS__
   init_conio ();
+  if (register_func (deinit_conio) == -1)
+    {
+      fputs ("ERROR: Could not register function with register_func()\n", stderr);
+      exit (1);
+    }
 #endif
 
   if (speed < 1 || speed > 4)
     speed = DEFAULT_SPEED;
   cyan_copy_rom (filename, speed, parport);
 
-#if     (defined __unix__ || defined __BEOS__) && !defined __MSDOS__
-  deinit_conio ();
-#endif
-
   return 0;
 }
 
 
 int
-cmc_test (int test, unsigned int parport, int speed)
+cmc_test (int test, unsigned short parport, int speed)
 {
 #if     (defined __unix__ || defined __BEOS__) && !defined __MSDOS__
   init_conio ();
+  if (register_func (deinit_conio) == -1)
+    {
+      fputs ("ERROR: Could not register function with register_func()\n", stderr);
+      exit (1);
+    }
 #endif
 
   if (test < 1 || test > 2)
@@ -975,10 +985,6 @@ cmc_test (int test, unsigned int parport, int speed)
     speed = DEFAULT_SPEED;
   cyan_test_copier (test, speed, parport);
 
-#if     (defined __unix__ || defined __BEOS__) && !defined __MSDOS__
-  deinit_conio ();
-#endif
-
   return 0;
 }
 
