@@ -1,7 +1,7 @@
 /*
 doctor64jr.h - Bung Doctor V64 Junior support for uCON64
 
-Copyright (c) 1999 - 2001 NoisyB <noisyb@gmx.net>
+Copyright (c) 1999 - 2001 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -21,11 +21,17 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef DOCTOR64JR_H
 #define DOCTOR64JR_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t doctor64jr_usage[];
 
-#ifdef USE_PARALLEL
-extern int doctor64jr_read (const char *filename, unsigned int parport);
-extern int doctor64jr_write (const char *filename, unsigned int parport);
+#ifdef  USE_PARALLEL
+extern int doctor64jr_read (const char *filename, unsigned short parport);
+extern int doctor64jr_write (const char *filename, unsigned short parport);
 #endif
 
 #endif
