@@ -2,7 +2,7 @@
 f2a.h - Flash 2 Advance support for uCON64
 
 Copyright (c) 2003 Ulrich Hecht <uli@emulinks.de>
-Copyright (c) 2004 NoisyB <noisyb@gmx.net>
+Copyright (c) 2004 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,12 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef F2A_H
 #define F2A_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t f2a_usage[];
 
 #if     defined USE_PARALLEL || defined USE_USB
