@@ -1,9 +1,9 @@
 /*
 fal.c - Flash Linker Advance support for uCON64
 
-Copyright (c) 2001        Jeff Frohwein
-Copyright (c) 2001        NoisyB <noisyb@gmx.net>
-Copyright (c) 2001 - 2004 dbjh
+Copyright (c) 2001                                  Jeff Frohwein
+Copyright (c) 2001                                  NoisyB
+Copyright (c) 2001 - 2005, 2015 - 2018, 2020 - 2021 dbjh
 
 
 This program is free software; you can redistribute it and/or modify
@@ -23,23 +23,33 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifdef  HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <string.h>
-#include "misc/misc.h"
-#include "misc/itypes.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
 #endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "ucon64.h"
-#include "ucon64_misc.h"
-#include "fal.h"
+#include <stdlib.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
+#include <string.h>
+#include "misc/archive.h"
 #include "misc/parallel.h"
+#include "misc/term.h"
+#include "ucon64_misc.h"
 #include "console/gba.h"
+#include "backup/fal.h"
 
 
+#ifdef  USE_PARALLEL
+static st_ucon64_obj_t fal_obj[] =
+  {
+    {UCON64_GBA, WF_DEFAULT | WF_STOP | WF_NO_ROM},
+    {UCON64_GBA, WF_DEFAULT | WF_STOP},
+    {UCON64_GBA, WF_STOP | WF_NO_ROM},
+    {UCON64_GBA, WF_SWITCH}
+  };
+#endif
+
 const st_getopt2_t fal_usage[] =
   {
     {
@@ -50,47 +60,47 @@ const st_getopt2_t fal_usage[] =
 #ifdef  USE_PARALLEL
     {
       "xfal", 0, 0, UCON64_XFAL,
-      NULL, "send/receive ROM to/from Flash Advance Linker; " OPTION_LONG_S "port=PORT\n"
+      NULL, "send/receive ROM to/from Flash Advance Linker; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically (32 Mbits) when ROM does not exist",
-      &ucon64_wf[WF_OBJ_GBA_DEFAULT_STOP_NO_ROM]
+      &fal_obj[0]
     },
     {
       "xfalmulti", 1, 0, UCON64_XFALMULTI, // send only
       "SIZE", "send multiple ROMs to Flash Advance Linker (makes temporary\n"
       "multi-game file truncated to SIZE Mbit); specify a loader in\n"
-      "the configuration file; " OPTION_LONG_S "port=PORT",
-      &ucon64_wf[WF_OBJ_GBA_DEFAULT_STOP]
+      "the configuration file; " OPTION_LONG_S "port" OPTARG_S "PORT",
+      &fal_obj[1]
     },
     {
       "xfalc", 1, 0, UCON64_XFALC,
-      "N", "receive N Mbits of ROM from Flash Advance Linker; " OPTION_LONG_S "port=PORT\n"
+      "N", "receive N Mbits of ROM from Flash Advance Linker; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "N can be 8, 16, 32, 64, 128 or 256",
-      &ucon64_wf[WF_OBJ_GBA_STOP_NO_ROM]
+      &fal_obj[2]
     },
     {
       "xfals", 0, 0, UCON64_XFALS,
-      NULL, "send/receive SRAM to/from Flash Advance Linker; " OPTION_LONG_S "port=PORT\n"
+      NULL, "send/receive SRAM to/from Flash Advance Linker; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when SRAM does not exist",
-      &ucon64_wf[WF_OBJ_GBA_STOP_NO_ROM]
+      &fal_obj[2]
     },
     {
       "xfalb", 1, 0, UCON64_XFALB,
       "BANK", "send/receive SRAM to/from Flash Advance Linker BANK\n"
-      "BANK can be 1, 2, 3 or 4; " OPTION_LONG_S "port=PORT\n"
+      "BANK can be 1, 2, 3 or 4; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when SRAM does not exist",
-      &ucon64_wf[WF_OBJ_GBA_STOP_NO_ROM]
+      &fal_obj[2]
     },
     {
       "xfalm", 0, 0, UCON64_XFALM,
-      NULL, "try to enable EPP mode, default is SPP mode",
-      &ucon64_wf[WF_OBJ_GBA_SWITCH]
+      NULL, "use SPP mode, default is EPP mode",
+      &fal_obj[3]
     },
 #endif // USE_PARALLEL
     {NULL, 0, 0, 0, NULL, NULL, NULL}
 };
 
 
-#ifdef USE_PARALLEL
+#ifdef  USE_PARALLEL
 
 /********************************************************/
 /* Flash Linker Advance                                 */
@@ -147,10 +157,10 @@ const st_getopt2_t fal_usage[] =
 //  5. If no diff get device Manuf ID for flash.
 //  6. If no Manuf ID detected then report no cart backup available.
 
-#define outpb(p, v) outportb((unsigned short) (p), (unsigned char) (v)); iodelay()
-#define inpb(p)     inportb((unsigned short) (p))
-#define outpw(p, v) outportw((unsigned short) (p), (unsigned char) (v)); iodelay()
-#define inpw(p)     inportw((unsigned short) (p))
+#define outpb(p, v) outportb ((unsigned short) (p), (unsigned char) (v)); iodelay ()
+#define inpb(p)     inportb ((unsigned short) (p))
+#define outpw(p, v) outportw ((unsigned short) (p), (unsigned short) (v)); iodelay ()
+#define inpw(p)     inportw ((unsigned short) (p))
 
 //#define HEADER_LENGTH 0xc0
 //#define OUTBUFLEN 256                   // Must be a multiple of 2! (ex:64,128,256...)
@@ -172,20 +182,17 @@ const st_getopt2_t fal_usage[] =
 #define u8      unsigned char
 #define u16     unsigned short int
 #define u32     unsigned int
-#define u64     unsigned long long int
-// u64 is needed to compile cartlib.c without warnings on Linux/x86_64
 #define CONST_U8 const unsigned char
 
 
 // ***Global Variables ***
 
 //int WaitDelay,WaitNCDelay;
-unsigned SPPDataPort;
-unsigned SPPStatPort;
-unsigned SPPCtrlPort;
-unsigned EPPAddrPort;
-unsigned EPPDataPort;
-unsigned ECPRegECR;
+unsigned short SPPDataPort;
+unsigned short SPPStatPort;
+unsigned short SPPCtrlPort;
+unsigned short EPPAddrPort;
+unsigned short EPPDataPort;
 
 // prototypes
 void WriteFlash (int addr, int data);
@@ -284,7 +291,7 @@ usage (char *name)
 
 
 void
-InitPort (int port)
+InitPort (unsigned short port)
 {
   // uCON64 comment: see the comment in fal_main() where port is initialised
   SPPDataPort = port;
@@ -292,7 +299,6 @@ InitPort (int port)
   SPPCtrlPort = SPPDataPort + 2;
   EPPAddrPort = SPPDataPort + 3;
   EPPDataPort = SPPDataPort + 4;
-  ECPRegECR = SPPDataPort + 0x402;
 }
 
 
@@ -502,9 +508,7 @@ ReadFlash (int addr)
 void
 l402684 (void)
 {
-#ifndef USE_PPDEV
-  outpb (SPPStatPort, 1);                       // clear EPP time flag
-#endif
+  parport_reset_timeout (SPPDataPort);
   l40226c ();
 }
 
@@ -541,26 +545,8 @@ LinkerInit (void)               // 4027c4
 {
   int linker_found = 0;
 
-  /*
-    uCON64 comment:
-    Accessing I/O ports with addresses higher than 0x3ff causes an access
-    violation under Windows XP (NT/2000) for _Windows_ executables without the
-    use of an appropriate I/O port driver. UserPort is an example of an
-    *inappropriate* I/O port driver, because it enables access to I/O ports up
-    to 0x3ff. For some (ridiculous) reason, DOS executables are allowed to
-    _access_ at least the ECP register this code uses. That doesn't mean it
-    will result in the expected behaviour like enabling EPP.
-  */
   if (EPPMode)
     {
-      /*
-        Writing to the ECP register seems to have no effect on my PC (which
-        supports ECP, used appropriate BIOS setting). Tested under Windows XP
-        with Windows executables (Cygwin, VC++ and MinGW) - dbjh
-      */
-#ifndef USE_PPDEV
-      outpb (ECPRegECR, 4);                     // set EPP mode for ECP chipsets
-#endif
       if (LookForLinker ())
         {
           // Linker found using EPP mode.
@@ -575,10 +561,7 @@ LinkerInit (void)               // 4027c4
   if (!linker_found)
     {
       // Look for linker in SPP mode.
-#ifndef USE_PPDEV
-      if (EPPMode)
-        outpb (ECPRegECR, 0);                   // set SPP mode for ECP chipsets
-#endif
+      ucon64.parport_mode = parport_setup (SPPDataPort, PPMODE_SPP);
 
       EPPMode = 0;
       if (LookForLinker ())
@@ -599,7 +582,7 @@ ReadStatusRegister (int addr)   // 402dd8
   int v;
   WriteFlash (addr, INTEL28F_READSR);
   outpb (SPPCtrlPort, 0);
-  v = PPReadWord ();            // & 0xff;
+  PPReadWord ();
   v = PPReadWord ();            // & 0xff;
   return v;
 }
@@ -659,8 +642,8 @@ RestoreSRAM (FILE *fp, int StartOS)
   int byteswritten = 0;
   time_t starttime;
 
-  printf ("Send: %d Bytes (%.4f Mb)\n\n", ucon64.file_size,
-          (float) ucon64.file_size / MBIT);
+  printf ("Send: %u Bytes (%.4f Mb)\n\n", (unsigned int) ucon64.fsize,
+          (float) ucon64.fsize / MBIT);
 
   starttime = time (NULL);
   i = fgetc (fp);
@@ -688,7 +671,7 @@ RestoreSRAM (FILE *fp, int StartOS)
             }
           byteswritten += 256;
           if ((byteswritten & 0x1fff) == 0)     // call ucon64_gauge() after sending 8 kB
-            ucon64_gauge (starttime, byteswritten, ucon64.file_size);
+            ucon64_gauge (starttime, byteswritten, (int) ucon64.fsize);
         }
       m++;
     }
@@ -704,7 +687,7 @@ BackupROM (FILE *fp, int SizekW)
   time_t starttime;
 
   printf ("Receive: %d Bytes (%.4f Mb)\n\n", size, (float) size / MBIT);
-  
+
   WriteFlash (0, INTEL28F_READARRAY);   // Set flash (intel 28F640J3A) Read Mode
 
   starttime = time (NULL);
@@ -717,7 +700,7 @@ BackupROM (FILE *fp, int SizekW)
 
       for (j = 0; j < 256; j++)
         {
-          valw = PPReadWord ();
+          valw = (unsigned short) PPReadWord ();
           fputc (valw & 0xff, fp);
           fputc (valw >> 8, fp);
         }
@@ -731,11 +714,9 @@ BackupROM (FILE *fp, int SizekW)
 void
 dump (u8 BaseAdr)
 {
-//   unsigned char low, high;
+//  unsigned char low, high;
   int i;
   u8 First = 1;
-  u16 v;
-  u8 val1, val2;
   u8 Display[17] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 
   WriteFlash (0, INTEL28F_READARRAY);   // Set flash (intel 28F640J3A) Read Mode
@@ -747,17 +728,19 @@ dump (u8 BaseAdr)
 
   for (i = 0; i < 128; i++)
     {
+      u16 v;
+      u8 val1, val2;
+
       if (First == 1)
         {
-          if (i * 2 < 256)
-            fputc ('0', stdout);
+          fputc ('0', stdout);
           if (i * 2 < 16)
             fputc ('0', stdout);
-          printf ("%hx - ", (i * 2));
+          printf ("%hx - ", (unsigned short) (i * 2));
           First = 0;
         }
 
-      v = PPReadWord ();
+      v = (unsigned short) PPReadWord ();
       val2 = v >> 8;
       val1 = v & 255;
 
@@ -768,7 +751,7 @@ dump (u8 BaseAdr)
 
       if (val1 < 16)
         fputc ('0', stdout);
-      printf ("%hx ", val1);
+      printf ("%hx ", (unsigned short) val1);
 
       if ((val2 > 31) & (val2 < 127))
 
@@ -778,7 +761,7 @@ dump (u8 BaseAdr)
 
       if (val2 < 16)
         fputc ('0', stdout);
-      printf ("%hx ", val2);
+      printf ("%hx ", (unsigned short) val2);
 
       if ((i & 7) == 7)
         {
@@ -883,19 +866,17 @@ int
 GetFileSize2 (FILE *fp)
 /*
   The name "GetFileSize" conflicts with a Windows function.
-  For some odd reason Jeff Frohwein returned the file size minus 1. I (dbjh)
-  guess that's a bug. I disabled the (terribly inefficient) file size code
-  because uCON64 already determined the file size at this point.
+  For some odd reason Jeff Frohwein returned the file size minus 1. I guess
+  that's a bug. I disabled the (terribly inefficient) file size code, because
+  uCON64 already determined the file size at this point. - dbjh
 */
 {
-  int FileSize;
-
   if (RepairHeader)
     {
       int i;
       int j;
+      int FileSize = 0;
 
-      FileSize = 0;
       while (!feof (fp) && (FileSize < 0xc0))
         FileHeader[FileSize++] = fgetc (fp);
 
@@ -931,7 +912,7 @@ GetFileSize2 (FILE *fp)
   else
     rewind (fp);
 
-  return ucon64.file_size;
+  return (int) ucon64.fsize;
 }
 
 
@@ -940,15 +921,9 @@ GetFileSize2 (FILE *fp)
 void
 ProgramNonTurboIntelFlash (FILE *fp)
 {
-  int i, j, k;
-  int addr = 0;
-  int FileSize;
   int Ready = 0;
-  int Timeout;
-  time_t starttime;
-
   // Get file size
-  FileSize = GetFileSize2 (fp);
+  int FileSize = GetFileSize2 (fp);
 
   puts ("Erasing Visoly non-turbo flash card...");
 
@@ -958,16 +933,20 @@ ProgramNonTurboIntelFlash (FILE *fp)
   clear_line ();                                // remove "erase gauge"
   if (Ready)
     {
+      int j;
+      int addr = 0;
+      time_t starttime = time (NULL);
+
       printf ("Send: %d Bytes (%.4f Mb)\n\n", FileSize, (float) FileSize / MBIT);
 
       //403018
-      starttime = time (NULL);
       j = GetFileByte (fp);
 
       while (!feof (fp))
         {
+          int Timeout = 0x4000;
+
           Ready = 0;
-          Timeout = 0x4000;
 
           while ((Ready == 0) && (Timeout != 0))
             {
@@ -980,6 +959,8 @@ ProgramNonTurboIntelFlash (FILE *fp)
 
           if (Ready)
             {
+              int i;
+
               WriteFlash (addr, 15);    // Write 15+1 16bit words
 
               SetCartAddr (addr);       // Set cart base addr to 0
@@ -987,10 +968,11 @@ ProgramNonTurboIntelFlash (FILE *fp)
 
               for (i = 0; i < 16; i++)
                 {
-                  k = j;
+                  int k = j;
+
                   if (j != EOF)
                     j = GetFileByte (fp);
-                  k += (j << 8);
+                  k += ((j & 0xff) << 8);
                   PPWriteWord (k);
 
                   if (j != EOF)
@@ -1039,13 +1021,11 @@ ProgramNonTurboIntelFlash (FILE *fp)
         }
 
       clear_line ();                            // remove last gauge
-      ucon64_gauge (starttime, addr << 1, FileSize);   // make gauge reach 100% when size % 32 k != 0
+      ucon64_gauge (starttime, addr << 1, FileSize); // make gauge reach 100% when size % 32 k != 0
       WriteFlash (0, INTEL28F_READARRAY);
       outpb (SPPCtrlPort, 0);
 
-      if (Ready)
-        ;
-      else
+      if (!Ready)
         fputs ("\nERROR: Flash card write failed\n", stderr);
     }
   else
@@ -1058,17 +1038,9 @@ ProgramNonTurboIntelFlash (FILE *fp)
 void
 ProgramTurboIntelFlash (FILE *fp)
 {
-  int i, j = 0;
-  int k;                        //z;
-  int addr = 0;
-  int done1, done2;
-  int FileSize;
-  int Timeout;
   int Ready;                    //= 0;
-  time_t starttime;
-
   // Get file size
-  FileSize = GetFileSize2 (fp);
+  int FileSize = GetFileSize2 (fp);
 
   puts ("Erasing Visoly turbo flash card...");
 
@@ -1078,18 +1050,22 @@ ProgramTurboIntelFlash (FILE *fp)
   clear_line ();                                // remove "erase gauge"
   if (Ready)
     {
+      int j = 0;
+      int addr = 0;
+      time_t starttime = time (NULL);
+
       printf ("Send: %d Bytes (%.4f Mb)\n\n", FileSize, (float) FileSize / MBIT);
 
       //403018
-      starttime = time (NULL);
       j = GetFileByte (fp);
 
       while (!feof (fp))
         {
-          done1 = 0;
-          done2 = 0;
+          int done1 = 0;
+          int done2 = 0;
+          int Timeout = 0x4000;
+
           Ready = 0;
-          Timeout = 0x4000;
 
           while ((!Ready) && (Timeout != 0))
             {
@@ -1111,6 +1087,9 @@ ProgramTurboIntelFlash (FILE *fp)
 
           if (Ready)
             {
+              int i;
+              int k;                    //z;
+
               WriteFlash (addr, 15);    // Write 15+1 16bit words
               PPWriteWord (15);
 
@@ -1122,7 +1101,7 @@ ProgramTurboIntelFlash (FILE *fp)
                   k = j;
                   if (j != EOF)
                     j = GetFileByte (fp);
-                  k += (j << 8);
+                  k += ((j & 0xff) << 8);
                   PPWriteWord (k);
 
                   if (j != EOF)
@@ -1156,15 +1135,13 @@ ProgramTurboIntelFlash (FILE *fp)
         }
 
       clear_line ();                            // remove last gauge
-      ucon64_gauge (starttime, addr << 1, FileSize);   // make gauge reach 100% when size % 32 k != 0
+      ucon64_gauge (starttime, addr << 1, FileSize); // make gauge reach 100% when size % 32 k != 0
       WriteFlash (0, INTEL28F_READARRAY);
       outpb (SPPCtrlPort, 0);
       WriteFlash (1, INTEL28F_READARRAY);
       outpb (SPPCtrlPort, 0);
 
-      if (Ready)
-        ;
-      else
+      if (!Ready)
         {
           WriteFlash (0, INTEL28F_CLEARSR);
           PPWriteWord (INTEL28F_CLEARSR);
@@ -1180,15 +1157,9 @@ ProgramTurboIntelFlash (FILE *fp)
 void
 ProgramSharpFlash (FILE *fp)
 {
-  int i, j;
-  int k = 0;
-  int addr = 0;
-  int FileSize;
   int Ready;
-  time_t starttime;
-
   // Get file size
-  FileSize = GetFileSize2 (fp);
+  int FileSize = GetFileSize2 (fp);
 
   puts ("Erasing Nintendo flash card...");
 
@@ -1198,13 +1169,19 @@ ProgramSharpFlash (FILE *fp)
   clear_line ();                                // remove "erase gauge"
   if (Ready)
     {
+      int j;
+      int addr = 0;
+      time_t starttime = time (NULL);
+
       printf ("Send: %d Bytes (%.4f Mb)\n\n", FileSize, (float) FileSize / MBIT);
 
-      starttime = time (NULL);
       j = GetFileByte (fp);
 
       while (!feof (fp))
         {
+          int i;
+          int k = 0;
+
           if (j != EOF)
             k = GetFileByte (fp);
 
@@ -1223,7 +1200,7 @@ ProgramSharpFlash (FILE *fp)
         }
 
       clear_line ();                            // remove last gauge
-      ucon64_gauge (starttime, addr << 1, FileSize);   // make gauge reach 100% when size % 32 k != 0
+      ucon64_gauge (starttime, addr << 1, FileSize); // make gauge reach 100% when size % 32 k != 0
       WriteFlash (0, INTEL28F_READARRAY);
       outpb (SPPCtrlPort, 0);
     }
@@ -1302,18 +1279,18 @@ SpaceCheck (char c)
 int
 fal_main (int argc, char **argv)
 {
-  int arg, i;
+  int arg;
   u8 Base = 0;
   FILE *fp;
-  char fname[128], fname2[128];
+  char fname[128];//, fname2[128];
   int OptB = 0;
   int OptD = 0;
   int OptP = 0;
   int OptR = 0;
   int OptS = 0;
-  int OptV = 0;
-  int OptZ = 0;
-  int port = 0x378;
+//  int OptV = 0;
+//  int OptZ = 0;
+  unsigned short port = 0x378;
   int ChipSize = 32;
   int BackupMemOffset = 0;
   int BackupMemSize = 0;
@@ -1326,7 +1303,7 @@ fal_main (int argc, char **argv)
 
   debug = 0;
   verbose = 1;
-  EPPMode = 0;                                  // uCON64 comment: use the most compatible setting as default
+  EPPMode = 1;
   DataSize16 = 0;
   WaitDelay = 0;
   VisolyTurbo = 0;
@@ -1359,8 +1336,7 @@ fal_main (int argc, char **argv)
           OptB = 1;
           break;
         case '2':
-          // 16-bit EPP support enable (doesn't work with
-          //  "Turbo FA 128M (2 x i28F640J3A)" - dbjh)
+          // 16bit EPP support enable
           DataSize16 = 1;
           break;
         case 'c':
@@ -1381,7 +1357,7 @@ fal_main (int argc, char **argv)
           SpaceCheck (argv[arg][2]);
           if (argv[++arg] != NULL)
             Base = (u8) (atoi (argv[arg]));
-          printf ("Base address: %hx\n", Base * 256);
+          printf ("Base address: %hx\n", (unsigned short) (Base * 256));
           OptD = 1;
           break;
         case 's':
@@ -1408,26 +1384,25 @@ fal_main (int argc, char **argv)
           strcpy (fname, argv[++arg]);
           OptR = 1;
           break;
+#if 0
         case 'v':
           // Verify flash cart
           SpaceCheck (argv[arg][2]);
           strcpy (fname2, argv[++arg]);
           OptV = 1;
           break;
+#endif
         case 'l':
           SpaceCheck (argv[arg][2]);
-          i = atoi (argv[++arg]);
           /*
             uCON64 comment: we want to support non-standard parallel port
             addresses too. So, instead of passing a number from 1 to 4, we pass
             the address itself
           */
-          port = i;
+          port = (unsigned short) atoi (argv[++arg]);;
           break;
         case 'm':
-          // uCON64 comment: See comment in LinkerInit(). Note that we reverse
-          //  the meaning compared to the original code.
-          EPPMode = 1;                          // Set EPP mode
+          EPPMode = 0;                          // Set SPP mode
           break;
         case 'n':
           // Don't repair header
@@ -1437,9 +1412,11 @@ fal_main (int argc, char **argv)
           SpaceCheck (argv[arg][2]);
           WaitDelay = atoi (argv[++arg]);
           break;
+#if 0
         case 'z':
           OptZ = 1;
           break;
+#endif
         default:
 //          usage (argv[0]);
           ProgramExit (1);
@@ -1458,10 +1435,11 @@ fal_main (int argc, char **argv)
           fprintf (stderr, ucon64_msg[OPEN_WRITE_ERROR], fname);
           ProgramExit (1);
         }
-
-      BackupSRAM (fp, BackupMemOffset, BackupMemSize);
-      fputc ('\n', stdout);
-      fclose (fp);
+      else
+        {
+          BackupSRAM (fp, BackupMemOffset, BackupMemSize);
+          fclose (fp);
+        }
     }
 
   if (OptD)
@@ -1480,10 +1458,11 @@ fal_main (int argc, char **argv)
           fprintf (stderr, ucon64_msg[OPEN_READ_ERROR], fname);
           ProgramExit (1);
         }
-
-      RestoreSRAM (fp, BackupMemOffset);
-      fputc ('\n', stdout);
-      fclose (fp);
+      else
+        {
+          RestoreSRAM (fp, BackupMemOffset);
+          fclose (fp);
+        }
     }
 
   if (OptP)
@@ -1493,18 +1472,19 @@ fal_main (int argc, char **argv)
           fprintf (stderr, ucon64_msg[OPEN_READ_ERROR], fname);
           ProgramExit (1);
         }
-
-      if (Device == 0xe2)
-        ProgramSharpFlash (fp);
       else
         {
-          if (VisolyTurbo)
-            ProgramTurboIntelFlash (fp);
+          if (Device == 0xe2)
+            ProgramSharpFlash (fp);
           else
-            ProgramNonTurboIntelFlash (fp);
+            {
+              if (VisolyTurbo)
+                ProgramTurboIntelFlash (fp);
+              else
+                ProgramNonTurboIntelFlash (fp);
+            }
+          fclose (fp);
         }
-      fputc ('\n', stdout);
-      fclose (fp);
     }
 
   if (OptS)
@@ -1514,10 +1494,11 @@ fal_main (int argc, char **argv)
           fprintf (stderr, ucon64_msg[OPEN_WRITE_ERROR], fname);
           ProgramExit (1);
         }
-
-      BackupROM (fp, ChipSize << 16);
-      fputc ('\n', stdout);
-      fclose (fp);
+      else
+        {
+          BackupROM (fp, ChipSize << 16);
+          fclose (fp);
+        }
     }
 
 #if 0
@@ -1528,14 +1509,16 @@ fal_main (int argc, char **argv)
           fprintf (stderr, ucon64_msg[OPEN_READ_ERROR], fname2);
           ProgramExit (1);
         }
-
-      VerifyFlash (fp);
-      fclose (fp);
+      else
+        {
+          VerifyFlash (fp);
+          fclose (fp);
+        }
     }
 #endif
 
-  ProgramExit (0);
-  exit (0);
+//  ProgramExit (0);
+  return 0;
 }
 
 
@@ -1543,22 +1526,23 @@ fal_main (int argc, char **argv)
   It will save you some work if you don't fully integrate the code above with
   uCON64's code, because it is a project separate from the uCON64 project.
 */
-int fal_argc = 0;
-char *fal_argv[128];
+static int fal_argc = 0;
+static char *fal_argv[128];
 
-void
+static void
 fal_args (unsigned int parport)
 {
-  char parport_str[80];
+  static char parport_str[80];
 
   parport_print_info ();
 
+  fal_argc = 0;
   fal_argv[fal_argc++] = "fl";
   fal_argv[fal_argc++] = "-l";
-  sprintf (parport_str, "%d", parport); // don't use %x, as Jeff Frohwein uses atoi()
+  sprintf (parport_str, "%u", parport); // don't use %x, as Jeff Frohwein uses atoi()
   fal_argv[fal_argc++] = parport_str;
 
-  if (ucon64.parport_mode == UCON64_EPP)
+  if (ucon64.parport_mode != PPMODE_EPP)
     fal_argv[fal_argc++] = "-m";
 }
 
@@ -1625,8 +1609,8 @@ fal_read_sram (const char *filename, unsigned int parport, int bank)
           fputs ("ERROR: Bank must be 1, 2, 3 or 4\n", stderr);
           exit (1);
         }
-      bank_str[0] = '0' + bank;
-      bank_str[1] = 0;                          // terminate string
+      bank_str[0] = (char) ('0' + bank);
+      bank_str[1] = '\0';                       // terminate string
       fal_argv[fal_argc++] = bank_str;
       fal_argv[fal_argc++] = "2";               // 64 kB
     }
@@ -1656,8 +1640,8 @@ fal_write_sram (const char *filename, unsigned int parport, int bank)
           fputs ("ERROR: Bank must be 1, 2, 3 or 4\n", stderr);
           exit (1);
         }
-      bank_str[0] = '0' + bank;
-      bank_str[1] = 0;                          // terminate string
+      bank_str[0] = (char) ('0' + bank);
+      bank_str[1] = '\0';                       // terminate string
       fal_argv[fal_argc++] = bank_str;
     }
   fal_argv[fal_argc++] = (char *) filename;
