@@ -1,9 +1,9 @@
 /*
 fal.h - Flash Linker Advance support for uCON64
 
-Copyright (c) 2001        Jeff Frohwein
-Copyright (c) 2001        NoisyB <noisyb@gmx.net>
-Copyright (c) 2001 - 2002 dbjh
+Copyright (c) 2001              Jeff Frohwein
+Copyright (c) 2001              NoisyB
+Copyright (c) 2001 - 2002, 2020 dbjh
 
 
 This program is free software; you can redistribute it and/or modify
@@ -23,13 +23,20 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef FAL_H
 #define FAL_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t fal_usage[];
 
-#ifdef USE_PARALLEL
+#ifdef  USE_PARALLEL
 extern int fal_read_rom (const char *filename, unsigned int parport, int size);
 extern int fal_write_rom (const char *filename, unsigned int parport);
 extern int fal_read_sram (const char *filename, unsigned int parport, int bank);
-extern int fal_write_sram (const char *filename, unsigned int parport, int bank);
+extern int fal_write_sram (const char *filename, unsigned int parport,
+                           int bank);
 #endif
 
 #endif
