@@ -1,9 +1,9 @@
 /*
 fig.c - Super PRO Fighter support for uCON64
 
-Copyright (c) 1999 - 2002 NoisyB <noisyb@gmx.net>
-Copyright (c) 2001 - 2004 dbjh
-Copyright (c) 2003 - 2004 JohnDie
+Copyright (c) 1999 - 2002                          NoisyB
+Copyright (c) 2001 - 2004, 2015, 2017, 2019 - 2021 dbjh
+Copyright (c) 2003 - 2004                          JohnDie
 
 
 This program is free software; you can redistribute it and/or modify
@@ -23,25 +23,30 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifdef  HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <string.h>
-#include "misc/misc.h"
-#include "misc/itypes.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
 #endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "misc/parallel.h"
+#include <stdlib.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
+#include <string.h>
+#include "misc/archive.h"
 #include "misc/file.h"
-#include "ucon64.h"
 #include "ucon64_misc.h"
-#include "ffe.h"
-#include "fig.h"
 #include "console/snes.h"                       // for snes_get_snes_hirom()
+#include "backup/ffe.h"
 
 
+#ifdef  USE_PARALLEL
+static st_ucon64_obj_t fig_obj[] =
+  {
+    {UCON64_SNES, WF_DEFAULT | WF_STOP | WF_NO_SPLIT | WF_NO_ROM},
+    {UCON64_SNES, WF_STOP | WF_NO_ROM}
+  };
+#endif
+
 const st_getopt2_t fig_usage[] =
   {
     {
@@ -53,57 +58,56 @@ const st_getopt2_t fig_usage[] =
 #ifdef  USE_PARALLEL
     {
       "xfig", 0, 0, UCON64_XFIG,
-      NULL, "send/receive ROM to/from *Pro Fighter*/FIG; " OPTION_LONG_S "port=PORT\n"
+      NULL, "send/receive ROM to/from *Pro Fighter*/FIG; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when ROM does not exist",
-      &ucon64_wf[WF_OBJ_SNES_DEFAULT_STOP_NO_SPLIT_NO_ROM]
+      &fig_obj[0]
     },
     {
       "xfigs", 0, 0, UCON64_XFIGS,
-      NULL, "send/receive SRAM to/from *Pro Fighter*/FIG; " OPTION_LONG_S "port=PORT\n"
+      NULL, "send/receive SRAM to/from *Pro Fighter*/FIG; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when SRAM does not exist",
-      &ucon64_wf[WF_OBJ_SNES_STOP_NO_ROM]
+      &fig_obj[1]
     },
     {
       "xfigc", 0, 0, UCON64_XFIGC, NULL,
-      "send/receive SRAM to/from cartridge in *Pro Fighter*/FIG;\n" OPTION_LONG_S "port=PORT\n"
+      "send/receive SRAM to/from cartridge in *Pro Fighter*/FIG;\n" OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when SRAM does not exist",
 //      "Press q to abort; ^C might cause invalid state of backup unit"
-      &ucon64_wf[WF_OBJ_SNES_STOP_NO_ROM]
+      &fig_obj[1]
     },
 #endif
     {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
 
 
-#ifdef USE_PARALLEL
+#ifdef  USE_PARALLEL
 
 #define BUFFERSIZE 8192                         // don't change, only 8192 works!
 
-
-static int receive_rom_info (unsigned char *buffer);
-static int get_rom_size (unsigned char *info_block);
+static unsigned int receive_rom_info (unsigned char *buffer);
+static unsigned char get_rom_size (unsigned char *info_block);
 static int check1 (unsigned char *info_block, int index);
 static int check2 (unsigned char *info_block, int index, unsigned char value);
 static int check3 (unsigned char *info_block, int index1, int index2, int size);
 static void handle_swc_header (unsigned char *header);
 
-static int hirom;
+static unsigned int hirom;
 
 
-#if BUFFERSIZE < 512
+#if     BUFFERSIZE < 512
 #error receive_rom_info() and fig_read_sram() expect BUFFERSIZE to be at least \
        512 bytes.
 #endif
-int
+static unsigned int
 receive_rom_info (unsigned char *buffer)
 /*
   - returns size of ROM in Mb (128 kB) units
-  - sets global `hirom'
+  - sets global 'hirom'
 */
 {
-  int n, size;
+  unsigned short n;
   volatile int m;
-  unsigned char byte;
+  unsigned char size;
 
   ffe_send_command0 (0xe00c, 0);
 
@@ -111,17 +115,19 @@ receive_rom_info (unsigned char *buffer)
     hirom = ucon64.snes_hirom ? 1 : 0;
   else
     {
+      unsigned char byte;
+
       ffe_send_command (5, 3, 0);
       byte = ffe_send_command1 (0xbfd5);
       if ((byte & 1 && byte != 0x23) || byte == 0x3a) // & 1 => 0x21, 0x31, 0x35
         hirom = 1;
     }
 
-  for (n = 0; n < (int) FIG_HEADER_LEN; n++)
+  for (n = 0; n < FIG_HEADER_LEN; n++)
     {
       for (m = 0; m < 65536; m++)               // a delay is necessary here
         ;
-      ffe_send_command (5, (unsigned short) (0x200 + n), 0);
+      ffe_send_command (5, 0x200 + n, 0);
       buffer[n] = ffe_send_command1 (0xa0a0);
     }
 
@@ -133,7 +139,7 @@ receive_rom_info (unsigned char *buffer)
 }
 
 
-int
+static unsigned char
 get_rom_size (unsigned char *info_block)
 // returns size of ROM in Mb units
 {
@@ -186,7 +192,7 @@ get_rom_size (unsigned char *info_block)
 }
 
 
-int
+static int
 check1 (unsigned char *info_block, int index)
 {
   int n;
@@ -199,7 +205,7 @@ check1 (unsigned char *info_block, int index)
 }
 
 
-int
+static int
 check2 (unsigned char *info_block, int index, unsigned char value)
 {
   int n;
@@ -212,7 +218,7 @@ check2 (unsigned char *info_block, int index, unsigned char value)
 }
 
 
-int
+static int
 check3 (unsigned char *info_block, int index1, int index2, int size)
 {
   int n;
@@ -225,7 +231,7 @@ check3 (unsigned char *info_block, int index1, int index2, int size)
 }
 
 
-void
+static void
 handle_swc_header (unsigned char *header)
 {
   if ((header[2] & 0x10) == 0x10)
@@ -274,14 +280,14 @@ handle_swc_header (unsigned char *header)
 
 
 int
-fig_read_rom (const char *filename, unsigned int parport)
+fig_read_rom (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer;
   int n, size, blocksleft, bytesreceived = 0;
   unsigned short address1, address2;
   time_t starttime;
-  st_rominfo_t rominfo;
+  st_ucon64_nfo_t rominfo;
 
   ffe_init_io (parport);
 
@@ -299,13 +305,13 @@ fig_read_rom (const char *filename, unsigned int parport)
   size = receive_rom_info (buffer);
   if (size == 0)
     {
-      fprintf (stderr, "ERROR: There is no cartridge present in the Pro Fighter\n");
+      fputs ("ERROR: There is no cartridge present in the Pro Fighter\n", stderr);
       fclose (file);
       remove (filename);
       exit (1);
     }
   blocksleft = size * 16;                       // 1 Mb (128 kB) unit == 16 8 kB units
-  printf ("Receive: %d Bytes (%.4f Mb)\n", size * MBIT, (float) size);
+  printf ("Receive: %u Bytes (%.4f Mb)\n", size * MBIT, (float) size);
   size *= MBIT;                                 // size in bytes for ucon64_gauge() below
 
   ffe_send_command (5, 0, 0);
@@ -319,7 +325,7 @@ fig_read_rom (const char *filename, unsigned int parport)
   if (hirom)
     blocksleft >>= 1;
 
-  printf ("Press q to abort\n\n");              // print here, NOT before first FIG I/O,
+  puts ("Press q to abort\n");                  // print here, NOT before first FIG I/O,
                                                 //  because if we get here q works ;-)
   address1 = 0x300;                             // address1 = 0x100, address2 = 0 should
   address2 = 0x200;                             //  also work
@@ -356,38 +362,36 @@ fig_read_rom (const char *filename, unsigned int parport)
 
   // Create a correct header. We can't obtain the header from the Pro Fighter
   //  unless a (the same) cartridge was just dumped to diskette...
-  ucon64.rom = filename;
-  ucon64.file_size = size + FIG_HEADER_LEN;
+  ucon64.fname = filename;
+  ucon64.fsize = size + FIG_HEADER_LEN;
   // override everything we know for sure
   ucon64.console = UCON64_SNES;
-  ucon64.buheader_len = FIG_HEADER_LEN;
+  ucon64.backup_header_len = FIG_HEADER_LEN;
   ucon64.split = 0;
   ucon64.snes_hirom = hirom ? SNES_HIROM : 0;
   ucon64.interleaved = 0;
-  memset (&rominfo, 0, sizeof (st_rominfo_t));
+  memset (&rominfo, 0, sizeof (st_ucon64_nfo_t));
 
   fflush (file);
   snes_init (&rominfo);
-  memset (buffer, 0, FIG_HEADER_LEN);
-  snes_set_fig_header (&rominfo, (st_fig_header_t *) buffer);
+  snes_set_fig_header ((st_fig_header_t *) buffer, rominfo.backup_header_len);
   fseek (file, 0, SEEK_SET);
   fwrite (buffer, 1, FIG_HEADER_LEN, file);     // write correct header
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-fig_write_rom (const char *filename, unsigned int parport)
+fig_write_rom (const char *filename, unsigned short parport)
 {
   FILE *file;
-  unsigned char *buffer;
-  int bytesread = 0, bytessend, totalblocks, blocksdone = 0, blocksleft, fsize,
-      n, emu_mode_select;
+  unsigned char *buffer, emu_mode_select;
+  size_t bytesread = 0, bytessent;
+  unsigned int blocksdone = 0, blocksleft, fsize, n;
   unsigned short address1, address2;
   time_t starttime;
 
@@ -404,28 +408,27 @@ fig_write_rom (const char *filename, unsigned int parport)
       exit (1);
     }
 
-  fsize = fsizeof (filename);
-  printf ("Send: %d Bytes (%.4f Mb)\n", fsize, (float) fsize / MBIT);
+  fsize = (unsigned int) ucon64.fsize;
+  printf ("Send: %u Bytes (%.4f Mb)\n", fsize, (float) fsize / MBIT);
 
   ffe_send_command0 (0xc008, 0);
-  fread (buffer, 1, FIG_HEADER_LEN, file);
+  fread_checked (buffer, 1, FIG_HEADER_LEN, file);
 
-  if (snes_get_file_type () == SWC)
+  if (snes_get_copier_type () == SWC)
     handle_swc_header (buffer);
   emu_mode_select = buffer[2];                  // this byte is needed later
 
   ffe_send_command (5, 0, 0);
   ffe_send_block (0x400, buffer, FIG_HEADER_LEN); // send header
-  bytessend = FIG_HEADER_LEN;
+  bytessent = FIG_HEADER_LEN;
 
   hirom = snes_get_snes_hirom ();
   if (hirom)
     ffe_send_command0 (0xe00f, 0);              // seems to enable HiROM mode,
                                                 //  value doesn't seem to matter
-  printf ("Press q to abort\n\n");              // print here, NOT before first FIG I/O,
+  puts ("Press q to abort\n");                  // print here, NOT before first FIG I/O,
                                                 //  because if we get here q works ;-)
-  totalblocks = (fsize - FIG_HEADER_LEN + BUFFERSIZE - 1) / BUFFERSIZE; // round up
-  blocksleft = totalblocks;
+  blocksleft = (fsize - FIG_HEADER_LEN + BUFFERSIZE - 1) / BUFFERSIZE; // round up
   address1 = 0x300;
   address2 = 0x200;
   starttime = time (NULL);
@@ -435,30 +438,30 @@ fig_write_rom (const char *filename, unsigned int parport)
         for (n = 0; n < 4; n++)
           {
             bytesread = fread (buffer, 1, BUFFERSIZE, file);
-            ffe_send_command0 ((unsigned short) 0xc010, (unsigned char) (blocksdone >> 9));
+            ffe_send_command0 (0xc010, (unsigned char) (blocksdone >> 9));
             ffe_send_command (5, address1, 0);
-            ffe_send_block (0x0000, buffer, bytesread);
+            ffe_send_block (0x0000, buffer, (unsigned short) bytesread);
             address1++;
             blocksleft--;
             blocksdone++;
 
-            bytessend += bytesread;
-            ucon64_gauge (starttime, bytessend, fsize);
+            bytessent += bytesread;
+            ucon64_gauge (starttime, bytessent, fsize);
             ffe_checkabort (2);
           }
 
       for (n = 0; n < 4; n++)
         {
           bytesread = fread (buffer, 1, BUFFERSIZE, file);
-          ffe_send_command0 ((unsigned short) 0xc010, (unsigned char) (blocksdone >> 9));
+          ffe_send_command0 (0xc010, (unsigned char) (blocksdone >> 9));
           ffe_send_command (5, address2, 0);
-          ffe_send_block (0x8000, buffer, bytesread);
+          ffe_send_block (0x8000, buffer, (unsigned short) bytesread);
           address2++;
           blocksleft--;
           blocksdone++;
 
-          bytessend += bytesread;
-          ucon64_gauge (starttime, bytessend, fsize);
+          bytessent += bytesread;
+          ucon64_gauge (starttime, bytessent, fsize);
           ffe_checkabort (2);
         }
     }
@@ -467,24 +470,17 @@ fig_write_rom (const char *filename, unsigned int parport)
     ffe_send_command0 (0xc010, 2);
 
   ffe_send_command (5, 0, 0);
-  ffe_send_command (6, (unsigned short) (1 | (emu_mode_select << 8)), 0);
-
-  ffe_wait_for_ready ();
-  outportb ((unsigned short) (parport + PARPORT_DATA), 0);
-  outportb ((unsigned short) (parport + PARPORT_CONTROL),
-            (unsigned char) (inportb ((unsigned short) // invert strobe
-                                      (parport + PARPORT_CONTROL)) ^ PARPORT_STROBE));
+  ffe_send_command (6, 1 | (emu_mode_select << 8), 0);
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-fig_read_sram (const char *filename, unsigned int parport)
+fig_read_sram (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer;
@@ -507,7 +503,7 @@ fig_read_sram (const char *filename, unsigned int parport)
 
   printf ("Receive: %d Bytes\n", 32 * 1024);
   memset (buffer, 0, FIG_HEADER_LEN);
-#if 0 // Not needed for FIG, as size is always 4 blocks
+#if 0 // not needed for FIG, as size is always 4 blocks
   buffer[0] = 4;                                // 32 kB == 4*8 kB, "size_high" is already 0
 #endif
   fwrite (buffer, 1, FIG_HEADER_LEN, file);
@@ -516,7 +512,7 @@ fig_read_sram (const char *filename, unsigned int parport)
   ffe_send_command0 (0xe00d, 0);
   ffe_send_command0 (0xc008, 0);
 
-  printf ("Press q to abort\n\n");              // print here, NOT before first FIG I/O,
+  puts ("Press q to abort\n");                  // print here, NOT before first FIG I/O,
                                                 //  because if we get here q works ;-)
   blocksleft = 4;                               // SRAM is 4*8 kB
   address = 0x100;
@@ -536,18 +532,18 @@ fig_read_sram (const char *filename, unsigned int parport)
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-fig_write_sram (const char *filename, unsigned int parport)
+fig_write_sram (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer;
-  int bytesread, bytessend = 0, size;
+  size_t bytesread, bytessent = 0;
+  unsigned int size;
   unsigned short address;
   time_t starttime;
 
@@ -564,39 +560,38 @@ fig_write_sram (const char *filename, unsigned int parport)
       exit (1);
     }
 
-  size = fsizeof (filename) - FIG_HEADER_LEN;   // FIG SRAM is 4*8 kB, emu SRAM often not
-  printf ("Send: %d Bytes\n", size);
+  size = (unsigned int) ucon64.fsize - FIG_HEADER_LEN; // FIG SRAM is 4*8 kB, emu SRAM often not
+  printf ("Send: %u Bytes\n", size);
   fseek (file, FIG_HEADER_LEN, SEEK_SET);       // skip the header
 
   ffe_send_command (5, 0, 0);
   ffe_send_command0 (0xe00d, 0);
   ffe_send_command0 (0xc008, 0);
 
-  printf ("Press q to abort\n\n");              // print here, NOT before first FIG I/O,
+  puts ("Press q to abort\n");                  // print here, NOT before first FIG I/O,
                                                 //  because if we get here q works ;-)
   address = 0x100;
   starttime = time (NULL);
-  while ((bytesread = fread (buffer, 1, BUFFERSIZE, file)))
+  while ((bytesread = fread (buffer, 1, BUFFERSIZE, file)) != 0)
     {
       ffe_send_command (5, address, 0);
-      ffe_send_block (0x2000, buffer, bytesread);
+      ffe_send_block (0x2000, buffer, (unsigned short) bytesread);
       address++;
 
-      bytessend += bytesread;
-      ucon64_gauge (starttime, bytessend, size);
+      bytessent += bytesread;
+      ucon64_gauge (starttime, bytessent, size);
       ffe_checkabort (2);
     }
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-fig_read_cart_sram (const char *filename, unsigned int parport)
+fig_read_cart_sram (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer, byte;
@@ -620,7 +615,7 @@ fig_read_cart_sram (const char *filename, unsigned int parport)
   size = receive_rom_info (buffer);
   if (size == 0)
     {
-      fprintf (stderr, "ERROR: There is no cartridge present in the Pro Fighter\n");
+      fputs ("ERROR: There is no cartridge present in the Pro Fighter\n", stderr);
       fclose (file);
       remove (filename);
       exit (1);
@@ -643,7 +638,7 @@ fig_read_cart_sram (const char *filename, unsigned int parport)
   ffe_send_command0 (0xe00c, 0);
 //  ffe_send_command0 (0xc008, 0);
 
-  printf ("Press q to abort\n\n");              // print here, NOT before first FIG I/O,
+  puts ("Press q to abort\n");                  // print here, NOT before first FIG I/O,
                                                 //  because if we get here q works ;-)
   address = hirom ? 0x2c3 : 0x1c0;
 
@@ -651,7 +646,7 @@ fig_read_cart_sram (const char *filename, unsigned int parport)
   while (bytesreceived < size)
     {
       ffe_send_command (5, address, 0);
-      ffe_receive_block ((unsigned short) (hirom ? 0x6000 : 0x2000), buffer, BUFFERSIZE);
+      ffe_receive_block (hirom ? 0x6000 : 0x2000, buffer, BUFFERSIZE);
       fwrite (buffer, 1, BUFFERSIZE, file);
       address += hirom ? 4 : 1;
 
@@ -662,18 +657,18 @@ fig_read_cart_sram (const char *filename, unsigned int parport)
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-fig_write_cart_sram (const char *filename, unsigned int parport)
+fig_write_cart_sram (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer, byte;
-  int bytesread, bytessend = 0, size;
+  size_t bytesread, bytessent = 0;
+  unsigned int size;
   unsigned short address;
   time_t starttime;
 
@@ -693,7 +688,7 @@ fig_write_cart_sram (const char *filename, unsigned int parport)
   size = receive_rom_info (buffer);
   if (size == 0)
     {
-      fprintf (stderr, "ERROR: There is no cartridge present in the Pro Fighter\n");
+      fputs ("ERROR: There is no cartridge present in the Pro Fighter\n", stderr);
       fclose (file);
       remove (filename);
       exit (1);
@@ -703,35 +698,35 @@ fig_write_cart_sram (const char *filename, unsigned int parport)
   ffe_send_command0 (0xe00c, 0);                //  want to write more data than necessary
   byte = ffe_send_command1 (0xbfd8);
 
-  size = fsizeof (filename) - FIG_HEADER_LEN;   // FIG SRAM is 4*8 kB, emu SRAM often not
-  size = MIN ((byte ? 1 << (byte + 10) : 0), size);
+  size = (unsigned int) ucon64.fsize - FIG_HEADER_LEN; // FIG SRAM is 4*8 kB, emu SRAM often not
+  size = MIN ((byte ? 1U << (byte + 10) : 0), size);
 
-  printf ("Send: %d Bytes\n", size);
+  printf ("Send: %u Bytes\n", size);
   fseek (file, FIG_HEADER_LEN, SEEK_SET);       // skip the header
 
   ffe_send_command (5, 0, 0);
   ffe_send_command0 (0xe00c, 0);
 //  ffe_send_command0 (0xc008, 0);
 
-  printf ("Press q to abort\n\n");              // print here, NOT before first FIG I/O,
+  puts ("Press q to abort\n");                  // print here, NOT before first FIG I/O,
                                                 //  because if we get here q works ;-)
   address = hirom ? 0x2c3 : 0x1c0;
 
   starttime = time (NULL);
-  while ((bytessend < size) && (bytesread = fread (buffer, 1, MIN (size, BUFFERSIZE), file)))
+  while (bytessent < size &&
+         (bytesread = fread (buffer, 1, MIN (size, BUFFERSIZE), file)) != 0)
     {
       ffe_send_command (5, address, 0);
-      ffe_send_block ((unsigned short) (hirom ? 0x6000 : 0x2000), buffer, bytesread);
+      ffe_send_block (hirom ? 0x6000 : 0x2000, buffer, (unsigned short) bytesread);
       address += hirom ? 4 : 1;
 
-      bytessend += bytesread;
-      ucon64_gauge (starttime, bytessend, size);
+      bytessent += bytesread;
+      ucon64_gauge (starttime, bytessent, size);
       ffe_checkabort (2);
     }
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
