@@ -1,8 +1,8 @@
 /*
 gbx.c - Game Boy Xchanger/GBDoctor support for uCON64
 
-Copyright (c) 1999 - 2001 NoisyB <noisyb@gmx.net>
-Copyright (c) 2001 - 2004 dbjh
+Copyright (c) 1999 - 2001                          NoisyB
+Copyright (c) 2001 - 2004, 2015, 2017, 2020 - 2021 dbjh
 Based on gbt15.c - Copyright (c) Bung Enterprises
 
 
@@ -58,25 +58,34 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifdef  HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
-#include <ctype.h>
-#include "misc/misc.h"
-#include "misc/itypes.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
 #endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "misc/parallel.h"
+#include <ctype.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
+#include <stdlib.h>
+#include "misc/archive.h"
 #include "misc/file.h"
-#include "ucon64.h"
+#include "misc/misc.h"
+#include "misc/parallel.h"
+#include "misc/term.h"
 #include "ucon64_misc.h"
-#include "gbx.h"
-#include "console/gb.h"                         // gb_logodata, rocket_logodata
+#include "console/gb.h"                         // GB_NAME_LEN, gb_logodata,
+#include "backup/gbx.h"                         //  rocket_logodata
 
 
+#ifdef  USE_PARALLEL
+static st_ucon64_obj_t gbx_obj[] =
+  {
+    {UCON64_GB, WF_DEFAULT | WF_STOP | WF_NO_ROM},
+    {UCON64_GB, WF_STOP | WF_NO_ROM},
+    {UCON64_GB, WF_SWITCH}
+  };
+#endif
+
 const st_getopt2_t gbx_usage[] =
   {
     {
@@ -87,27 +96,27 @@ const st_getopt2_t gbx_usage[] =
 #ifdef  USE_PARALLEL
     {
       "xgbx", 0, 0, UCON64_XGBX,
-      NULL, "send/receive ROM to/from GB Xchanger; " OPTION_LONG_S "port=PORT\n"
+      NULL, "send/receive ROM to/from GB Xchanger; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when ROM does not exist",
-      &ucon64_wf[WF_OBJ_GB_DEFAULT_STOP_NO_ROM]
+      &gbx_obj[0]
     },
     {
       "xgbxs", 0, 0, UCON64_XGBXS,
-      NULL, "send/receive SRAM to/from GB Xchanger; " OPTION_LONG_S "port=PORT\n"
+      NULL, "send/receive SRAM to/from GB Xchanger; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when SRAM does not exist",
-      &ucon64_wf[WF_OBJ_GB_STOP_NO_ROM]
+      &gbx_obj[1]
     },
     {
       "xgbxb", 1, 0, UCON64_XGBXB,
       "BANK", "send/receive 64 kbits SRAM to/from GB Xchanger BANK\n"
-      "BANK can be a number from 0 to 15; " OPTION_LONG_S "port=PORT\n"
+      "BANK can be a number from 0 to 15; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when ROM does not exist",
-      &ucon64_wf[WF_OBJ_GB_STOP_NO_ROM]
+      &gbx_obj[1]
     },
     {
       "xgbxm", 0, 0, UCON64_XGBXM,
-      NULL, "try to enable EPP mode, default is SPP mode",
-      &ucon64_wf[WF_OBJ_GB_SWITCH]
+      NULL, "use SPP mode, default is EPP mode",
+      &gbx_obj[2]
     },
 #endif
     {NULL, 0, 0, 0, NULL, NULL, NULL}
@@ -124,17 +133,16 @@ const st_getopt2_t gbx_usage[] =
 typedef enum { UNKNOWN_MBC, BUNG, ROM, MBC1, MBC2, MBC3, MBC5, CAMERA, ROCKET } mbc_t;
 typedef enum { UNKNOWN_EEPROM, WINBOND, MX, INTEL } eeprom_t;
 
-static unsigned short int port_8, port_9, port_a, port_b, port_c, rocket_game_no;
+static unsigned short port_8, port_9, port_a, port_b, port_c, rocket_game_no;
 static unsigned char buffer[32768];
 static mbc_t mbc_type;
 static eeprom_t eeprom_type;
-static parport_mode_t port_mode;
 
 
-static volatile void
+static void
 delay_us (int n_us)
-// Hmmm, usleep() can be used under UNIX, but default under DOS is delay() which
-//  waits for a number of milliseconds... Better use the same code on all platforms.
+// Hmmm, usleep() can be used on UNIX, but default on DOS is delay() which waits
+//  for a number of milliseconds... Better use the same code on all platforms.
 {
   volatile int n;
   int n_max;
@@ -223,7 +231,7 @@ static void
 set_ai (unsigned char ai)
 {
   set_data_write
-  if (port_mode == UCON64_SPP)
+  if (ucon64.parport_mode != PPMODE_EPP)
     spp_set_ai (ai);
   else
     epp_set_ai (ai);
@@ -233,7 +241,7 @@ set_ai (unsigned char ai)
 static void
 set_ai_data (unsigned char ai, unsigned char data)
 {
-  if (port_mode == UCON64_SPP)
+  if (ucon64.parport_mode != PPMODE_EPP)
     spp_set_ai_data (ai, data);                 // SPP mode
   else
     epp_set_ai_data (ai, data);                 // EPP mode
@@ -243,7 +251,7 @@ set_ai_data (unsigned char ai, unsigned char data)
 static void
 write_data (unsigned char data)
 {
-  if (port_mode == UCON64_SPP)
+  if (ucon64.parport_mode != PPMODE_EPP)
     spp_write_data (data);                      // SPP write data
   else
     outportb (port_c, data);                    // EPP write data
@@ -253,7 +261,7 @@ write_data (unsigned char data)
 static unsigned char
 read_data (void)
 {
-  if (port_mode == UCON64_SPP)
+  if (ucon64.parport_mode != PPMODE_EPP)
     return spp_read_data ();                    // SPP read data
   else
     return inportb (port_c);                    // EPP read data
@@ -263,9 +271,7 @@ read_data (void)
 static void
 init_port (void)
 {
-#ifndef USE_PPDEV
-  outportb (port_9, 1);                         // clear EPP time flag
-#endif
+  parport_reset_timeout (port_8);
   set_ai_data ((unsigned char) 2, 0);           // rst=0, wei=0(dis.), rdi=0(dis.)
   set_ai_data ((unsigned char) 2, 0x80);        // rst=1, wei=0(dis.), rdi=0(dis.)
 }
@@ -292,7 +298,7 @@ set_adr (unsigned int adr)
 static void
 set_adr_long (unsigned int adr, int ignore_xh)  // real address
 {
-  unsigned char xh, h, m, l;
+  unsigned char h, m, l;
 
   set_ai_data ((unsigned char) 2, 0x80);        // disable wr/rd inc.
   l = (unsigned char) adr;                      // a7-a0
@@ -303,7 +309,8 @@ set_adr_long (unsigned int adr, int ignore_xh)  // real address
 
   if (!ignore_xh)
     {
-      xh = (unsigned char) (adr >> 22) & 0x7;   // max. 256Mbit
+      unsigned char xh =
+        (unsigned char) (adr >> 22) & 0x7;      // max. 256Mbit
       if (xh)
         m |= 0x40;                              // > bank 0
       set_adr (0x3000);                         // write 3000:xh
@@ -447,13 +454,12 @@ data_polling_data (unsigned char last_data)
 static int
 data_polling (void)
 {
-  unsigned char predata, currdata;
   unsigned int timeout = 0;
+  unsigned char predata = read_byte () & 0x40;
 
-  predata = read_byte () & 0x40;
   while (timeout++ < 0x07ffffff)
     {
-      currdata = read_byte () & 0x40;
+      unsigned char currdata = read_byte () & 0x40;
       if (predata == currdata)
         return 0;
       predata = currdata;
@@ -489,7 +495,7 @@ wait_status (void)
 
   while ((temp & 0xfc) != 0x80)
     {
-      if ((temp & 0x20) == 0x20 && port_mode == UCON64_SPP)
+      if ((temp & 0x20) == 0x20 && ucon64.parport_mode != PPMODE_EPP)
         {
           fputs ("\nERROR: Erase failed\n", stderr);
           return -1;
@@ -733,7 +739,7 @@ check_mbc (void)
         mbc_type = ROCKET;                      // rom_type == 0x97 || rom_type == 0x99
       else if (rom_type == 0)
         mbc_type = ROM;
-      else if ((rom_type >= 1 && rom_type <= 3) || rom_type == 0xff)
+      else if ((/* rom_type >= 1 && */ rom_type <= 3) || rom_type == 0xff)
         mbc_type = MBC1;
       else if (rom_type == 5 || rom_type == 6)
         mbc_type = MBC2;
@@ -757,8 +763,8 @@ check_card (void)
 */
 {
   unsigned char sum = 0;
-  char game_name[16];
-  int i;
+  char game_name[GB_NAME_LEN + 2];
+  int gb_name_len, i;
 
   puts ("Checking ROM data...");
 
@@ -770,9 +776,11 @@ check_card (void)
       dumper (stdout, buffer, 0x50, 0x100, DUMPER_HEX);
     }
 
-  memcpy (game_name, buffer + 0x34, 15);
-  game_name[15] = 0;
-  for (i = 0; i < 15; i++)
+  gb_name_len = (buffer[0x43] == 0x80 || buffer[0x43] == 0xc0) ?
+                  GB_NAME_LEN : GB_NAME_LEN + 1;
+  memcpy (game_name, buffer + 0x34, gb_name_len);
+  game_name[gb_name_len] = 0;
+  for (i = 0; i < gb_name_len; i++)
     if (!isprint ((int) game_name[i]) && game_name[i] != 0)
       game_name[i] = '.';
   printf ("Game name: \"%s\"\n", game_name);
@@ -823,7 +831,14 @@ set_sram_bank (unsigned char bank)
           break;
         default:
           break;
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4061) // enumerator 'identifier' in switch of enum 'enumeration' is not explicitly handled by a case label
+#endif
         }
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
     }
 }
 
@@ -859,7 +874,14 @@ set_bank2 (unsigned int bank)
       break;
     default:
       break;
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4061) // enumerator 'identifier' in switch of enum 'enumeration' is not explicitly handled by a case label
+#endif
     }
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
 }
 
 
@@ -867,7 +889,7 @@ static void
 read_rom_16k (unsigned int bank)                // ROM or EEPROM
 {
   int idx = 0, i, j;
-  char game_name[16];
+  char game_name[GB_NAME_LEN + 2];
 
   set_bank2 (bank);
   for (j = 0; j < 64; j++)
@@ -901,7 +923,7 @@ read_rom_16k (unsigned int bank)                // ROM or EEPROM
                 // Reread the last two pages, because the data came from the
                 //  previously selected game (data is "mirrored"). This does not
                 //  apply to the first game.
-                int k;
+                int k, gb_name_len;
 
                 idx = 0;
                 for (k = 0; k < 2; k++)
@@ -917,9 +939,11 @@ read_rom_16k (unsigned int bank)                // ROM or EEPROM
                   }
 
                 clear_line ();                  // remove last gauge
-                memcpy (game_name, buffer + 0x134, 15);
-                game_name[15] = 0;
-                for (i = 0; i < 15; i++)
+                gb_name_len = (buffer[0x143] == 0x80 || buffer[0x143] == 0xc0) ?
+                                GB_NAME_LEN : GB_NAME_LEN + 1;
+                memcpy (game_name, buffer + 0x134, gb_name_len);
+                game_name[gb_name_len] = 0;
+                for (i = 0; i < gb_name_len; i++)
                   if (!isprint ((int) game_name[i]) && game_name[i] != 0)
                     game_name[i] = '.';
                 printf ("Found another game: \"%s\"\n\n", game_name);
@@ -962,16 +986,19 @@ verify_rom_16k (unsigned int bank)              // ROM or EEPROM
 static int
 win_write_eeprom_16k (unsigned int bank)
 {
-  int wr_done, err_cnt, idx = 0, i, j;
+  int idx = 0, j;
 
 //  disable_protection ();
 
   for (j = 0; j < 64; j++)
     {                                           // 16k bytes = 64 x 256 bytes
-      err_cnt = 16;                             // retry write counter
-      wr_done = 1;
+      int err_cnt = 16;                         // retry write counter
+      int wr_done = 1;
+
       while (wr_done)
         {
+          int i;
+
           enable_protection ();
           // write 256 bytes
           set_bank (0x2000, (unsigned char) bank); // for MCB1 16k bank
@@ -1036,10 +1063,12 @@ set_page_write (void)                           // start page write command
 static int
 page_write_128 (unsigned int bank, unsigned char hi_lo, int j, int idx)
 {
-  int retry = 3, verify_ok, i;
+  int retry = 3;
 
   while (retry)
     {
+      int verify_ok, i;
+
       set_page_write ();                        // each page is 128 bytes
       set_bank (0x2000, (unsigned char) bank);  // for MCB1 16k bank
       if (bank)
@@ -1059,7 +1088,7 @@ page_write_128 (unsigned int bank, unsigned char hi_lo, int j, int idx)
 
       // verify data
       reset_to_read ();                         // return to read mode
-      verify_ok = 1;                            // verify ok
+      verify_ok = 1;                            // verify OK
       set_bank (0x2000, (unsigned char) bank);  // for MCB1 16k bank
       if (bank)
         set_ai_data ((unsigned char) 1, (unsigned char) (j | 0x40)); // set adr[15..8]
@@ -1197,7 +1226,7 @@ static int
 intel_write_eeprom_16k (unsigned int bank)
 {
   unsigned int block_adr = bank << 14;          // convert to real address
-  int idx, j;
+  int j;
 
   if ((bank & 0x07) == 0)
     if (intel_block_erase (block_adr))
@@ -1206,7 +1235,7 @@ intel_write_eeprom_16k (unsigned int bank)
 //  set_adr_long (block_adr, 0);                  // set real address
   for (j = 0; j < 512; j++)
     {                                           // 16k bytes = 512 x 32 bytes
-      idx = j * 32;
+      int idx = j * 32;
 //      if (intel_byte_write_32 (block_adr, idx)) return -1;
       if (intel_buffer_write_32 (block_adr, idx))
         {
@@ -1309,14 +1338,9 @@ check_port_mode (void)
 static int
 check_port (void)
 {
-  if (ucon64.parport_mode == UCON64_EPP && port_8 != 0x3bc)
-    port_mode = UCON64_EPP;                     // if port == 0x3bc => no EPP available
-  else
-    port_mode = UCON64_SPP;
-
   if (check_port_mode ())
     {
-      port_mode = UCON64_SPP;
+      ucon64.parport_mode = parport_setup (port_8, PPMODE_SPP);
       if (check_port_mode ())
         return 1;
       else
@@ -1324,7 +1348,7 @@ check_port (void)
     }
 
   // If we get here, a GBX was detected
-  if (port_mode == UCON64_EPP)
+  if (ucon64.parport_mode == PPMODE_EPP)
     puts ("GBX found. EPP found");
   else
     puts ("GBX found. EPP not found or not enabled - SPP used");
@@ -1481,7 +1505,7 @@ test_sram_wv (int n_banks)
 }
 
 
-int cart_type = 0;                              // should be set to value of ROM[0x147]
+static int cart_type = 0;                       // should be set to value of ROM[0x147]
 
 static void
 set_rom_bank (unsigned char bank)
@@ -1555,7 +1579,8 @@ test_intel (void)
 }
 
 
-static void gbx_init (unsigned int parport, int read_header);
+static void gbx_init (unsigned short parport, int read_header);
+
 
 static int
 verify_card_from_file (const char *filename, unsigned int parport)
@@ -1612,10 +1637,8 @@ verify_card_from_file (const char *filename, unsigned int parport)
 
 
 static void
-gbx_init (unsigned int parport, int read_header)
+gbx_init (unsigned short parport, int read_header)
 {
-  int i;
-
   eeprom_type = UNKNOWN_EEPROM;
   rocket_game_no = 0;
 
@@ -1636,11 +1659,15 @@ gbx_init (unsigned int parport, int read_header)
   check_eeprom ();
 
   if (read_header)
-    for (i = 0x100; i < 0x150; i++)
-      {                                         // read 0x100-0x150 to buffer
-        set_adr (i);
-        buffer[i - 0x100] = read_data ();
-      }
+    {
+      int i;
+
+      for (i = 0x100; i < 0x150; i++)
+        {                                       // read 0x100-0x150 to buffer
+          set_adr (i);
+          buffer[i - 0x100] = read_data ();
+        }
+    }
   /*
     buffer is undefined if read_header == 0. This is not a problem as
     read_header is only 0 if a flash card should be programmed
@@ -1651,7 +1678,7 @@ gbx_init (unsigned int parport, int read_header)
 
 
 int
-gbx_read_rom (const char *filename, unsigned int parport)
+gbx_read_rom (const char *filename, unsigned short parport)
 {
   unsigned int bank, n_banks, rom_size, n_bytes = 0, totalbytes;
   time_t starttime;
@@ -1707,9 +1734,9 @@ gbx_read_rom (const char *filename, unsigned int parport)
 
 
 int
-gbx_write_rom (const char *filename, unsigned int parport)
+gbx_write_rom (const char *filename, unsigned short parport)
 {
-  int bank, n_banks, n_bytes = 0, filesize;
+  int bank, n_banks, n_bytes = 0;
   time_t starttime;
   FILE *file;
 
@@ -1721,13 +1748,13 @@ gbx_write_rom (const char *filename, unsigned int parport)
       exit (1);
     }
 
-  filesize = fsizeof (filename);
-  if ((filesize < 0x8000) || (filesize & 0x7fff) || (filesize > 64 * MBIT))
+  if ((ucon64.fsize < 0x8000) || (ucon64.fsize & 0x7fff) ||
+      (ucon64.fsize > 64 * MBIT))
     {
       fputs ("ERROR: File size error\n", stderr);
       exit (1);
     }
-  n_banks = (filesize / 0x8000) * 2;            // how many 16k banks (rounded
+  n_banks = ((int) ucon64.fsize / 0x8000) * 2;  // how many 16k banks (rounded
                                                 //  down to 32k boundary)
   if (eeprom_type == MX)
     if (mx_erase ())                            // erase 16M flash
@@ -1765,7 +1792,7 @@ gbx_write_rom (const char *filename, unsigned int parport)
           exit (1);
         }
       n_bytes += 0x4000;
-      ucon64_gauge (starttime, n_bytes, filesize);
+      ucon64_gauge (starttime, n_bytes, (size_t) ucon64.fsize);
     }
 
 #if 0 // write_eeprom_16k() already calls verify_rom_16k() (indirectly)...
@@ -1791,7 +1818,7 @@ gbx_write_rom (const char *filename, unsigned int parport)
           exit (1);
         }
       n_bytes += 0x4000;
-      ucon64_gauge (starttime, n_bytes, filesize);
+      ucon64_gauge (starttime, n_bytes, ucon64.fsize);
     }
 #endif
 
@@ -1838,9 +1865,9 @@ sram_size_banks (int pocket_camera, unsigned char sram_size_byte)
 
 
 int
-gbx_read_sram (const char *filename, unsigned int parport, int start_bank)
+gbx_read_sram (const char *filename, unsigned short parport, int start_bank)
 {
-  int bank, n_banks, n_bytes = 0, totalbytes, idx, i, j;
+  int bank, n_banks, n_bytes = 0, totalbytes, i, j;
   time_t starttime;
   FILE *file;
 
@@ -1883,7 +1910,7 @@ gbx_read_sram (const char *filename, unsigned int parport, int start_bank)
   starttime = time (NULL);
   for (bank = start_bank; bank < start_bank + n_banks; bank++)
     {
-      idx = 0;
+      int idx = 0;
       set_sram_bank ((unsigned char) bank);
       for (j = 0; j < 32; j++)
         {                                       // 32 x 256 = 8192 (8 kbytes)
@@ -1909,9 +1936,9 @@ gbx_read_sram (const char *filename, unsigned int parport, int start_bank)
 
 
 int
-gbx_write_sram (const char *filename, unsigned int parport, int start_bank)
+gbx_write_sram (const char *filename, unsigned short parport, int start_bank)
 {
-  int bank, n_banks, n_bytes = 0, totalbytes, idx, i, j;
+  unsigned int bank, n_banks, n_bytes = 0, totalbytes, i, j;
   time_t starttime;
   FILE *file;
 
@@ -1928,12 +1955,11 @@ gbx_write_sram (const char *filename, unsigned int parport, int start_bank)
   if (start_bank == -1)
     {
       start_bank = 0;
-      i = fsizeof (filename);
-      n_banks = MIN (n_banks, (i + 8192 - 1) / 8192); // "+ 8192 - 1" to round up
+      n_banks = MIN (n_banks, ((unsigned int) ucon64.fsize + 8192 - 1) / 8192); // "+ 8192 - 1" to round up
     }
   else
     {
-      if (start_bank >= n_banks)
+      if ((unsigned int) start_bank >= n_banks)
         {
           fprintf (stderr, "ERROR: Bank must be a value 0 - %d (for this card)\n",
                    n_banks - 1);
@@ -1958,7 +1984,7 @@ gbx_write_sram (const char *filename, unsigned int parport, int start_bank)
   starttime = time (NULL);
   for (bank = start_bank; bank < start_bank + n_banks; bank++)
     {
-      idx = 0;
+      unsigned int idx = 0;
       if (!fread (buffer, 1, 8192, file))
         { // handle/allow files that are not an exact multiple of 8 kB
           fprintf (stderr, ucon64_msg[READ_ERROR], filename);
