@@ -5,25 +5,46 @@
  * Library routines for CD64 handling
  *
  * (c) 2004 Ryan Underwood
- * Portions (c) 2004 Daniel Horchner (Win32, read/write/seek callbacks)
+ * Portions (c) 2004, 2019 - 2020 Daniel Horchner (Win32, read/write/seek callbacks)
  *
  * May be distributed under the terms of the GNU Lesser/Library General
  * Public License, or any later version of the same, as published by the Free
  * Software Foundation.
  */
 
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
+#endif
 #include <stdlib.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
 #include <stdarg.h>
 #include <sys/types.h>
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4820) /* 'bytes' bytes padding added after construct 'member_name' */
+#endif
 #include <sys/stat.h>
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
 #if defined __unix__ || defined __BEOS__
 #include <unistd.h>
 #endif
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4820) /* 'bytes' bytes padding added after construct 'member_name' */
+#endif
 #include <sys/timeb.h>
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
 #else
 #include <sys/time.h>
 #endif
@@ -74,11 +95,11 @@ int cd64_notice2(const char *format, ...) {
 }
 
 int cd64_read(void *io_id, void *buffer, uint32_t size) {
-	return fread(buffer, 1, size, (FILE *) io_id);
+	return (int) fread(buffer, 1, size, (FILE *) io_id);
 }
 
 int cd64_write(void *io_id, void *buffer, uint32_t size) {
-	return fwrite(buffer, 1, size, (FILE *) io_id);
+	return (int) fwrite(buffer, 1, size, (FILE *) io_id);
 }
 
 int32_t cd64_tell(void *io_id) {
@@ -219,13 +240,13 @@ int cd64_bios_grab(struct cd64_t *cd64, void *io_id, uint32_t addr,
 	/* Try to get in sync with the CD64
 	 * We use a delay here to give the PPA a chance to power up. */
 	send = 0xff;
-	cd64->xfer(cd64, &send, NULL, 1000);
+	cd64->xfer(cd64, &send, NULL, 1);
 	cd64_bios_sync(cd64);
 
 	if (elapsed != NULL) {
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 		ftime(&tb);
-		sec = tb.time;
+		sec = (unsigned long int) tb.time;
 		usec = tb.millitm*1000;
 #else
 		gettimeofday(&tv, 0);
@@ -262,10 +283,10 @@ int cd64_bios_grab(struct cd64_t *cd64, void *io_id, uint32_t addr,
 	if (elapsed != NULL) {
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 		ftime(&tb);
-		*elapsed = ((tb.time - sec)*1000000) + ((tb.millitm*1000) - usec);
+		*elapsed = (((unsigned long int) tb.time-sec)*1000000)+((tb.millitm*1000)-usec);
 #else
 		gettimeofday(&tv, 0);
-		*elapsed = ((tv.tv_sec - sec)*1000000) + (tv.tv_usec - usec);
+		*elapsed = ((tv.tv_sec-sec)*1000000)+(tv.tv_usec-usec);
 #endif
 	}
 
@@ -345,13 +366,13 @@ int cd64_bios_send(struct cd64_t *cd64, void *io_id, uint32_t addr,
 	/* Try to get in sync with the CD64
 	 * We use a delay here to give the PPA a chance to power up. */
 	send = 0xff;
-	cd64->xfer(cd64, &send, NULL, 1000);
+	cd64->xfer(cd64, &send, NULL, 1);
 	cd64_bios_sync(cd64);
 
 	if (elapsed != NULL) {
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 		ftime(&tb);
-		sec = tb.time;
+		sec = (unsigned long int) tb.time;
 		usec = tb.millitm*1000;
 #else
 		gettimeofday(&tv, 0);
@@ -416,10 +437,10 @@ int cd64_bios_send(struct cd64_t *cd64, void *io_id, uint32_t addr,
 	if (elapsed != NULL) {
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 		ftime(&tb);
-		*elapsed = ((tb.time - sec)*1000000) + ((tb.millitm*1000) - usec);
+		*elapsed = (((unsigned long int) tb.time-sec)*1000000)+((tb.millitm*1000)-usec);
 #else
 		gettimeofday(&tv, 0);
-		*elapsed = ((tv.tv_sec - sec)*1000000) + (tv.tv_usec - usec);
+		*elapsed = ((tv.tv_sec-sec)*1000000)+(tv.tv_usec-usec);
 #endif
 	}
 
@@ -445,7 +466,7 @@ int cd64_ghemor_grab(struct cd64_t *cd64, void *io_id, uint8_t slow, int *elapse
 
 	int ret;
 	uint8_t tmp;
-	int sec = 0, usec = 0;
+	unsigned long int sec = 0, usec = 0;
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 	struct timeb tb;
 #else
@@ -463,7 +484,7 @@ int cd64_ghemor_grab(struct cd64_t *cd64, void *io_id, uint8_t slow, int *elapse
 	if (elapsed != NULL) {
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 		ftime(&tb);
-		sec = tb.time;
+		sec = (unsigned long int) tb.time;
 		usec = tb.millitm*1000;
 #else
 		gettimeofday(&tv, 0);
@@ -510,10 +531,10 @@ int cd64_ghemor_grab(struct cd64_t *cd64, void *io_id, uint8_t slow, int *elapse
 	if (elapsed != NULL) {
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 		ftime(&tb);
-		*elapsed = ((tb.time - sec)*1000000) + ((tb.millitm*1000) - usec);
+		*elapsed = (((unsigned long int) tb.time-sec)*1000000)+((tb.millitm*1000)-usec);
 #else
 		gettimeofday(&tv, 0);
-		*elapsed = ((tv.tv_sec - sec)*1000000) + (tv.tv_usec - usec);
+		*elapsed = ((tv.tv_sec-sec)*1000000)+(tv.tv_usec-usec);
 #endif
 	}
 
@@ -523,7 +544,7 @@ int cd64_ghemor_grab(struct cd64_t *cd64, void *io_id, uint8_t slow, int *elapse
 int cd64_ghemor_send(struct cd64_t *cd64, void *io_id, uint32_t length,
 		int *elapsed) {
 
-	int sec = 0, usec = 0;
+	unsigned long int sec = 0, usec = 0;
 	uint16_t mycsum = 0;
 	unsigned int i;
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
@@ -536,7 +557,7 @@ int cd64_ghemor_send(struct cd64_t *cd64, void *io_id, uint32_t length,
 	if (elapsed != NULL) {
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 		ftime(&tb);
-		sec = tb.time;
+		sec = (unsigned long int) tb.time;
 		usec = tb.millitm*1000;
 #else
 		gettimeofday(&tv, 0);
@@ -576,10 +597,10 @@ int cd64_ghemor_send(struct cd64_t *cd64, void *io_id, uint32_t length,
 	if (elapsed != NULL) {
 #if (defined _WIN32 && !defined __CYGWIN__) || defined __MSDOS__
 		ftime(&tb);
-		*elapsed = ((tb.time - sec)*1000000) + ((tb.millitm*1000) - usec);
+		*elapsed = (((unsigned long int) tb.time-sec)*1000000)+((tb.millitm*1000)-usec);
 #else
 		gettimeofday(&tv, 0);
-		*elapsed = ((tv.tv_sec - sec)*1000000) + (tv.tv_usec - usec);
+		*elapsed = ((tv.tv_sec-sec)*1000000)+(tv.tv_usec-usec);
 #endif
 	}
 
@@ -615,8 +636,8 @@ int cd64_upload_bootemu(struct cd64_t *cd64, FILE *infile, uint32_t length, int
 
 	if (cd64->protocol == CD64BIOS) {
 
-		if (cd64->protocol == CD64BIOS && length == 0) {
-			cd64->notice_callback2("CD64 BIOS needs a file length.\n");
+		if (length == 0) {
+			cd64->notice_callback2("CD64 BIOS needs a file length.");
 			return 0;
 		}
 
@@ -659,7 +680,7 @@ int cd64_upload_mempak(struct cd64_t *cd64, FILE *infile, int8_t which) {
 
 	if (cd64->protocol == CD64BIOS) {
 		if (which != -1) {
-			cd64->notice_callback2("CD64 BIOS doesn't let mempak index be chosen.");
+			cd64->notice_callback2("CD64 BIOS does not let mempak index be chosen.");
 			return 0;
 		}
 		cd64->notice_callback("Choose Memory Manager->Up/Download Pak.");
@@ -763,7 +784,7 @@ int cd64_download_header(struct cd64_t *cd64, n64header_t *head, uint32_t locati
 		cd64->write_callback = cd64_write_mem;
 		ret = cd64_bios_grab(cd64, (void *) -1, location, size, NULL); /* -1 is just a random (non-zero) value */
 		cd64->write_callback = org_write_cb; /* restore original callback */
-		return 1;
+		return ret;
 	}
 	cd64->notice_callback2("Operation not supported by protocol.");
 	return 0;
@@ -797,7 +818,7 @@ int cd64_download_cart(struct cd64_t *cd64, FILE *outfile, uint32_t length,
 
 			curpos = cd64->tell_callback(outfile);
 
-			while(i+j < length) {
+			while (i+j < length) {
 				cd64->read_callback(outfile, &buf, 4);
 
 				/* To elaborate on what we are checking here:
@@ -839,7 +860,9 @@ int cd64_download_cart(struct cd64_t *cd64, FILE *outfile, uint32_t length,
 			 * outfile may not be a real FILE *. */
 			if (cd64->read_callback == cd64_read) {
 				cd64->notice_callback("Truncating to %dMbits.", i/BYTES_IN_MBIT);
-				ftruncate(fileno(outfile), curpos+i);
+				if (ftruncate(fileno(outfile), curpos+i)) {
+					cd64->notice_callback2("Truncating failed.");
+				}
 			}
 #endif
 		}
@@ -907,7 +930,7 @@ int cd64_download_mempak(struct cd64_t *cd64, FILE *outfile, int8_t which) {
 
 	if (cd64->protocol == CD64BIOS) {
 		if (which != -1) {
-			cd64->notice_callback2("CD64 BIOS doesn't let mempak index be chosen.");
+			cd64->notice_callback2("CD64 BIOS does not let mempak index be chosen.");
 			return 0;
 		}
 		cd64->notice_callback("Choose Memory Manager->Up/Download Pak.");
