@@ -8,10 +8,10 @@
 #define BYTES_IN_MBIT 0x20000
 
 #define SwapEndian(x) \
-	  ((x >> 24)&0x000000FF) \
-	| ((x >> 8 )&0x0000FF00) \
-	| ((x << 8 )&0x00FF0000) \
-	| ((x << 24)&0xFF000000)
+	  (((x) >> 24)&0x000000FF) \
+	| (((x) >> 8 )&0x0000FF00) \
+	| (((x) << 8 )&0x00FF0000) \
+	| (((x) << 24)&0xFF000000)
 
 typedef enum {
 	UNKNOWN = 0,
