@@ -2,7 +2,7 @@
 lynxit.c - lynxit support for uCON64
 
 Copyright (c) 1997 - ???? K. Wilkins
-Copyright (c) 2002        NoisyB <noisyb@gmx.net>
+Copyright (c) 2002        NoisyB
 Copyright (c) 2004        dbjh
 
 
@@ -23,26 +23,30 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifdef  HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <fcntl.h>
-#include <stdio.h>
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
+#endif
 #include <stdlib.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
 #include <string.h>
-#ifdef  HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include "misc/misc.h"
-#include "misc/itypes.h"
-#ifdef  USE_ZLIB
 #include "misc/archive.h"
-#endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "ucon64.h"
+#include "misc/parallel.h"
+#include "misc/term.h"
 #include "ucon64_misc.h"
 #include "console/lynx.h"
-#include "lynxit.h"
-#include "misc/parallel.h"
+#include "backup/lynxit.h"
 
 
+#ifdef  USE_PARALLEL
+static st_ucon64_obj_t lynxit_obj[] =
+  {
+    {UCON64_LYNX, WF_STOP | WF_NO_ROM}
+  };
+#endif
+
 const st_getopt2_t lynxit_usage[] =
   {
     {
@@ -53,9 +57,9 @@ const st_getopt2_t lynxit_usage[] =
 #ifdef  USE_PARALLEL
     {
       "xlit", 0, 0, UCON64_XLIT,
-      NULL, "receive ROM from Lynxit interface; " OPTION_LONG_S "port=PORT",
-//    "                  receives automatically when ROM does not exist\n"
-      &ucon64_wf[WF_OBJ_LYNX_STOP_NO_ROM]
+      NULL, "receive ROM from Lynxit interface; " OPTION_LONG_S "port" OPTARG_S "PORT",
+//      "receives automatically when ROM does not exist",
+      &lynxit_obj[0]
     },
 #endif
     {NULL, 0, 0, 0, NULL, NULL, NULL}
@@ -96,8 +100,8 @@ const st_getopt2_t lynxit_usage[] =
 #define    DATA_LOAD   0x20
 #define    PAGE_STB    0x40
 
-unsigned int printer_port = 1, print_data = 0, print_ctrl = 0, print_stat = 0;
-char cartname[32], manufname[16];
+static unsigned int printer_port = 1, print_data = 0, print_ctrl = 0, print_stat = 0;
+static char cartname[32], manufname[16];
 
 //
 // CONTROL REGISTER DEFINITIONS
@@ -113,9 +117,9 @@ char cartname[32], manufname[16];
 #define    CTRL_INACTIVE   (CTRL_BANK0B|CTRL_BANK1B|CTRL_CTR_CLKB)
 
 
-// Make cart strobes inactive & counter clock & reset inactive 
+// Make cart strobes inactive & counter clock & reset inactive
 
-unsigned char control_register = CTRL_INACTIVE;
+static unsigned char control_register = CTRL_INACTIVE;
 static void lynxit_write_control (unsigned char data);
 
 
@@ -128,12 +132,12 @@ static int debug = FALSE, quiet = FALSE, verify = TRUE;
 
 #define  MESSAGE(body) printf body
 
-#define  INPUT(port)        inportb((unsigned short) (port))
-#define  OUTPUT(port, data) outportb((unsigned short) (port), (unsigned char) (data))
+#define  INPUT(port)        inportb ((unsigned short) (port))
+#define  OUTPUT(port, data) outportb ((unsigned short) (port), (unsigned char) (data))
 
 
 #if 0
-void
+static void
 usage (void)
 {
   MESSAGE (("\nUsage: lynxit [-pX] [-d] [-q] <command> <filename> [cartname] [manuf]\n"));
@@ -148,7 +152,7 @@ usage (void)
 #endif
 
 
-int
+static int
 ptr_port_init (unsigned int port)
 {
 #if 0
@@ -178,19 +182,19 @@ ptr_port_init (unsigned int port)
 }
 
 
-void
+static void
 lynxit_shift_out_byte (unsigned char data)
 {
   unsigned int loop;
-  unsigned char outbyte, dbgdata;
-
-  dbgdata = data;
+#if 0
+  unsigned char dbgdata = data;
+#endif
 
   OUTPUT (print_data, PORT_BASE);       // Set inactive
 
   for (loop = 0; loop < 8; loop++)
     {
-      outbyte = PORT_BASE;
+      unsigned char outbyte = PORT_BASE;
       outbyte |= (data & 0x80) ? DATA_OUT : 0;
       OUTPUT (print_data, outbyte);     // Output data; clock low
       data = data << 1;
@@ -206,7 +210,7 @@ lynxit_shift_out_byte (unsigned char data)
 }
 
 
-unsigned char
+static unsigned char
 lynxit_shift_in_byte (void)
 {
   unsigned int loop;
@@ -231,7 +235,7 @@ lynxit_shift_in_byte (void)
 }
 
 
-void
+static void
 lynxit_write_control (unsigned char data)
 {
 #if 0
@@ -247,7 +251,7 @@ lynxit_write_control (unsigned char data)
 }
 
 
-void
+static void
 lynxit_write_page (unsigned char page)
 {
 #if 0
@@ -262,7 +266,7 @@ lynxit_write_page (unsigned char page)
 }
 
 
-void
+static void
 lynxit_counter_reset (void)
 {
   lynxit_write_control ((unsigned char) (control_register | CTRL_CTR_RST));
@@ -270,7 +274,7 @@ lynxit_counter_reset (void)
 }
 
 
-void
+static void
 lynxit_counter_increment (void)
 {
   lynxit_write_control ((unsigned char) (control_register & (CTRL_CTR_CLKB ^ 0xff)));
@@ -278,7 +282,7 @@ lynxit_counter_increment (void)
 }
 
 
-unsigned char
+static unsigned char
 cart_read_byte (unsigned int cart)
 {
   unsigned char data;
@@ -333,7 +337,8 @@ cart_read_byte (unsigned int cart)
 }
 
 
-void
+#if 0
+static void
 cart_write_byte (unsigned int cart, unsigned char data)
 {
 #if 0
@@ -382,9 +387,10 @@ cart_write_byte (unsigned int cart, unsigned char data)
   lynxit_write_control ((unsigned char) (control_register | CTRL_WR_EN));
 
 }
+#endif
 
 
-void
+static void
 cart_read_page (unsigned int cart, unsigned int page_number,
                 unsigned int page_size, unsigned char *page_ptr)
 {
@@ -403,7 +409,7 @@ cart_read_page (unsigned int cart, unsigned int page_number,
 
 
 #if 0
-void
+static void
 cart_write_page (unsigned int cart, unsigned int page_number,
                  unsigned int page_size, unsigned char *page_ptr)
 {
@@ -411,12 +417,11 @@ cart_write_page (unsigned int cart, unsigned int page_number,
 #endif
 
 
-int
+static int
 cart_analyse (int cart)
 {
   unsigned char image[MAX_PAGE_SIZE];
   unsigned int page = 0;
-  unsigned char test = 0;
   unsigned int loop = 0;
 
   MESSAGE (("ANALYSE  : BANK%d ", cart));
@@ -425,6 +430,8 @@ cart_analyse (int cart)
     {
       // Read a page - start at zero, try a max of 8 pages
 
+      unsigned char test = 0xff;
+
       if (page > 8)
         {
           MESSAGE (("N/A\n"));
@@ -435,7 +442,6 @@ cart_analyse (int cart)
 
       // Explicit check for no bank
 
-      test = 0xff;
       for (loop = 0; loop < CART_PAGE_512K; loop++)
         test &= image[loop];
 
@@ -532,7 +538,7 @@ cart_analyse (int cart)
 
 #define  MAX_ERRORS    16
 
-int
+static int
 cart_verify (char *filename)
 {
   unsigned char image1[MAX_PAGE_SIZE], image2[MAX_PAGE_SIZE];
@@ -561,7 +567,7 @@ cart_verify (char *filename)
       return FALSE;
     }
 
-  if (strcmp (header.magic, "LYNX") != 0)
+  if (memcmp (header.magic, MAGIC_STRING, sizeof header.magic) != 0)
     {
       MESSAGE (("ERROR    : %s is not a lynx image\n", filename));
       fclose (fp);
@@ -668,7 +674,7 @@ cart_verify (char *filename)
 }
 
 
-int
+static int
 cart_read (char *filename)
 {
   unsigned char image[MAX_PAGE_SIZE];
@@ -681,12 +687,14 @@ cart_read (char *filename)
   DEBUG (("read_cart() called with <%s>\n\n", filename));
 #endif
 
-  memset (&header, 0, sizeof (st_lnx_header_t));
-  strcpy (header.magic, MAGIC_STRING);
-  strcpy (header.cartname, cartname);
-  strcpy (header.manufname, manufname);
-  header.page_size_bank0 = cart_analyse (BANK0);
-  header.page_size_bank1 = cart_analyse (BANK1);
+  memset (&header, 0, sizeof header);
+  memcpy (header.magic, MAGIC_STRING, sizeof header.magic);
+  strncpy (header.cartname, cartname, sizeof header.cartname - 1)
+    [sizeof header.cartname - 1] = '\0';
+  strncpy (header.manufname, manufname, sizeof header.manufname - 1)
+    [sizeof header.manufname - 1] = '\0';
+  header.page_size_bank0 = (short) cart_analyse (BANK0);
+  header.page_size_bank1 = (short) cart_analyse (BANK1);
   header.version = FILE_FORMAT_VERSION;
 
   if ((fp = fopen (filename, "wb")) == NULL)
@@ -747,7 +755,7 @@ cart_read (char *filename)
 
 
 #if 0
-int
+static int
 cart_write (char *filename)
 {
   DEBUG (("write_cart() called with <%s>\n\n", filename));
@@ -756,7 +764,7 @@ cart_write (char *filename)
 #endif
 
 
-int
+static int
 perform_test (char *testname)
 {
   if (strcmp (testname, "LOOPBACK") == 0)
@@ -855,7 +863,7 @@ perform_test (char *testname)
       int stepmode = TRUE;
 
       MESSAGE (("\nPress <space> key to step counter U5\n\n"));
-      MESSAGE (("  'q' - Quit to DOS\n"));
+      MESSAGE (("  'q' - Quit\n"));
       MESSAGE (("  's' - Step mode (default)\n"));
       MESSAGE (("  'r' - Run mode\n"));
       MESSAGE (("  'c' - Clear counter\n\n"));
@@ -888,7 +896,7 @@ perform_test (char *testname)
                 }
             }
         }
-      return TRUE;
+//      return TRUE;
     }
   else
     {
@@ -907,7 +915,7 @@ perform_test (char *testname)
 static char buf[FILENAME_MAX];
 
 
-int
+static int
 lynxit_main (int argc, char **argv)
 {
   int loop;
@@ -929,7 +937,7 @@ lynxit_main (int argc, char **argv)
         {
 //          usage ();
           printf ("\nInvalid argument %d <%s>\n", loop, argv[loop]);
-          exit (FALSE);
+          return FALSE;
         }
 
       switch (argv[loop][1])
@@ -948,11 +956,11 @@ lynxit_main (int argc, char **argv)
           break;
         case 'H':
 //          usage ();
-          exit (FALSE);
+          return FALSE;
         default:
 //          usage ();
           printf ("\nUnrecognised argument %d <%s>\n", loop, argv[loop]);
-          exit (FALSE);
+          return FALSE;
         }
     }
 
@@ -962,16 +970,16 @@ lynxit_main (int argc, char **argv)
     {
 //      usage ();
       MESSAGE (("\nERROR    : Missing command/filename\n"));
-      exit (FALSE);
+      return FALSE;
     }
 
   // Initialise the printer port
 
   if (!ptr_port_init (printer_port))
     {
-      MESSAGE (("ERROR    : Couldn't initialise printer port LPT%d\n",
+      MESSAGE (("ERROR    : Could not initialise printer port LPT%d\n",
                 printer_port));
-      exit (FALSE);
+      return FALSE;
     }
 
   // Perform loopback tests to prove operation
@@ -979,9 +987,9 @@ lynxit_main (int argc, char **argv)
   if (!debug && strcmp (argv[loop], "TEST") != 0
       && !perform_test ("LOOPBACK"))
     {
-      MESSAGE (("ERROR    : LYNXIT doesn't appear to be working ????\n"));
+      MESSAGE (("ERROR    : LYNXIT does not appear to be working\n"));
       MESSAGE (("ERROR    : (Check its plugged in and switched on)\n"));
-      exit (FALSE);
+      return FALSE;
     }
 
   if (strcmp (argv[loop], "READ") == 0)
@@ -989,21 +997,25 @@ lynxit_main (int argc, char **argv)
       if (loop + 3 >= argc)
         {
           MESSAGE (("ERROR    : Missing Cartname/Manufacturer arguments\n"));
-          exit (FALSE);
+          return FALSE;
         }
-      strcpy (cartname, argv[argc - 2]);
-      strcpy (manufname, argv[argc - 1]);
+      strncpy (cartname, argv[argc - 2], sizeof cartname - 1)
+        [sizeof cartname - 1] = '\0';
+      strncpy (manufname, argv[argc - 1], sizeof manufname - 1)
+        [sizeof manufname - 1] = '\0';
+#if 0
       if (strlen (cartname) > 32 || strlen (manufname) > 16)
         {
           MESSAGE (("ERROR    : Cartname/Manufacturer arguments too long (32/16)\n"));
-          exit (FALSE);
+          return FALSE;
         }
+#endif
 
       strcpy (buf, argv[++loop]);
       if (cart_read (buf) == FALSE)
         {
           MESSAGE (("ERROR    : Cartridge read failed\n"));
-          exit (FALSE);
+          return FALSE;
         }
     }
   else if (strcmp (argv[loop], "WRITE") == 0)
@@ -1023,10 +1035,10 @@ lynxit_main (int argc, char **argv)
     {
 //      usage ();
       printf ("\nInvalid command argument - Use READ/WRITE/VERIFY/TEST\n");
-      exit (FALSE);
+      return FALSE;
     }
 
-  exit (TRUE);
+  return TRUE;
 }
 
 
@@ -1045,10 +1057,10 @@ lynxit_read_rom (const char *filename, unsigned int parport)
 
   if (lynxit_main (3, argv) != 0)
     {
-      fprintf (stderr, ucon64_msg[PARPORT_ERROR]);
+      fputs (ucon64_msg[PARPORT_ERROR], stderr);
       exit (1);
     }
-  
+
   return 0;
 }
 
