@@ -2,7 +2,7 @@
 lynxit.h - lynxit support for uCON64
 
 Copyright (c) 1997 - ???? K. Wilkins
-Copyright (c) 2002        NoisyB <noisyb@gmx.net>
+Copyright (c) 2002        NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,15 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef LYNXIT_H
 #define LYNXIT_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t lynxit_usage[];
 
-#ifdef USE_PARALLEL
+#ifdef  USE_PARALLEL
 extern int lynxit_read_rom (const char *filename, unsigned int parport);
 #endif
 
