@@ -1,7 +1,7 @@
 /*
 mccl.h - Mad Catz Camera Link (Game Boy Camera) support for uCON64
 
-Copyright (c) 2002 NoisyB <noisyb@gmx.net>
+Copyright (c) 2002 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -21,10 +21,16 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef MCCL_H
 #define MCCL_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t mccl_usage[];
 
-#ifdef USE_PARALLEL
+#ifdef  USE_PARALLEL
 extern int mccl_read (const char *filename, unsigned int parport);
 #endif
 
-#endif // MCCL_H
+#endif
