@@ -1,7 +1,7 @@
 /*
 msg.h - Magic Super Griffin support for uCON64
 
-Copyright (c) 2003 dbjh
+Copyright (c) 2003, 2020 dbjh
 
 
 This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,12 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef MSG_H
 #define MSG_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t msg_usage[];
 
 // For the header format, see ffe.h
@@ -28,19 +34,18 @@ typedef struct st_msg_header
 {
   unsigned char size;
   unsigned char emulation;
-  unsigned char pad[6];
+  unsigned char pad1[6];
   unsigned char id1;
   unsigned char id2;
   unsigned char type;
   unsigned char pad2[501];
 } st_msg_header_t;
 
-#define MSG_HEADER_START 0
 #define MSG_HEADER_LEN (sizeof (st_msg_header_t))
 
-#ifdef USE_PARALLEL
-extern int msg_read_rom (const char *filename, unsigned int parport);
-extern int msg_write_rom (const char *filename, unsigned int parport);
+#ifdef  USE_PARALLEL
+extern int msg_read_rom (const char *filename, unsigned short parport);
+extern int msg_write_rom (const char *filename, unsigned short parport);
 #endif
 
-#endif // MSG_H
+#endif
