@@ -1,7 +1,7 @@
 /*
-pce-pro.h - PCE-PRO flash card programmer support for uCON64
+pce-pro.h - PCE-PRO support for uCON64
 
-Copyright (c) 2004 dbjh
+Copyright (c) 2004 - 2005, 2015, 2017, 2020 dbjh
 
 Based on Delphi source code by ToToTEK Multi Media. Information in that source
 code has been used with permission. However, ToToTEK Multi Media explicitly
@@ -25,11 +25,20 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef PCE_PRO_H
 #define PCE_PRO_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
+#define PCE_PRO_LOADER_SIZE (56 * 1024)
+
 extern const st_getopt2_t pcepro_usage[];
 
-#ifdef USE_PARALLEL
-extern int pce_read_rom (const char *filename, unsigned int parport, int size);
-extern int pce_write_rom (const char *filename, unsigned int parport);
+#ifdef  USE_PARALLEL
+extern int pce_read_rom (const char *filename, unsigned short parport,
+                         unsigned int size);
+extern int pce_write_rom (const char *filename, unsigned short parport);
 #endif
 
 #endif
