@@ -22,12 +22,18 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef PL_H
 #define PL_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t pl_usage[];
 
 #ifdef  USE_PARALLEL
-extern int pl_info (unsigned int parport);
-extern int pl_read_rom (const char *filename, unsigned int parport);
-extern int pl_write_rom (const char *filename, unsigned int parport);
+extern int pl_info (unsigned short parport);
+extern int pl_read_rom (const char *filename, unsigned short parport);
+extern int pl_write_rom (const char *filename, unsigned short parport);
 #endif
 
 #endif
