@@ -1,7 +1,8 @@
 /*
-sflash.h - Super Flash flash card programmer support for uCON64
+sflash.h - Super Flash support for uCON64
 
-Copyright (c) 2004 JohnDie
+Copyright (c) 2004             JohnDie
+Copyright (c) 2015, 2017, 2020 dbjh
 
 
 This program is free software; you can redistribute it and/or modify
@@ -21,13 +22,20 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef SFLASH_H
 #define SFLASH_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t sflash_usage[];
 
-#ifdef USE_PARALLEL
-extern int sf_read_rom (const char *filename, unsigned int parport, int size);
-extern int sf_write_rom (const char *filename, unsigned int parport);
-extern int sf_read_sram (const char *filename, unsigned int parport);
-extern int sf_write_sram (const char *filename, unsigned int parport);
+#ifdef  USE_PARALLEL
+extern int sf_read_rom (const char *filename, unsigned short parport,
+                        unsigned int size);
+extern int sf_write_rom (const char *filename, unsigned short parport);
+extern int sf_read_sram (const char *filename, unsigned short parport);
+extern int sf_write_sram (const char *filename, unsigned short parport);
 #endif
 
 #endif
