@@ -1,7 +1,7 @@
 /*
-smsgg-pro.h - SMS-PRO/GG-PRO flash card programmer support for uCON64
+smsgg-pro.h - SMS-PRO/GG-PRO support for uCON64
 
-Copyright (c) 2004 dbjh
+Copyright (c) 2004 - 2005, 2015, 2017, 2020 dbjh
 
 Based on Delphi source code by ToToTEK Multi Media. Information in that source
 code has been used with permission. However, ToToTEK Multi Media explicitly
@@ -25,13 +25,24 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef SMSGG_PRO_H
 #define SMSGG_PRO_H
 
+#ifdef  HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
+#define SMSGG_PRO_LOADER_SIZE (48 * 1024)
+
 extern const st_getopt2_t smsggpro_usage[];
 
-#ifdef USE_PARALLEL
-extern int smsgg_read_rom (const char *filename, unsigned int parport, int size);
-extern int smsgg_write_rom (const char *filename, unsigned int parport);
-extern int smsgg_read_sram (const char *filename, unsigned int parport, int start_bank);
-extern int smsgg_write_sram (const char *filename, unsigned int parport, int start_bank);
+#ifdef  USE_PARALLEL
+extern int smsgg_read_rom (const char *filename, unsigned short parport,
+                           unsigned int size);
+extern int smsgg_write_rom (const char *filename, unsigned short parport);
+extern int smsgg_read_sram (const char *filename, unsigned short parport,
+                            int start_bank);
+extern int smsgg_write_sram (const char *filename, unsigned short parport,
+                             int start_bank);
 #endif
 
 #endif
