@@ -1,7 +1,7 @@
 /*
 ssc.c - support for Super Smart Card
 
-Copyright (c) 1999 - 2001 NoisyB <noisyb@gmx.net>
+Copyright (c) 1999 - 2001 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -18,21 +18,15 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-#ifdef  HAVE_CONFIG_H
-#include "config.h"
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
 #endif
 #include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <string.h>
-#include "misc/itypes.h"
-#include "misc/misc.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
+#ifdef  _MSC_VER
+#pragma warning(pop)
 #endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "ucon64.h"
-#include "ssc.h"
+#include "backup/ssc.h"
 
 
 const st_getopt2_t ssc_usage[] =
