@@ -1,11 +1,11 @@
 /*
 swc.c - Super Wild Card support for uCON64
 
-Copyright (c) 1999 - 2001 NoisyB <noisyb@gmx.net>
-Copyright (c) 2001 - 2004 dbjh
-Copyright (c) 2001        Caz
-Copyright (c) 2003        John Weidman
-Copyright (c) 2004        JohnDie
+Copyright (c) 1999 - 2001                          NoisyB
+Copyright (c) 2001 - 2005, 2015, 2017, 2019 - 2021 dbjh
+Copyright (c) 2001                                 Caz
+Copyright (c) 2003                                 John Weidman
+Copyright (c) 2004                                 JohnDie
 
 
 This program is free software; you can redistribute it and/or modify
@@ -25,87 +25,94 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifdef  HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <string.h>
-#include "misc/misc.h"
-#include "misc/itypes.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
 #endif
-#include "misc/getopt2.h"                       // st_getopt2_t
+#include <stdlib.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
+#include "misc/archive.h"
 #include "misc/file.h"
-#include "misc/parallel.h"
-#include "ucon64.h"
+#include "misc/misc.h"
 #include "ucon64_misc.h"
-#include "ffe.h"
-#include "swc.h"
-#include "console/snes.h"                       // for snes_get_file_type ()
+#include "console/snes.h"                       // for snes_get_copier_type()
+#include "backup/ffe.h"
+#include "backup/swc.h"
 
 
+#ifdef  USE_PARALLEL
+static st_ucon64_obj_t swc_obj[] =
+  {
+    {UCON64_SNES, WF_DEFAULT | WF_STOP | WF_NO_SPLIT | WF_NO_ROM},
+    {UCON64_SNES, WF_STOP | WF_NO_ROM},
+    {UCON64_SNES, WF_SWITCH}
+  };
+#endif
+
 const st_getopt2_t swc_usage[] =
   {
     {
       NULL, 0, 0, 0,
-      NULL, "Super Com Pro/Super Magicom/SMC/Super Wild Card (1.6XC/2.7CC/2.8CC/DX/DX2)/SWC"
+      NULL, "Supercom PRO/Super Magicom/SMC/Super Wild Card (1.6XC/2.7CC/2.8CC/DX/DX2)/SWC"
       /*"1993/1994/1995/19XX Front Far East/FFE http://www.front.com.tw"*/,
       NULL
     },
 #ifdef  USE_PARALLEL
     {
       "xswc", 0, 0, UCON64_XSWC,
-      NULL, "send/receive ROM to/from Super Wild Card*/SWC; " OPTION_LONG_S "port=PORT\n"
+      NULL, "send/receive ROM to/from Super Wild Card*/SWC; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when ROM does not exist",
-      &ucon64_wf[WF_OBJ_SNES_DEFAULT_STOP_NO_SPLIT_NO_ROM]
+      &swc_obj[0]
     },
     {
       "xswc2", 0, 0, UCON64_XSWC2,
       NULL, "same as " OPTION_LONG_S "xswc, but enables Real Time Save mode (SWC only)",
-      &ucon64_wf[WF_OBJ_SNES_DEFAULT_STOP_NO_SPLIT_NO_ROM]
+      &swc_obj[0]
     },
 #if 1
-/*
-  The following help text used to be hidden, because we wanted to avoid people
-  to "accidentally" create overdumps, bad dumps or report bugs that aren't bugs
-  (SA-1). However, now that ucon64.io_mode is useful for -xswcc I guess the
-  help should be complete. - dbjh
-*/
+    /*
+      The following help text used to be hidden, because we wanted to avoid people
+      to "accidentally" create overdumps, bad dumps or report bugs that aren't bugs
+      (SA-1). However, now that ucon64.io_mode is useful for -xswcc I guess the
+      help should be complete. - dbjh
+    */
     {
       "xswc-io", 1, 0, UCON64_XSWC_IO,
-      "MODE", "specify SWC I/O mode; use with -xswc or -xswcc\n"
-      "MODE=0x001 force 32 Mbit dump\n"
-      "MODE=0x002 use alternative method for determining ROM size\n"
-      "MODE=0x004 Super FX\n"
-      "MODE=0x008 S-DD1\n"
-      "MODE=0x010 SA-1\n"
-      "MODE=0x020 SPC7110\n"
-      "MODE=0x040 DX2 trick (might work with other SWC models)\n"
-      "MODE=0x080 Mega Man X 2\n"
-      "MODE=0x100 dump BIOS\n"
-      "It is possible to combine flags. MODE=0x44 makes it possible\n"
+      "MODE", "specify SWC I/O mode; use with " OPTION_LONG_S "xswc or " OPTION_LONG_S "xswcc\n"
+      "MODE" OPTARG_S "0x001 force 32 Mbit dump\n"
+      "MODE" OPTARG_S "0x002 use alternative method for determining ROM size\n"
+      "MODE" OPTARG_S "0x004 Super FX\n"
+      "MODE" OPTARG_S "0x008 S-DD1\n"
+      "MODE" OPTARG_S "0x010 SA-1\n"
+      "MODE" OPTARG_S "0x020 SPC7110\n"
+      "MODE" OPTARG_S "0x040 DX2 trick (might work with other SWC models)\n"
+      "MODE" OPTARG_S "0x080 Mega Man X 2\n"
+      "MODE" OPTARG_S "0x100 dump BIOS\n"
+      "It is possible to combine flags. MODE" OPTARG_S "0x44 makes it possible\n"
       "to dump for example Yoshi's Island",
-      &ucon64_wf[WF_OBJ_SNES_SWITCH]
+      &swc_obj[2]
     },
 #endif
     {
       "xswcs", 0, 0, UCON64_XSWCS,
       NULL,
-      "send/receive SRAM to/from Super Wild Card*/SWC; " OPTION_LONG_S "port=PORT\n"
+      "send/receive SRAM to/from Super Wild Card*/SWC; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when SRAM does not exist",
-      &ucon64_wf[WF_OBJ_SNES_STOP_NO_ROM]
+      &swc_obj[1]
     },
     {
       "xswcc", 0, 0, UCON64_XSWCC,
       NULL, "send/receive SRAM to/from cartridge in Super Wild Card*/SWC;\n"
-      OPTION_LONG_S "port=PORT\n" "receives automatically when SRAM does not exist",
-      &ucon64_wf[WF_OBJ_SNES_STOP_NO_ROM]
+      OPTION_LONG_S "port" OPTARG_S "PORT\n" "receives automatically when SRAM does not exist",
+      &swc_obj[1]
     },
     {
       "xswcr", 0, 0, UCON64_XSWCR,
-      NULL, "send/receive RTS data to/from Super Wild Card*/SWC; " OPTION_LONG_S "port=PORT\n"
+      NULL, "send/receive RTS data to/from Super Wild Card*/SWC; " OPTION_LONG_S "port" OPTARG_S "PORT\n"
       "receives automatically when RTS file does not exist",
-      &ucon64_wf[WF_OBJ_SNES_STOP_NO_ROM]
+      &swc_obj[1]
     },
 #endif // USE_PARALLEL
     {NULL, 0, 0, 0, NULL, NULL, NULL}
@@ -134,8 +141,8 @@ const st_getopt2_t swc_usage[] =
 #define DUMP_SDD1
 #define DUMP_SPC7110
 
-static int receive_rom_info (unsigned char *buffer, int io_mode);
-static int get_rom_size (unsigned char *info_block);
+static unsigned int receive_rom_info (unsigned char *buffer, int io_mode);
+static unsigned char get_rom_size (unsigned char *info_block);
 static int check1 (unsigned char *info_block, int index);
 static int check2 (unsigned char *info_block, int index, unsigned char value);
 static int check3 (unsigned char *info_block, int index1, int index2, int size);
@@ -143,19 +150,19 @@ static unsigned char get_emu_mode_select (unsigned char byte, int size);
 static void handle_fig_header (unsigned char *header);
 static void set_bank_and_page (unsigned char bank, unsigned char page);
 static void read_cartridge (unsigned int address, unsigned char *buffer,
-                            unsigned int length);
+                            unsigned short length);
 static unsigned char read_cartridge1 (unsigned int address);
 static void write_cartridge (unsigned int address, unsigned char *buffer,
-                             unsigned int length);
+                             unsigned short length);
 static void write_cartridge1 (unsigned int address, unsigned char byte);
 static void dump_rom (FILE *file, int size, int numblocks, unsigned int mask1,
                       unsigned int mask2, unsigned int address);
 static void dump_bios (FILE *file);
 static int sub (void);
-static int mram_helper (int x);
-static int mram (void);
+static int mram_helper (unsigned short x);
+static unsigned short mram (void);
 
-static int hirom;                               // `hirom' was `special'
+static int hirom;                               // 'hirom' was 'special'
 static int dx2_trick = 0;
 
 #ifdef  DUMP_SA1
@@ -176,18 +183,18 @@ static int snes_spc7110 = 0;
 #error receive_rom_info() and swc_read_sram() expect BUFFERSIZE to be at least \
        512 bytes.
 #endif
-int
+static unsigned int
 receive_rom_info (unsigned char *buffer, int io_mode)
 /*
   - returns size of ROM in Mb (128 kB) units
   - returns ROM header in buffer (index 2 (emulation mode select) is not yet
     filled in)
-  - sets global `hirom'
+  - sets global 'hirom'
 */
 {
-  int n, size;
+  unsigned short n;
   volatile int m;
-  unsigned char byte;
+  unsigned char byte, size;
 
 #ifdef  DUMP_MMX2
   if (io_mode & SWC_IO_MMX2)
@@ -204,15 +211,15 @@ receive_rom_info (unsigned char *buffer, int io_mode)
       unsigned short address = 0x7f52;
       ffe_send_command0 (0xe00c, 0);
 
-      ffe_send_command (5, (unsigned short) (address / 0x2000), 0);
-      ffe_receive_block ((unsigned short) ((address & 0x1fff) + 0x2000), buffer, 8);
+      ffe_send_command (5, address / 0x2000, 0);
+      ffe_receive_block ((address & 0x1fff) + 0x2000, buffer, 8);
       dumper (stdout, buffer, 8, address, DUMPER_HEX);
 
-      ffe_send_command (5, (unsigned short) (address / 0x2000), 0);
-      ffe_send_command0 ((unsigned short) ((address & 0x1fff) + 0x2000), 0);
+      ffe_send_command (5, address / 0x2000, 0);
+      ffe_send_command0 ((address & 0x1fff) + 0x2000, 0);
 
-      ffe_send_command (5, (unsigned short) (address / 0x2000), 0);
-      ffe_receive_block ((unsigned short) ((address & 0x1fff) + 0x2000), buffer, 8);
+      ffe_send_command (5, address / 0x2000, 0);
+      ffe_receive_block ((address & 0x1fff) + 0x2000, buffer, 8);
       dumper (stdout, buffer, 8, address, DUMPER_HEX);
     }
 #endif
@@ -227,11 +234,11 @@ receive_rom_info (unsigned char *buffer, int io_mode)
       hirom = ((byte & 1 && byte != 0x23) || byte == 0x3a) ? 1 : 0; // & 1 => 0x21, 0x31, 0x35
     }
 
-  for (n = 0; n < (int) SWC_HEADER_LEN; n++)
+  for (n = 0; n < SWC_HEADER_LEN; n++)
     {
       for (m = 0; m < 65536; m++)               // a delay is necessary here
         ;
-      ffe_send_command (5, (unsigned short) (0x200 + n), 0);
+      ffe_send_command (5, 0x200 + n, 0);
       buffer[n] = ffe_send_command1 (0xa0a0);
     }
 
@@ -323,7 +330,7 @@ receive_rom_info (unsigned char *buffer, int io_mode)
 }
 
 
-int
+static unsigned char
 get_rom_size (unsigned char *info_block)
 // returns size of ROM in Mb units
 {
@@ -376,7 +383,7 @@ get_rom_size (unsigned char *info_block)
 }
 
 
-int
+static int
 check1 (unsigned char *info_block, int index)
 {
   int n;
@@ -389,7 +396,7 @@ check1 (unsigned char *info_block, int index)
 }
 
 
-int
+static int
 check2 (unsigned char *info_block, int index, unsigned char value)
 {
   int n;
@@ -402,7 +409,7 @@ check2 (unsigned char *info_block, int index, unsigned char value)
 }
 
 
-int
+static int
 check3 (unsigned char *info_block, int index1, int index2, int size)
 {
   int n;
@@ -415,11 +422,10 @@ check3 (unsigned char *info_block, int index1, int index2, int size)
 }
 
 
-unsigned char
+static unsigned char
 get_emu_mode_select (unsigned char byte, int size)
 {
-  int x;
-  unsigned char ems;
+  unsigned char x, ems;
 
   if (byte == 0)
     x = 0xc;
@@ -452,7 +458,7 @@ get_emu_mode_select (unsigned char byte, int size)
 }
 
 
-void
+static void
 handle_fig_header (unsigned char *header)
 {
   if ((header[4] == 0x77 && header[5] == 0x83) ||
@@ -462,7 +468,8 @@ handle_fig_header (unsigned char *header)
   else if (header[4] == 0xfd && header[5] == 0x82)
     header[2] = 0x08;                           // 2 kB
   else if ((header[4] == 0xdd && header[5] == 0x82) ||
-           (header[4] == 0x00 && header[5] == 0x80))
+           (header[4] == 0x00 && header[5] == 0x80) ||
+           (header[4] == 0x40 && header[5] == 0x80))
     /*
       8 kB *or* 2 kB (shortcoming of FIG header format). We give the emu mode
       select byte a value as if the game uses 8 kB. At least this makes games
@@ -474,6 +481,7 @@ handle_fig_header (unsigned char *header)
     header[2] = 0x04;
   else // if ((header[4] == 0xdd && header[5] == 0x02) ||
        //     (header[4] == 0x00 && header[5] == 0x00) ||
+       //     (header[4] == 0x40 && header[5] == 0x00) ||
        //     (header[4] == 0x11 && header[5] == 0x02))
     header[2] = 0;                              // 32 kB
 
@@ -483,7 +491,7 @@ handle_fig_header (unsigned char *header)
 
 
 void
-swc_unlock (unsigned int parport)
+swc_unlock (unsigned short parport)
 /*
   "Unlock" the SWC. However, just starting to send, then stopping with ^C,
   gives the same result.
@@ -491,12 +499,11 @@ swc_unlock (unsigned int parport)
 {
   ffe_init_io (parport);
   ffe_send_command (6, 0, 0);
-  ffe_deinit_io ();
 }
 
 
 #ifdef  DUMP_SA1
-void
+static void
 set_sa1_map (unsigned short chunk)
 {
   volatile int m;
@@ -510,7 +517,7 @@ set_sa1_map (unsigned short chunk)
 
 
 #ifdef  DUMP_SDD1
-void
+static void
 set_sdd1_map (unsigned short chunk)
 {
   volatile int m;
@@ -524,7 +531,7 @@ set_sdd1_map (unsigned short chunk)
 
 
 #ifdef  DUMP_SPC7110
-void
+static void
 set_spc7110_map (unsigned short chunk)
 {
   volatile int m;
@@ -538,10 +545,10 @@ set_spc7110_map (unsigned short chunk)
 #endif
 
 
-void
+static void
 set_bank_and_page (unsigned char bank, unsigned char page)
 {
-  static unsigned char currentbank = 0, currentpage = 4; // Force update on first call
+  static unsigned char currentbank = 0, currentpage = 4; // force update on first call
 
   page &= 3;
 
@@ -573,28 +580,27 @@ set_bank_and_page (unsigned char bank, unsigned char page)
           ffe_send_command0 (0x0007, currentbank);
         }
       else
-        ffe_send_command (5, (unsigned short) ((currentbank << 2) | currentpage), 0);
+        ffe_send_command (5, (currentbank << 2) | currentpage, 0);
     }
 }
 
 
-void
-read_cartridge (unsigned int address, unsigned char *buffer, unsigned int length)
+static void
+read_cartridge (unsigned int address, unsigned char *buffer, unsigned short length)
 {
   address &= 0xffffff;
   set_bank_and_page ((unsigned char) (address >> 16),
                      (unsigned char) ((address & 0x7fff) / 0x2000));
 
   if ((address & 0x00ffff) < 0x8000)
-    ffe_receive_block ((unsigned short) (((address & 0x7fffff) < 0x400000 ?
-                         0x6000 : 0x2000) + (address & 0x001fff)), buffer, length);
+    ffe_receive_block (((address & 0x7fffff) < 0x400000 ? 0x6000 : 0x2000) +
+                       (address & 0x001fff), buffer, length);
   else
-    ffe_receive_block ((unsigned short) (0xa000 + (address & 0x001fff)),
-                       buffer, length);
+    ffe_receive_block (0xa000 + (address & 0x001fff), buffer, length);
 }
 
 
-unsigned char
+static unsigned char
 read_cartridge1 (unsigned int address)
 {
   unsigned char byte;
@@ -605,30 +611,29 @@ read_cartridge1 (unsigned int address)
 }
 
 
-void
-write_cartridge (unsigned int address, unsigned char *buffer, unsigned int length)
+static void
+write_cartridge (unsigned int address, unsigned char *buffer, unsigned short length)
 {
   address &= 0xffffff;
   set_bank_and_page ((unsigned char) (address >> 16),
                      (unsigned char) ((address & 0x7fff) / 0x2000));
 
   if ((address & 0x00ffff) < 0x8000)
-    ffe_send_block ((unsigned short) (((address & 0x7fffff) < 0x400000 ?
-                      0x6000 : 0x2000) + (address & 0x001fff)), buffer, length);
+    ffe_send_block (((address & 0x7fffff) < 0x400000 ? 0x6000 : 0x2000) +
+                    (address & 0x001fff), buffer, length);
   else
-    ffe_send_block ((unsigned short) (0xa000 + (address & 0x001fff)),
-                    buffer, length);
+    ffe_send_block (0xa000 + (address & 0x001fff), buffer, length);
 }
 
 
-void
+static void
 write_cartridge1 (unsigned int address, unsigned char byte)
 {
   write_cartridge (address, &byte, 1);
 }
 
 
-void
+static void
 dump_rom (FILE *file, int size, int numblocks, unsigned int mask1,
           unsigned int mask2, unsigned int address)
 {
@@ -681,7 +686,7 @@ dump_rom (FILE *file, int size, int numblocks, unsigned int mask1,
 }
 
 
-void
+static void
 dump_bios (FILE *file)
 {
   unsigned short int address;
@@ -712,7 +717,7 @@ dump_bios (FILE *file)
 
 
 int
-swc_read_rom (const char *filename, unsigned int parport, int io_mode)
+swc_read_rom (const char *filename, unsigned short parport, int io_mode)
 {
   FILE *file;
   unsigned char buffer[SWC_HEADER_LEN], byte;
@@ -747,7 +752,6 @@ swc_read_rom (const char *filename, unsigned int parport, int io_mode)
 
       dump_bios (file);
 
-      ffe_deinit_io ();
       fclose (file);
       return 0;                                 // skip the other code in this function
     }
@@ -761,7 +765,7 @@ swc_read_rom (const char *filename, unsigned int parport, int io_mode)
       exit (1);
     }
   blocksleft = size * 16;                       // 1 Mb (128 kB) unit == 16 8 kB units
-  printf ("Receive: %d Bytes (%.4f Mb)\n", size * MBIT, (float) size);
+  printf ("Receive: %u Bytes (%.4f Mb)\n", size * MBIT, (float) size);
 #ifdef  DUMP_SA1
   if (snes_sa1)
     puts ("NOTE: Dumping SA-1 cartridge");
@@ -824,20 +828,20 @@ swc_read_rom (const char *filename, unsigned int parport, int io_mode)
   ffe_send_command (5, 0, 0);
 
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-swc_write_rom (const char *filename, unsigned int parport, int enableRTS)
+swc_write_rom (const char *filename, unsigned short parport, unsigned short enableRTS)
 {
   FILE *file;
-  unsigned char *buffer;
-  int bytesread, bytessend, totalblocks, blocksdone = 0, emu_mode_select, fsize;
-  unsigned short address;
-  time_t starttime;
+  unsigned char *buffer, emu_mode_select;
+  size_t bytesread, bytessent;
+  unsigned int fsize;
+  unsigned short blocksdone = 0, address = 0x200; // VGS '00 uses 0x200, VGS '96 uses
+  time_t starttime;                               //  0, but then some ROMs don't work
 
   ffe_init_io (parport);
 
@@ -852,18 +856,18 @@ swc_write_rom (const char *filename, unsigned int parport, int enableRTS)
       exit (1);
     }
 
-  fsize = fsizeof (filename);
-  printf ("Send: %d Bytes (%.4f Mb)\n", fsize, (float) fsize / MBIT);
+  fsize = (unsigned int) ucon64.fsize;
+  printf ("Send: %u Bytes (%.4f Mb)\n", fsize, (float) fsize / MBIT);
 
   ffe_send_command0 (0xc008, 0);
-  fread (buffer, 1, SWC_HEADER_LEN, file);
+  fread_checked (buffer, 1, SWC_HEADER_LEN, file);
 
-  if (snes_get_file_type () == FIG)
+  if (snes_get_copier_type () == FIG)
     handle_fig_header (buffer);
 #if 1
   /*
     0x0c == no SRAM & LoROM; we use the header, so that the user can override this
-    bit 4 == 0 => DRAM mode 20 (LoROM); disable SRAM by setting SRAM mem map mode 2
+    bit 4 == 0 => DRAM mode 20 (LoROM); disable SRAM by setting SRAM mem map mode 21
   */
   if ((buffer[2] & 0x1c) == 0x0c)
     buffer[2] |= 0x20;
@@ -872,9 +876,9 @@ swc_write_rom (const char *filename, unsigned int parport, int enableRTS)
   if ((buffer[2] & 0x0c) == 0x0c)               // 0x0c == no SRAM; we use the header, so
     {                                           //  that the user can override this
       if (buffer[2] & 0x10)                     // bit 4 == 1 => DRAM mode 21 (HiROM)
-        buffer[2] &= ~0x20;                     // disable SRAM by setting SRAM mem map mode 1
+        buffer[2] &= ~0x20;                     // disable SRAM by setting SRAM mem map mode 20
       else                                      // bit 4 == 0 => DRAM mode 20 (LoROM)
-        buffer[2] |= 0x20;                      // disable SRAM by setting SRAM mem map mode 2
+        buffer[2] |= 0x20;                      // disable SRAM by setting SRAM mem map mode 21
     }
 #endif
   emu_mode_select = buffer[2];                  // this byte is needed later
@@ -883,22 +887,21 @@ swc_write_rom (const char *filename, unsigned int parport, int enableRTS)
   ffe_send_command (5, 0, 0);
   ffe_send_block (0x400, buffer, SWC_HEADER_LEN); // send header
 #endif
-  bytessend = SWC_HEADER_LEN;
+  bytessent = SWC_HEADER_LEN;
 
   puts ("Press q to abort\n");                  // print here, NOT before first SWC I/O,
                                                 //  because if we get here q works ;-)
-  address = 0x200;                              // VGS '00 uses 0x200, VGS '96 uses 0,
-  starttime = time (NULL);                      //  but then some ROMs don't work
-  while ((bytesread = fread (buffer, 1, BUFFERSIZE, file)))
+  starttime = time (NULL);
+  while ((bytesread = fread (buffer, 1, BUFFERSIZE, file)) != 0)
     {
-      ffe_send_command0 ((unsigned short) 0xc010, (unsigned char) (blocksdone >> 9));
+      ffe_send_command0 (0xc010, (unsigned char) (blocksdone >> 9));
       ffe_send_command (5, address, 0);
-      ffe_send_block (0x8000, buffer, bytesread);
+      ffe_send_block (0x8000, buffer, (unsigned short) bytesread);
       address++;
       blocksdone++;
 
-      bytessend += bytesread;
-      ucon64_gauge (starttime, bytessend, fsize);
+      bytessent += bytesread;
+      ucon64_gauge (starttime, bytessent, fsize);
       ffe_checkabort (2);
     }
 
@@ -906,26 +909,18 @@ swc_write_rom (const char *filename, unsigned int parport, int enableRTS)
     ffe_send_command0 (0xc010, 2);
 
   ffe_send_command (5, 0, 0);
-  totalblocks = (fsize - SWC_HEADER_LEN + BUFFERSIZE - 1) / BUFFERSIZE; // round up
-  ffe_send_command (6, (unsigned short) (5 | (totalblocks << 8)), (unsigned short) (totalblocks >> 8)); // bytes: 6, 5, #8 K L, #8 K H, 0
-  ffe_send_command (6, (unsigned short) (1 | (emu_mode_select << 8)), (unsigned short) enableRTS); // last arg = 1 enables RTS
+  ffe_send_command (6, 5 | (blocksdone << 8), blocksdone >> 8); // bytes: 6, 5, #8 K L, #8 K H, 0
+  ffe_send_command (6, 1 | (emu_mode_select << 8), enableRTS); // last arg = 1 enables RTS
                                                                //  mode, 0 disables it
-  ffe_wait_for_ready ();
-  outportb ((unsigned short) (parport + PARPORT_DATA), 0);
-  outportb ((unsigned short) (parport + PARPORT_CONTROL),
-            (unsigned char) (inportb ((unsigned short) // invert strobe
-                                      (parport + PARPORT_CONTROL)) ^ PARPORT_STROBE));
-
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-swc_read_sram (const char *filename, unsigned int parport)
+swc_read_sram (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer;
@@ -977,18 +972,18 @@ swc_read_sram (const char *filename, unsigned int parport)
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-swc_write_sram (const char *filename, unsigned int parport)
+swc_write_sram (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer;
-  int bytesread, bytessend = 0, size;
+  size_t bytesread, bytessent = 0;
+  unsigned int size;
   unsigned short address;
   time_t starttime;
 
@@ -1005,8 +1000,8 @@ swc_write_sram (const char *filename, unsigned int parport)
       exit (1);
     }
 
-  size = fsizeof (filename) - SWC_HEADER_LEN;   // SWC SRAM is 4*8 kB, emu SRAM often not
-  printf ("Send: %d Bytes\n", size);
+  size = (unsigned int) ucon64.fsize - SWC_HEADER_LEN; // SWC SRAM is 4*8 kB, emu SRAM often not
+  printf ("Send: %u Bytes\n", size);
   fseek (file, SWC_HEADER_LEN, SEEK_SET);       // skip the header
 
   ffe_send_command (5, 0, 0);
@@ -1017,26 +1012,25 @@ swc_write_sram (const char *filename, unsigned int parport)
                                                 //  because if we get here q works ;-)
   address = 0x100;
   starttime = time (NULL);
-  while ((bytesread = fread (buffer, 1, BUFFERSIZE, file)))
+  while ((bytesread = fread (buffer, 1, BUFFERSIZE, file)) != 0)
     {
       ffe_send_command (5, address, 0);
-      ffe_send_block (0x2000, buffer, bytesread);
+      ffe_send_block (0x2000, buffer, (unsigned short) bytesread);
       address++;
 
-      bytessend += bytesread;
-      ucon64_gauge (starttime, bytessend, size);
+      bytessent += bytesread;
+      ucon64_gauge (starttime, bytessent, size);
       ffe_checkabort (2);
     }
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
-int
+static int
 sub (void)
 {
   ffe_send_command (5, 7 * 4, 0);
@@ -1054,21 +1048,23 @@ sub (void)
 }
 
 
-int
-mram_helper (int x)
+static int
+mram_helper (unsigned short x)
 {
-  ffe_send_command (5, (unsigned short) x, 0);
-  x = ffe_send_command1 (0x8000);
-  ffe_send_command0 (0x8000, (unsigned char) (x ^ 0xff));
-  if (ffe_send_command1 (0x8000) != (unsigned char) (x ^ 0xff))
+  unsigned char y;
+
+  ffe_send_command (5, x, 0);
+  y = ffe_send_command1 (0x8000);
+  ffe_send_command0 (0x8000, y ^ 0xff);
+  if (ffe_send_command1 (0x8000) != (y ^ 0xff))
     return 0;
 
-  ffe_send_command0 (0x8000, (unsigned char) x);
+  ffe_send_command0 (0x8000, y);
   return 1;
 }
 
 
-int
+static unsigned short
 mram (void)
 {
   if (mram_helper (0x76 * 4))
@@ -1082,7 +1078,7 @@ mram (void)
 
 
 int
-swc_read_rts (const char *filename, unsigned int parport)
+swc_read_rts (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer;
@@ -1144,18 +1140,18 @@ swc_read_rts (const char *filename, unsigned int parport)
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-swc_write_rts (const char *filename, unsigned int parport)
+swc_write_rts (const char *filename, unsigned short parport)
 {
   FILE *file;
   unsigned char *buffer;
-  int bytesread, bytessend = 0, size;
+  size_t bytesread, bytessent = 0;
+  unsigned int size;
   unsigned short address1, address2;
   time_t starttime;
 
@@ -1172,8 +1168,8 @@ swc_write_rts (const char *filename, unsigned int parport)
       exit (1);
     }
 
-  size = fsizeof (filename) - SWC_HEADER_LEN;
-  printf ("Send: %d Bytes\n", size);
+  size = (unsigned int) ucon64.fsize - SWC_HEADER_LEN;
+  printf ("Send: %u Bytes\n", size);
   fseek (file, SWC_HEADER_LEN, SEEK_SET);       // skip the header
 
   puts ("Press q to abort\n");
@@ -1189,30 +1185,29 @@ swc_write_rts (const char *filename, unsigned int parport)
     }
 
   starttime = time (NULL);
-  while ((bytesread = fread (buffer, 1, BUFFERSIZE, file)))
+  while ((bytesread = fread (buffer, 1, BUFFERSIZE, file)) != 0)
     {
       ffe_send_command (5, address1, 0);
       if (address2 == 0x8000)
         ffe_send_command0 (0xc010, 1);
-      ffe_send_block (address2, buffer, bytesread);
+      ffe_send_block (address2, buffer, (unsigned short) bytesread);
       address1++;
 
-      bytessend += bytesread;
-      ucon64_gauge (starttime, bytessend, size);
+      bytessent += bytesread;
+      ucon64_gauge (starttime, bytessent, size);
       ffe_checkabort (2);
     }
   ffe_send_command (6, 3, 0);
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-swc_read_cart_sram (const char *filename, unsigned int parport, int io_mode)
+swc_read_cart_sram (const char *filename, unsigned short parport, int io_mode)
 {
   FILE *file;
   unsigned char *buffer, byte;
@@ -1269,7 +1264,7 @@ swc_read_cart_sram (const char *filename, unsigned int parport, int io_mode)
   while (bytesreceived < size)
     {
       set_bank_and_page ((unsigned char) (address >> 2), (unsigned char) (address & 3));
-      ffe_receive_block ((unsigned short) (hirom ? 0x6000 : 0x2000), buffer, BUFFERSIZE);
+      ffe_receive_block (hirom ? 0x6000 : 0x2000, buffer, BUFFERSIZE);
       fwrite (buffer, 1, BUFFERSIZE, file);
       address += hirom ? 4 : 1;
 
@@ -1280,18 +1275,18 @@ swc_read_cart_sram (const char *filename, unsigned int parport, int io_mode)
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
 
 
 int
-swc_write_cart_sram (const char *filename, unsigned int parport, int io_mode)
+swc_write_cart_sram (const char *filename, unsigned short parport, int io_mode)
 {
   FILE *file;
   unsigned char *buffer, byte;
-  int bytesread, bytessend = 0, size;
+  size_t bytesread, bytessent = 0;
+  unsigned int size;
   unsigned short address;
   time_t starttime;
 
@@ -1322,10 +1317,10 @@ swc_write_cart_sram (const char *filename, unsigned int parport, int io_mode)
   ffe_send_command (5, 3, 0);                   // detect cartridge SRAM size because we don't
   ffe_send_command0 (0xe00c, 0);                //  want to write more data than necessary
   byte = read_cartridge1 (io_mode & SWC_IO_SUPER_FX ? 0x00ffbd : 0x00ffd8);
-  size = fsizeof (filename) - SWC_HEADER_LEN;   // SWC SRAM is 4*8 kB, emu SRAM often not
-  size = MIN ((byte ? 1 << (byte + 10) : 0), size);
+  size = (unsigned int) ucon64.fsize - SWC_HEADER_LEN; // SWC SRAM is 4*8 kB, emu SRAM often not
+  size = MIN ((byte ? 1U << (byte + 10) : 0), size);
 
-  printf ("Send: %d Bytes\n", size);
+  printf ("Send: %u Bytes\n", size);
   fseek (file, SWC_HEADER_LEN, SEEK_SET);       // skip the header
 
   ffe_send_command (5, 0, 0);
@@ -1337,20 +1332,20 @@ swc_write_cart_sram (const char *filename, unsigned int parport, int io_mode)
   address = hirom ? 0x2c3 : 0x1c0;
 
   starttime = time (NULL);
-  while ((bytessend < size) && (bytesread = fread (buffer, 1, MIN (size, BUFFERSIZE), file)))
+  while (bytessent < size &&
+         (bytesread = fread (buffer, 1, MIN (size, BUFFERSIZE), file)) != 0)
     {
       set_bank_and_page ((unsigned char) (address >> 2), (unsigned char) (address & 3));
-      ffe_send_block ((unsigned short) (hirom ? 0x6000 : 0x2000), buffer, bytesread);
+      ffe_send_block (hirom ? 0x6000 : 0x2000, buffer, (unsigned short) bytesread);
       address += hirom ? 4 : 1;
 
-      bytessend += bytesread;
-      ucon64_gauge (starttime, bytessend, size);
+      bytessent += bytesread;
+      ucon64_gauge (starttime, bytessent, size);
       ffe_checkabort (2);
     }
 
   free (buffer);
   fclose (file);
-  ffe_deinit_io ();
 
   return 0;
 }
