@@ -1,7 +1,7 @@
 /*
-ufo.c - Super UFO support for uCON64
+ufo.c - Super UFO Super Drive PRO 7/8 support for uCON64
 
-Copyright (c) 2003 dbjh
+Copyright (c) 2003, 2019 dbjh
 
 
 This program is free software; you can redistribute it and/or modify
@@ -18,27 +18,22 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-#ifdef  HAVE_CONFIG_H
-#include "config.h"
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
 #endif
 #include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <string.h>
-#include "misc/itypes.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
+#ifdef  _MSC_VER
+#pragma warning(pop)
 #endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "ucon64.h"
-#include "ufo.h"
+#include "backup/ufo.h"
 
 
 const st_getopt2_t ufo_usage[] =
   {
     {
       NULL, 0, 0, 0,
-      NULL, "Super UFO Pro 8"/*"UFO Enterprises"*/,
+      NULL, "Super UFO Super Drive PRO 7/8"/*"UFO Enterprises"*/,
       NULL
     },
     {NULL, 0, 0, 0, NULL, NULL, NULL}
