@@ -1,7 +1,7 @@
 /*
 dc.c - Dreamcast support for uCON64
 
-Copyright (c) 2004 NoisyB <noisyb@gmx.net>
+Copyright (c) 2004 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -21,25 +21,33 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifdef  HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdio.h>
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
+#pragma warning(disable: 4820) // 'bytes' bytes padding added after construct 'member_name'
+#include <io.h>                                 // access()
+#pragma warning(pop)
+#define F_OK 00
+#endif
 #include <stdlib.h>
-#include <string.h>
 #ifdef  HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include "misc/archive.h"
 #include "misc/file.h"
-#include "misc/misc.h"
 #include "misc/property.h"
 #include "misc/string.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
-#endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "ucon64.h"
 #include "ucon64_misc.h"
-#include "dc.h"
+#include "console/dc.h"
 
 
+static st_ucon64_obj_t dc_obj[] =
+  {
+    {UCON64_DC, WF_SWITCH},
+    {UCON64_DC, WF_DEFAULT},
+    {UCON64_DC, WF_NO_ROM}
+  };
+
 const st_getopt2_t dc_usage[] =
   {
     {
@@ -48,9 +56,9 @@ const st_getopt2_t dc_usage[] =
       NULL
     },
     {
-      "dc", 0, 0, UCON64_DC,
+      UCON64_DC_S, 0, 0, UCON64_DC,
       NULL, "force recognition",
-      &ucon64_wf[WF_OBJ_DC_SWITCH]
+      &dc_obj[0]
     },
 #if 0
     {
@@ -62,12 +70,12 @@ const st_getopt2_t dc_usage[] =
     {
       "scr", 0, 0, UCON64_SCR,
       NULL, "scramble 1ST_READ.BIN for selfboot CDs",
-      &ucon64_wf[WF_OBJ_DC_DEFAULT]
+      &dc_obj[1]
     },
     {
       "unscr", 0, 0, UCON64_UNSCR,
       NULL, "unscramble 1ST_READ.BIN for non-selfboot CDs",
-      &ucon64_wf[WF_OBJ_DC_DEFAULT]
+      &dc_obj[1]
     },
 #if 0
     {
@@ -79,13 +87,13 @@ const st_getopt2_t dc_usage[] =
     {
       "mkip", 0, 0, UCON64_MKIP,
       NULL, "generate IP.BIN file with default values",
-      &ucon64_wf[WF_OBJ_DC_NO_ROM]
+      &dc_obj[2]
     },
     {
       "parse", 1, 0, UCON64_PARSE,
       "TEMPLATE", "parse TEMPLATE file into a IP.BIN;\n"
       "creates an empty template when TEMPLATE does not exist",
-      &ucon64_wf[WF_OBJ_DC_NO_ROM]
+      &dc_obj[2]
     },
     {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
@@ -134,7 +142,7 @@ dc_srand (unsigned int n)
 
 
 static unsigned int
-dc_rand ()
+dc_rand (void)
 {
   seed = (seed * 2109 + 9273) & 0x7fff;
   return (seed + 0xc000) & 0xffff;
@@ -277,7 +285,7 @@ static const uint8_t nstrsave_bin[1024] = {
 
 
 int
-dc_init (st_rominfo_t *rominfo)
+dc_init (st_ucon64_nfo_t *rominfo)
 {
   int result = -1;
 
@@ -355,13 +363,17 @@ parse_templ (const char *templ_file, char *ip)
   static char buf[MAXBUFSIZE];
   int i;
 
-  memset (filled_in, 0, sizeof (filled_in));
+  memset (filled_in, 0, sizeof filled_in);
   for (i = 0; templ[i].name; i++)
     {
-      char *p = buf;
-      get_property (templ_file, templ[i].name, p, templ[i].def);
+      char *p = (char *) get_property (templ_file, templ[i].name, PROPERTY_MODE_TEXT);
+      size_t len;
 
-      strtriml (strtrimr (p));
+      if (!p)
+        p = templ[i].def;
+      len = strnlen (p, sizeof buf - 1);
+      strncpy (buf, p, len)[len] = '\0';
+      p = strtriml (strtrimr (buf));
 
       if (!(*p))
         continue;
@@ -372,7 +384,7 @@ parse_templ (const char *templ_file, char *ip)
         {
           fprintf (stderr, "ERROR: Data for field \"%s\" is too long... stripping to %d chars\n",
                    templ[i].name, templ[i].len);
-          p[templ[i].len] = 0;
+          p[templ[i].len] = '\0';
         }
 
       memcpy (ip + templ[i].pos, p, strlen (p));
@@ -398,7 +410,7 @@ parse_templ (const char *templ_file, char *ip)
 int
 dc_parse (const char *templ_file)
 {
-  char ip[0x8000], dest_name[FILENAME_MAX];
+  char ip[0x8000] = { 0 }, dest_name[FILENAME_MAX];
 
   if (access (templ_file, F_OK) == -1)
     {
@@ -555,27 +567,35 @@ save_file (FILE * fh, unsigned char *ptr, uint32_t filesz)
 
 
 static int
-descramble (const char *src, char *dst)
+descramble (const char *src, uint32_t sz, char *dst)
 {
   unsigned char *ptr = NULL;
-  uint32_t sz = 0;
   FILE *fh;
 
-  if (!(fh = fopen (src, "rb")))
+  if ((fh = fopen (src, "rb")) == NULL)
     return -1;
 
-  sz = fsizeof (src);
-  if (!(ptr = (unsigned char *) malloc (sz)))
-    return -1;
+  if ((ptr = (unsigned char *) malloc (sz)) == NULL)
+    {
+      fclose (fh);
+      return -1;
+    }
 
   load_file (fh, ptr, sz);
   fclose (fh);
 
-  if (!(fh = fopen (dst, "wb")))
-    return -1;
+  if ((fh = fopen (dst, "wb")) == NULL)
+    {
+      free (ptr);
+      return -1;
+    }
 
   if (fwrite (ptr, 1, sz, fh) != sz)
-    return -1;
+    {
+      fclose (fh);
+      free (ptr);
+      return -1;
+    }
 
   fclose (fh);
   free (ptr);
@@ -584,31 +604,37 @@ descramble (const char *src, char *dst)
 
 
 static int
-scramble (const char *src, char *dst)
+scramble (const char *src, uint32_t sz, char *dst)
 {
   unsigned char *ptr = NULL;
-  uint32_t sz = 0;
   FILE *fh;
 
-  if (!(fh = fopen (src, "rb")))
+  if ((fh = fopen (src, "rb")) == NULL)
     return -1;
 
-  sz = fsizeof (src);
-
-  if (!(ptr = (unsigned char *) malloc (sz)))
-    return -1;
+  if ((ptr = (unsigned char *) malloc (sz)) == NULL)
+    {
+      fclose (fh);
+      return -1;
+    }
 
   if (fread (ptr, 1, sz, fh) != sz)
-    return -1;
+    {
+      free (ptr);
+      fclose (fh);
+      return -1;
+    }
 
   fclose (fh);
 
-  if (!(fh == fopen (dst, "wb")))
-    return -1;
+  if ((fh = fopen (dst, "wb")) == NULL)
+    {
+      free (ptr);
+      return -1;
+    }
   save_file (fh, ptr, sz);
 
   fclose (fh);
-
   free (ptr);
   return 0;
 }
@@ -619,10 +645,10 @@ dc_scramble (void)
 {
   char dest_name[FILENAME_MAX];
 
-  strcpy (dest_name, ucon64.rom);
+  strcpy (dest_name, ucon64.fname);
   ucon64_file_handler (dest_name, NULL, 0);
 
-  if (!scramble (ucon64.rom, dest_name))
+  if (!scramble (ucon64.fname, (uint32_t) ucon64.fsize, dest_name))
     printf (ucon64_msg[WROTE], dest_name);
   else
     fprintf (stderr, ucon64_msg[WRITE_ERROR], dest_name);
@@ -635,10 +661,10 @@ dc_unscramble (void)
 {
   char dest_name[FILENAME_MAX];
 
-  strcpy (dest_name, ucon64.rom);
+  strcpy (dest_name, ucon64.fname);
   ucon64_file_handler (dest_name, NULL, 0);
 
-  if (!descramble (ucon64.rom, dest_name))
+  if (!descramble (ucon64.fname, (uint32_t) ucon64.fsize, dest_name))
     printf (ucon64_msg[WROTE], dest_name);
   else
     fprintf (stderr, ucon64_msg[WRITE_ERROR], dest_name);
