@@ -1,7 +1,7 @@
 /*
 dc.h - Dreamcast support for uCON64
 
-Copyright (c) 2004 NoisyB <noisyb@gmx.net>
+Copyright (c) 2004 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -20,10 +20,17 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #ifndef DC_H
 #define DC_H
-extern int dc_init (st_rominfo_t *rominfo);
+
+#include "misc/getopt2.h"                       // st_getopt2_t
+#include "ucon64.h"
+
+
 extern const st_getopt2_t dc_usage[];
+
+extern int dc_init (st_ucon64_nfo_t *rominfo);
 extern int dc_parse (const char *template_file);
 extern int dc_mkip (void);
 extern int dc_scramble (void);
 extern int dc_unscramble (void);
+
 #endif
