@@ -1,8 +1,8 @@
 /*
 gba.c - Game Boy Advance support for uCON64
 
-Copyright (c) 2001        NoisyB <noisyb@gmx.net>
-Copyright (c) 2001 - 2004 dbjh
+Copyright (c) 2001 - 2005              NoisyB
+Copyright (c) 2001 - 2005, 2015 - 2021 dbjh
 
 
 This program is free software; you can redistribute it and/or modify
@@ -22,78 +22,111 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifdef  HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdio.h>
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
+#pragma warning(disable: 4820) // 'bytes' bytes padding added after construct 'member_name'
+#include <io.h>
+#pragma warning(pop)
+#endif
 #include <stdlib.h>
-#include <string.h>
 #ifdef  HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4820) // 'bytes' bytes padding added after construct 'member_name'
+#endif
 #include <sys/stat.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
+#include "misc/archive.h"
+#include "misc/bswap.h"
 #include "misc/file.h"
 #include "misc/misc.h"
 #include "misc/property.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
-#endif
-#include "misc/getopt2.h"                       // st_getopt2_t
 #include "misc/string.h"
-#include "ucon64.h"
 #include "ucon64_misc.h"
-#include "gba.h"
+#include "console/console.h"
+#include "console/gba.h"
+#include "backup/backup.h"
 #include "backup/fal.h"
 
 
 #define GBA_NAME_LEN 12
 #define GBA_HEADER_START 0
 #define GBA_HEADER_LEN (sizeof (st_gba_header_t))
-
+#define GBA_MENU_SIZE 20916
+#define GBA_BLANK_SIZE 52232
+#define GBA_SAV_TEMPLATE_SIZE 65536
+#define GBA_SCI_TEMPLATE_SIZE 458752
 
 static int gba_chksum (void);
 
+
+static st_ucon64_obj_t gba_obj[] =
+  {
+    {0, WF_DEFAULT},
+    {0, WF_INIT | WF_PROBE | WF_STOP},
+    {UCON64_GBA, WF_SWITCH},
+    {UCON64_GBA, WF_DEFAULT}
+  };
+
 const st_getopt2_t gba_usage[] =
   {
     {
       NULL, 0, 0, 0,
-      NULL, "Game Boy Advance"/*"2001 Nintendo http://www.nintendo.com"*/,
+      NULL, "Game Boy Advance (SP)"/*"2001 Nintendo http://www.nintendo.com"*/,
       NULL
     },
     {
-      "gba", 0, 0, UCON64_GBA,
+      UCON64_GBA_S, 0, 0, UCON64_GBA,
       NULL, "force recognition",
-      &ucon64_wf[WF_OBJ_GBA_SWITCH]
+      &gba_obj[2]
     },
     {
       "n", 1, 0, UCON64_N,
       "NEW_NAME", "change internal ROM name to NEW_NAME",
-      &ucon64_wf[WF_OBJ_ALL_DEFAULT]
+      &gba_obj[0]
     },
     {
       "logo", 0, 0, UCON64_LOGO,
       NULL, "restore ROM logo character data (offset: 0x04-0x9F)",
-      &ucon64_wf[WF_OBJ_ALL_DEFAULT]
+      &gba_obj[0]
     },
     {
       "chk", 0, 0, UCON64_CHK,
       NULL, "fix ROM header checksum",
-      &ucon64_wf[WF_OBJ_ALL_DEFAULT]
+      &gba_obj[0]
     },
     {
       "sram", 0, 0, UCON64_SRAM,
       NULL, "patch ROM for SRAM saving",
-      &ucon64_wf[WF_OBJ_GBA_DEFAULT]
+      &gba_obj[3]
+    },
+    {
+      "sc", 0, 0, UCON64_SC,
+      NULL, "convert to Super Card (CF to GBA Adapter)\n"
+#if 0
+            "enables \"Saver patch\", \"restart to Menu\" and\n"
+            "\"Real Time Save\""
+#endif
+            "(creates SAV and SCI templates)",
+      &gba_obj[3]
     },
     {
       "crp", 1, 0, UCON64_CRP,
       "WAIT_TIME", "slow down ROM access (\"crash patch\");\n"
-      "WAIT_TIME=0  default in most crash patches\n"
-      "WAIT_TIME=4  faster than 0, slower than 8\n"
-      "WAIT_TIME=8  faster than 4, slower than 28\n"
-      "WAIT_TIME=12 slowest cartridge access speed\n"
-      "WAIT_TIME=16 faster than 28, but slower than 20\n"
-      "WAIT_TIME=20 default in most original cartridges\n"
-      "WAIT_TIME=24 fastest cartridge access speed\n"
-      "WAIT_TIME=28 faster than 8 but slower than 16",
-      &ucon64_wf[WF_OBJ_GBA_DEFAULT]
+      "WAIT_TIME" OPTARG_S "0  default in most crash patches\n"
+      "WAIT_TIME" OPTARG_S "4  faster than 0, slower than 8\n"
+      "WAIT_TIME" OPTARG_S "8  faster than 4, slower than 28\n"
+      "WAIT_TIME" OPTARG_S "12 slowest cartridge access speed\n"
+      "WAIT_TIME" OPTARG_S "16 faster than 28, but slower than 20\n"
+      "WAIT_TIME" OPTARG_S "20 default in most original cartridges\n"
+      "WAIT_TIME" OPTARG_S "24 fastest cartridge access speed\n"
+      "WAIT_TIME" OPTARG_S "28 faster than 8 but slower than 16",
+      &gba_obj[3]
     },
 //  "n 0 and 28, with a stepping of 4. I.e. 0, 4, 8, 12 ...\n"
     {
@@ -101,14 +134,13 @@ const st_getopt2_t gba_usage[] =
       "SIZE", "make multi-game file for use with FAL/F2A flash card, truncated\n"
       "to SIZE Mbit; file with loader must be specified first, then\n"
       "all the ROMs, multi-game file to create last",
-      &ucon64_wf[WF_OBJ_ALL_INIT_PROBE_STOP]
+      &gba_obj[1]
     },
     {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
 
-
 /*
-Offset 00h-03h - Start  Address - A 32 bit ARM B command with jump destination
+Offset 00h-03h - Start  address - A 32 bit ARM B command with jump destination
                  to  the  start  address of the program, cannot be manipulated
                  with this tool, there's no reason.
 
@@ -215,7 +247,7 @@ typedef struct st_gba_header
 } st_gba_header_t;
 
 static st_gba_header_t gba_header;
-const unsigned char gba_logodata[] =            // Note: not a static variable
+const unsigned char gba_logodata[] =            // NOTE: not a static variable
   {
                             0x24, 0xff, 0xae, 0x51,
     0x69, 0x9a, 0xa2, 0x21, 0x3d, 0x84, 0x82, 0x0a,
@@ -241,17 +273,23 @@ const unsigned char gba_logodata[] =            // Note: not a static variable
 
 
 int
-gba_n (st_rominfo_t *rominfo, const char *name)
+gba_n (st_ucon64_nfo_t *rominfo, const char *name)
 {
   char buf[GBA_NAME_LEN], dest_name[FILENAME_MAX];
 
-  memset (buf, 0, GBA_NAME_LEN);
+#if     defined __GNUC__ && __GNUC__ >= 8
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#endif
   strncpy (buf, name, GBA_NAME_LEN);
-  strcpy (dest_name, ucon64.rom);
+#if     defined __GNUC__ && __GNUC__ >= 8
+#pragma GCC diagnostic pop
+#endif
+  strcpy (dest_name, ucon64.fname);
   ucon64_file_handler (dest_name, NULL, 0);
-  fcopy (ucon64.rom, 0, ucon64.file_size, dest_name, "wb");
-  ucon64_fwrite (buf, GBA_HEADER_START + rominfo->buheader_len + 0xa0, GBA_NAME_LEN,
-            dest_name, "r+b");
+  fcopy (ucon64.fname, 0, ucon64.fsize, dest_name, "wb");
+  ucon64_fwrite (buf, GBA_HEADER_START + rominfo->backup_header_len + 0xa0,
+                 GBA_NAME_LEN, dest_name, "r+b");
 
   printf (ucon64_msg[WROTE], dest_name);
   return 0;
@@ -259,15 +297,15 @@ gba_n (st_rominfo_t *rominfo, const char *name)
 
 
 int
-gba_logo (st_rominfo_t *rominfo)
+gba_logo (st_ucon64_nfo_t *rominfo)
 {
   char dest_name[FILENAME_MAX];
 
-  strcpy (dest_name, ucon64.rom);
+  strcpy (dest_name, ucon64.fname);
   ucon64_file_handler (dest_name, NULL, 0);
-  fcopy (ucon64.rom, 0, ucon64.file_size, dest_name, "wb");
-  ucon64_fwrite (gba_logodata, GBA_HEADER_START + rominfo->buheader_len + 0x04,
-            GBA_LOGODATA_LEN, dest_name, "r+b");
+  fcopy (ucon64.fname, 0, ucon64.fsize, dest_name, "wb");
+  ucon64_fwrite (gba_logodata, GBA_HEADER_START + rominfo->backup_header_len +
+                 0x04, GBA_LOGODATA_LEN, dest_name, "r+b");
 
   printf (ucon64_msg[WROTE], dest_name);
   return 0;
@@ -275,19 +313,20 @@ gba_logo (st_rominfo_t *rominfo)
 
 
 int
-gba_chk (st_rominfo_t *rominfo)
+gba_chk (st_ucon64_nfo_t *rominfo)
 {
   char buf, dest_name[FILENAME_MAX];
 
-  strcpy (dest_name, ucon64.rom);
+  strcpy (dest_name, ucon64.fname);
   ucon64_file_handler (dest_name, NULL, 0);
-  fcopy (ucon64.rom, 0, ucon64.file_size, dest_name, "wb");
+  fcopy (ucon64.fname, 0, ucon64.fsize, dest_name, "wb");
 
-  buf = rominfo->current_internal_crc;
-  ucon64_fputc (dest_name, GBA_HEADER_START + rominfo->buheader_len + 0xbd,
-    buf, "r+b");
+  buf = (char) rominfo->current_internal_crc;
+  ucon64_fputc (dest_name, GBA_HEADER_START + rominfo->backup_header_len + 0xbd,
+                buf, "r+b");
 
-  dumper (stdout, &buf, 1, GBA_HEADER_START + rominfo->buheader_len + 0xbd, DUMPER_HEX);
+  dumper (stdout, &buf, 1, GBA_HEADER_START + rominfo->backup_header_len + 0xbd,
+          DUMPER_HEX);
 
   printf (ucon64_msg[WROTE], dest_name);
   return 0;
@@ -380,19 +419,19 @@ gba_sram (void)
     major, minor, micro, *buffer, *bufferptr, *ptr, value;
   char dest_name[FILENAME_MAX];
   int p_size[2] = { 188, 168 }, p_off, st_off;
-  unsigned int fsize = ucon64.file_size;
+  unsigned int fsize = (unsigned int) ucon64.fsize;
   FILE *destfile;
 
-  strcpy (dest_name, ucon64.rom);
+  strcpy (dest_name, ucon64.fname);
   ucon64_file_handler (dest_name, NULL, 0);
-  fcopy (ucon64.rom, 0, ucon64.file_size, dest_name, "wb");
+  fcopy (ucon64.fname, 0, ucon64.fsize, dest_name, "wb");
 
-  if ((destfile = fopen (dest_name, "rb+")) == NULL)
+  if ((destfile = fopen (dest_name, "r+b")) == NULL)
     {
       fprintf (stderr, ucon64_msg[OPEN_WRITE_ERROR], dest_name);
       return -1;
     }
-  if (!(buffer = (unsigned char *) malloc (fsize)))
+  if ((buffer = (unsigned char *) malloc (fsize)) == NULL)
     {
       fprintf (stderr, ucon64_msg[ROM_BUFFER_ERROR], fsize);
       fclose (destfile);
@@ -409,9 +448,9 @@ gba_sram (void)
   bufferptr = buffer + 160 + 12 + 4;
 
   ptr = (unsigned char *) memmem2 (bufferptr, fsize, "EEPROM_", 7, 0);
-  if (ptr == 0)
+  if (ptr == NULL)
     {
-      printf ("This ROM does not appear to use EEPROM saving\n");
+      puts ("This ROM does not appear to use EEPROM saving");
       free (buffer);
       fclose (destfile);
       return -1;
@@ -420,19 +459,19 @@ gba_sram (void)
   minor = ptr[9] - '0';
   micro = ptr[10] - '0';
   if (ucon64.quiet < 0)
-    printf ("version: %d.%d.%d; offset: 0x%08x\n",
+    printf ("version: %u.%u.%u; offset: 0x%08x\n",
             major, minor, micro, (int) (ptr - buffer));
-  if (minor > 2)
+  if (minor - 1 >= 2)
     {
-      fputs ("ERROR: ROMs with an EEPROM minor version higher than 2 are not supported\n", stderr);
+      fputs ("ERROR: ROMs with an EEPROM minor version other than 1 or 2 are not supported\n", stderr);
       free (buffer);
       fclose (destfile);
       return -1;
     }
 
   ptr = (unsigned char *) memmem2 (bufferptr, fsize,
-                                   fl_orig[minor - 1], sizeof (fl_orig[minor - 1]), 0);
-  if (ptr == 0)
+                                   fl_orig[minor - 1], sizeof fl_orig[minor - 1], 0);
+  if (ptr == NULL)
     {
       fputs ("ERROR: Could not find fl pattern. Perhaps this file is already patched?\n", stderr);
       free (buffer);
@@ -441,8 +480,8 @@ gba_sram (void)
     }
   if (ucon64.quiet < 0)
     printf ("fl offset: 0x%08x\n", (int) (ptr - buffer));
-  fseek (destfile, ptr - buffer, SEEK_SET);
-  fwrite (fl_repl[minor - 1], 1, sizeof (fl_repl[minor - 1]), destfile);
+  fseek (destfile, (long) (ptr - buffer), SEEK_SET);
+  fwrite (fl_repl[minor - 1], 1, sizeof fl_repl[minor - 1], destfile);
 
   ptr = buffer + fsize - 1;
   value = *ptr;
@@ -464,15 +503,15 @@ gba_sram (void)
     }
 
   ptr = (unsigned char *) memmem2 (bufferptr, fsize,
-                                   st_orig[minor - 1], sizeof (st_orig[minor - 1]), 0);
-  if (ptr == 0)
+                                   st_orig[minor - 1], sizeof st_orig[minor - 1], 0);
+  if (ptr == NULL)
     {
       fputs ("ERROR: Could not find st pattern\n", stderr);
       free (buffer);
       fclose (destfile);
       return -1;
     }
-  st_off = ptr - buffer;
+  st_off = (int) (ptr - buffer);
   if (ucon64.quiet < 0)
     printf ("st offset: 0x%08x\n", st_off);
 
@@ -509,7 +548,7 @@ gba_sram (void)
       break;
     }
   fseek (destfile, st_off, SEEK_SET);
-  fwrite (st_repl[minor - 1], 1, sizeof (st_repl[minor - 1]), destfile);
+  fwrite (st_repl[minor - 1], 1, sizeof st_repl[minor - 1], destfile);
   fseek (destfile, p_off, SEEK_SET);
   fwrite (p_repl[minor - 1], 1, p_size[minor - 1], destfile);
 
@@ -523,23 +562,24 @@ gba_sram (void)
 
 
 int
-gba_crp (st_rominfo_t *rominfo, const char *value)
+gba_crp (st_ucon64_nfo_t *rominfo, const char *value)
 {
   FILE *srcfile, *destfile;
-  int bytesread, n = 0;
+  size_t bytesread;
+  int n = 0;
   char buffer[32 * 1024], src_name[FILENAME_MAX], dest_name[FILENAME_MAX],
-       replace[2], wait_time = atoi (value);
+       replace[2], wait_time = (char) atoi (value);
 
   if (wait_time % 4 != 0 || wait_time > 28 || wait_time < 0)
     {
-      fprintf (stderr, "ERROR: You specified an incorrect WAIT_TIME value\n");
+      fputs ("ERROR: You specified an incorrect WAIT_TIME value\n", stderr);
       return -1;
     }
 
   puts ("Applying crash patch...");
 
-  strcpy (src_name, ucon64.rom);
-  strcpy (dest_name, ucon64.rom);
+  strcpy (src_name, ucon64.fname);
+  strcpy (dest_name, ucon64.fname);
   ucon64_file_handler (dest_name, src_name, 0);
   if ((srcfile = fopen (src_name, "rb")) == NULL)
     {
@@ -549,18 +589,19 @@ gba_crp (st_rominfo_t *rominfo, const char *value)
   if ((destfile = fopen (dest_name, "wb")) == NULL)
     {
       fprintf (stderr, ucon64_msg[OPEN_WRITE_ERROR], dest_name);
+      fclose (srcfile);
       return -1;
     }
-  if (rominfo->buheader_len)                    // copy header (if present)
+  if (rominfo->backup_header_len)               // copy header (if present)
     {
-      fread (buffer, 1, rominfo->buheader_len, srcfile);
-      fwrite (buffer, 1, rominfo->buheader_len, destfile);
+      fread_checked (buffer, 1, rominfo->backup_header_len, srcfile);
+      fwrite (buffer, 1, rominfo->backup_header_len, destfile);
     }
 
   replace[0] = wait_time;
   replace[1] = 0x40;
-  while ((bytesread = fread (buffer, 1, 32 * 1024, srcfile)))
-    {                           // '!' == ASCII 33 (\x21), '*' == 42 (\x2a)
+  while ((bytesread = fread (buffer, 1, 32 * 1024, srcfile)) != 0)
+    {                            // '!' == ASCII 33 (\x21), '*' == 42 (\x2a)
       n += change_mem (buffer, bytesread, "\x04\x02\x00\x04\x14\x40", 6, '*', '!', replace, 1, -1);
       n += change_mem (buffer, bytesread, "\x02\x00\x04\x14\x40\x00", 6, '*', '!', replace, 1, -2);
       n += change_mem (buffer, bytesread, "\x04\x02\x00\x04\xB4\x45", 6, '*', '!', replace, 2, -1);
@@ -580,27 +621,27 @@ gba_crp (st_rominfo_t *rominfo, const char *value)
 
 
 int
-gba_init (st_rominfo_t *rominfo)
+gba_init (st_ucon64_nfo_t *rominfo)
 {
   int result = -1, value;
-  char buf[MAXBUFSIZE];
+  unsigned int pos = (unsigned int) strlen (rominfo->misc);
 
-  rominfo->buheader_len = UCON64_ISSET (ucon64.buheader_len) ?
-    ucon64.buheader_len : 0;
+  rominfo->backup_header_len = UCON64_ISSET2 (ucon64.backup_header_len, unsigned int) ?
+                                 ucon64.backup_header_len : 0;
 
   ucon64_fread (&gba_header, GBA_HEADER_START +
-           rominfo->buheader_len, GBA_HEADER_LEN, ucon64.rom);
+                rominfo->backup_header_len, GBA_HEADER_LEN, ucon64.fname);
   if (/*gba_header.game_id_prefix == 'A' && */ // 'B' in Mario vs. Donkey Kong
       gba_header.start[3] == 0xea && gba_header.pad1 == 0x96 && gba_header.gba_type == 0)
     result = 0;
   else
     {
 #if 0 // AFAIK (dbjh) GBA ROMs never have a header
-      rominfo->buheader_len = UCON64_ISSET (ucon64.buheader_len) ?
-        ucon64.buheader_len : UNKNOWN_HEADER_LEN;
+      rominfo->backup_header_len = UCON64_ISSET (ucon64.backup_header_len, unsigned int) ?
+                                     ucon64.backup_header_len : UNKNOWN_HEADER_LEN;
 
       ucon64_fread (&gba_header, GBA_HEADER_START +
-               rominfo->buheader_len, GBA_HEADER_LEN, ucon64.rom);
+                    rominfo->backup_header_len, GBA_HEADER_LEN, ucon64.fname);
       if (gba_header.game_id_prefix == 'A' && gba_header.gba_type == 0)
         result = 0;
       else
@@ -615,8 +656,8 @@ gba_init (st_rominfo_t *rominfo)
   rominfo->header = &gba_header;
 
   // internal ROM name
-  strncpy (rominfo->name, (const char *) gba_header.name, GBA_NAME_LEN);
-  rominfo->name[GBA_NAME_LEN] = 0;
+  strncpy (rominfo->name, (char *) gba_header.name, GBA_NAME_LEN);
+  rominfo->name[GBA_NAME_LEN] = '\0';
 
   // ROM maker
   {
@@ -638,11 +679,8 @@ gba_init (st_rominfo_t *rominfo)
     "Unknown country";
 
   // misc stuff
-  sprintf (buf, "Version: %d\n", gba_header.version);
-  strcat (rominfo->misc, buf);
-
-  sprintf (buf, "Device type: 0x%02x\n", gba_header.device_type);
-  strcat (rominfo->misc, buf);
+  pos += sprintf (rominfo->misc + pos, "Version: 1.%u\n", gba_header.version);
+  pos += sprintf (rominfo->misc + pos, "Device type: 0x%02x\n", gba_header.device_type);
 
   /*
     start address = current address + (parameter of B instruction * 4) + 8
@@ -650,28 +688,16 @@ gba_init (st_rominfo_t *rominfo)
   */
   value = 0x8000008 +
           (gba_header.start[2] << 18 | gba_header.start[1] << 10 | gba_header.start[0] << 2);
-  sprintf (buf, "Start address: 0x%08x\n", value);
-  strcat (rominfo->misc, buf);
+  pos += sprintf (rominfo->misc + pos, "Start address: 0x%08x\n", value);
 
-  strcat (rominfo->misc, "Logo data: ");
-  if (memcmp (gba_header.logo, gba_logodata, GBA_LOGODATA_LEN) == 0)
-    {
+  sprintf (rominfo->misc + pos, "Logo data: %s",
+           memcmp (gba_header.logo, gba_logodata, GBA_LOGODATA_LEN) == 0 ?
 #ifdef  USE_ANSI_COLOR
-      if (ucon64.ansi_color)
-        strcat (rominfo->misc, "\x1b[01;32mOk\x1b[0m");
-      else
+             ucon64.ansi_color ? "\x1b[01;32mOK\x1b[0m" : "OK" :
+             ucon64.ansi_color ? "\x1b[01;31mBad\x1b[0m" : "Bad");
+#else
+             "OK" : "Bad");
 #endif
-        strcat (rominfo->misc, "Ok");
-    }
-  else
-    {
-#ifdef  USE_ANSI_COLOR
-      if (ucon64.ansi_color)
-        strcat (rominfo->misc, "\x1b[01;31mBad\x1b[0m");
-      else
-#endif
-        strcat (rominfo->misc, "Bad");
-    }
 
   // internal ROM crc
   if (!UCON64_ISSET (ucon64.do_not_calc_crc) && result == 0)
@@ -685,8 +711,8 @@ gba_init (st_rominfo_t *rominfo)
     }
 
   rominfo->console_usage = gba_usage[0].help;
-  // We use fal_usage, but we could just as well use f2a_usage
-  rominfo->copier_usage = (!rominfo->buheader_len ? fal_usage[0].help : unknown_usage[0].help);
+  // we use fal_usage, but we could just as well use f2a_usage
+  rominfo->backup_usage = (!rominfo->backup_header_len ? fal_usage[0].help : unknown_backup_usage[0].help);
 
   return result;
 }
@@ -707,19 +733,20 @@ gba_chksum (void)
 
 
 int
-gba_multi (int truncate_size, char *multi_fname)
+gba_multi (unsigned int truncate_size, char *multi_fname)
 // TODO: Check if 1024 Mbit multi-game files are supported by the FAL code
 {
-#define BUFSIZE (32 * 1024)
-  int n, n_files, file_no, bytestowrite, byteswritten, totalsize = 0, done,
-      truncated = 0, size_pow2_lesser = 1, size_pow2 = 1, truncate_size_ispow2 = 0;
+  size_t n, n_files, bytestowrite, byteswritten, totalsize = 0;
+  unsigned int file_no, done, truncated = 0, size_pow2_lesser = 1,
+               size_pow2 = 1, truncate_size_ispow2 = 0;
   struct stat fstate;
   FILE *srcfile, *destfile;
-  char buffer[BUFSIZE], fname[FILENAME_MAX], *fname_ptr;
+  char buffer[32 * 1024], fname[FILENAME_MAX], *fname_ptr;
+  const char *p = NULL;
 
   if (truncate_size == 0)
     {
-      fprintf (stderr, "ERROR: Can't make multi-game file of 0 bytes\n");
+      fputs ("ERROR: Cannot make multi-game file of 0 bytes\n", stderr);
       return -1;
     }
 
@@ -728,21 +755,22 @@ gba_multi (int truncate_size, char *multi_fname)
       truncate_size != 256 * MBIT && truncate_size != 512 * MBIT &&
       truncate_size != 1024 * MBIT)
     {
-      fprintf (stderr, "ERROR: Truncate size must be 64, 128, 256, 512 or 1024\n");
+      fputs ("ERROR: Truncate size must be 64, 128, 256, 512 or 1024\n", stderr);
       return -1;
     }
 #endif
 
   if (multi_fname != NULL)                      // -xfalmulti
     {
-      strcpy (fname, multi_fname);
       n_files = ucon64.argc;
+      snprintf (fname, FILENAME_MAX, "%s", multi_fname);
     }
   else                                          // -multi
     {
-      strcpy (fname, ucon64.argv[ucon64.argc - 1]);
       n_files = ucon64.argc - 1;
+      snprintf (fname, FILENAME_MAX, "%s", ucon64.argv[n_files]);
     }
+  fname[FILENAME_MAX - 1] = '\0';
 
   ucon64_file_handler (fname, NULL, OF_FORCE_BASENAME);
   if ((destfile = fopen (fname, "wb")) == NULL)
@@ -765,10 +793,17 @@ gba_multi (int truncate_size, char *multi_fname)
         {
           if (multi_fname != NULL)              // -xfalmulti
             {
-              get_property_fname (ucon64.configfile, "gbaloader", fname, "loader.bin");
+              size_t len;
+
+              p = get_property (ucon64.configfile, "gbaloader", PROPERTY_MODE_FILENAME);
+              if (!p)
+                p = "loader.bin";
+              len = strnlen (p, sizeof fname - 1);
+              strncpy (fname, p, len)[len] = '\0';
               if (access (fname, F_OK))
                 {
                   fprintf (stderr, "ERROR: Cannot open loader binary (%s)\n", fname);
+                  fclose (destfile);
                   return -1;
                 }
               fname_ptr = fname;
@@ -786,7 +821,7 @@ gba_multi (int truncate_size, char *multi_fname)
       else
         {
           fname_ptr = ucon64.argv[n];
-          printf ("ROM%d: %s\n", file_no, fname_ptr);
+          printf ("ROM%u: %s\n", file_no, fname_ptr);
         }
 
       if ((srcfile = fopen (fname_ptr, "rb")) == NULL)
@@ -798,15 +833,15 @@ gba_multi (int truncate_size, char *multi_fname)
       byteswritten = 0;                         // # of bytes written per file
       while (!done)
         {
-          bytestowrite = fread (buffer, 1, BUFSIZE, srcfile);
+          bytestowrite = fread (buffer, 1, sizeof buffer, srcfile);
           if (totalsize + bytestowrite > truncate_size)
             {
               bytestowrite = truncate_size - totalsize;
               done = 1;
               truncated = 1;
-              printf ("Output file is %d Mbit, truncating %s, skipping %d bytes\n",
+              printf ("Output file is %u Mbit, truncating %s, skipping %u bytes\n",
                       truncate_size / MBIT, fname_ptr,
-                      fsizeof (fname_ptr) - (byteswritten + bytestowrite));
+                      (unsigned) (fsizeof (fname_ptr) - (byteswritten + bytestowrite)));
               // DON'T use fstate.st_size, because file could be compressed
             }
           totalsize += bytestowrite;
@@ -840,11 +875,475 @@ gba_multi (int truncate_size, char *multi_fname)
 
   if (totalsize > 64 * MBIT && !truncate_size_ispow2)
     printf("\n"
-           "NOTE: This multi-game file can only be written to a card >= %d Mbit.\n"
-           "      Use -multi=%d to create a file truncated to %d Mbit.\n"
+           "NOTE: This multi-game file can only be written to a card >= %u Mbit.\n"
+           "      Use -multi=%u to create a file truncated to %u Mbit.\n"
            "      Current size is %.5f Mbit\n", // 5 digits to have 1 byte resolution
            size_pow2 / MBIT, size_pow2_lesser / MBIT, size_pow2_lesser / MBIT,
            totalsize / (float) MBIT);
 
   return 0;
 }
+
+
+static int
+gba_saver_patch (FILE *destfile, unsigned char *buffer, unsigned int fsize)
+{
+  // reverse engineered from SuperCard.exe
+  const unsigned char saver_patch_orig[38] =
+    {
+      0xf0, 0xb5, 0xa0, 0xb0, 0x0d, 0x1c, 0x16, 0x1c,
+      0x1f, 0x1c, 0x03, 0x04, 0x1c, 0x0c, 0x0f, 0x4a,
+      0x10, 0x88, 0x0f, 0x49, 0x08, 0x40, 0x03, 0x21,
+      0x08, 0x43, 0x10, 0x80, 0x0d, 0x48, 0x00, 0x68,
+      0x01, 0x68, 0x80, 0x20, 0x80, 0x02
+    },
+    saver_patch_repl[38] =
+    {
+      0x70, 0xb5, 0xa0, 0xb0, 0x00, 0x03, 0x40, 0x18,
+      0xe0, 0x21, 0x09, 0x05, 0x09, 0x18, 0x08, 0x78,
+      0x10, 0x70, 0x01, 0x3b, 0x01, 0x32, 0x01, 0x31,
+      0x00, 0x2b, 0xf8, 0xd1, 0x00, 0x20, 0x20, 0xb0,
+      0x70, 0xbc, 0x02, 0xbc, 0x08, 0x47
+    },
+    saver_patch2_orig[38] =
+    {
+      0xf0, 0xb5, 0x90, 0xb0, 0x0f, 0x1c, 0x00, 0x04,
+      0x04, 0x0c, 0x0f, 0x2c, 0x04, 0xd9, 0x01, 0x48,
+      0x40, 0xe0, 0x00, 0x00, 0xff, 0x80, 0x00, 0x00,
+      0x20, 0x1c, 0xff, 0xf7, 0xd7, 0xfe, 0x00, 0x04,
+      0x05, 0x0c, 0x00, 0x2d, 0x35, 0xd1
+    },
+    saver_patch2_repl[38] =
+    {
+      0x70, 0xb5, 0x00, 0x03, 0x0a, 0x1c, 0xe0, 0x21,
+      0x09, 0x05, 0x41, 0x18, 0x01, 0x23, 0x1b, 0x03,
+      0x10, 0x78, 0x08, 0x70, 0x01, 0x3b, 0x01, 0x32,
+      0x01, 0x31, 0x00, 0x2b, 0xf8, 0xd1, 0x00, 0x20,
+      0x70, 0xbc, 0x02, 0xbc, 0x08, 0x47
+    },
+    saver_patch3_orig[38] =
+    {
+      0xa2, 0xb0, 0x0d, 0x1c, 0x00, 0x04, 0x03, 0x0c,
+      0x03, 0x48, 0x00, 0x68, 0x80, 0x88, 0x83, 0x42,
+      0x05, 0xd3, 0x01, 0x48,
+      0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f,       // 6 byte wildcard
+      0xff, 0x80, 0x00, 0x00,
+      0x3f,                                     // 1 byte wildcard
+      0x48, 0x06, 0x1c, 0x00, 0x68, 0x01, 0x7a
+    },
+    saver_patch3_repl[38] =
+    {
+      0x00, 0x04, 0x0a, 0x1c, 0x40, 0x0b, 0xe0, 0x21,
+      0x09, 0x05, 0x41, 0x18, 0x07, 0x31, 0x00, 0x23,
+      0x08, 0x78, 0x10, 0x70,
+      0x01, 0x33, 0x01, 0x32, 0x01, 0x39,
+      0x07, 0x2b, 0xf8, 0xd9,
+      0x00,
+      0x20, 0x70, 0xbc, 0x02, 0xbc, 0x08, 0x47
+    },
+    saver_patch4_orig[40] =
+    {
+      0x30, 0xb5,
+      0xa9, 0xb0, 0x0d, 0x1c, 0x00, 0x04, 0x04, 0x0c,
+      0x03, 0x48, 0x00, 0x68, 0x80, 0x88, 0x84, 0x42,
+      0x05, 0xd3, 0x01, 0x48,
+      0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f,       // 6 byte wildcard
+      0xff, 0x80, 0x00, 0x00, 0x0f, 0x48,
+      0x00, 0x68, 0x00, 0x7a, 0x40, 0x00
+    },
+    saver_patch4_repl[40] =
+    {
+      0x70, 0xb5,
+      0x00, 0x04, 0x0a, 0x1c, 0x40, 0x0b, 0xe0, 0x21,
+      0x09, 0x05, 0x41, 0x18, 0x07, 0x31, 0x00, 0x23,
+      0x10, 0x78, 0x08, 0x70,
+      0x01, 0x33, 0x01, 0x32, 0x01, 0x39,
+      0x07, 0x2b, 0xf8, 0xd9, 0x00, 0x20,
+      0x70, 0xbc, 0x02, 0xbc, 0x08, 0x47
+    },
+    saver_patch5_orig[42] =
+    {
+      0xf0, 0xb5, 0x90, 0xb0, 0x0f, 0x1c, 0x00, 0x04,
+      0x04, 0x0c, 0x03, 0x48, 0x00, 0x68, 0x40, 0x89,
+      0x84, 0x42, 0x05, 0xd3, 0x01, 0x48, 0x41, 0xe0,
+      0xf0, 0x89, 0x03, 0x02, 0xff, 0x80, 0x00, 0x00,
+      0x20, 0x1c, 0xff, 0xf7, 0xd3, 0xfe, 0x00, 0x04,
+      0x05, 0x0c
+    },
+    saver_patch5_repl[42] =
+    {
+      0x7c, 0xb5, 0x90, 0xb0, 0x00, 0x03, 0x0a, 0x1c,
+      0xe0, 0x21, 0x09, 0x05, 0x09, 0x18, 0x01, 0x23,
+      0x1b, 0x03, 0x10, 0x78, 0x08, 0x70, 0x01, 0x3b,
+      0x01, 0x32, 0x01, 0x31, 0x00, 0x2b, 0xf8, 0xd1,
+      0x00, 0x20, 0x10, 0xb0, 0x7c, 0xbc, 0x02, 0xbc,
+      0x08, 0x47
+    };
+  int found = 0;
+  unsigned char *ptr = NULL;
+
+  for (ptr = buffer;
+       (ptr = (unsigned char *) memmem2 (ptr, fsize - (ptr - buffer),
+                                         saver_patch_orig, 38, 0)) != NULL;
+       ptr++)
+    {
+      if (ucon64.quiet < 0)
+        printf ("offset: 0x%08x\n", (int) (ptr - buffer));
+      fseek (destfile, (long) (ptr - buffer), SEEK_SET);
+      fwrite (saver_patch_repl, 1, 38, destfile);
+      found++;
+    }
+
+  for (ptr = buffer;
+       (ptr = (unsigned char *) memmem2 (ptr, fsize - (ptr - buffer),
+                                         saver_patch2_orig, 38, 0)) != NULL;
+       ptr++)
+    {
+      if (ucon64.quiet < 0)
+        printf ("offset: 0x%08x\n", (int) (ptr - buffer));
+      fseek (destfile, (long) (ptr - buffer), SEEK_SET);
+      fwrite (saver_patch2_repl, 1, 38, destfile);
+      found++;
+    }
+
+  for (ptr = buffer;
+       (ptr = (unsigned char *) memmem2 (ptr, fsize - (ptr - buffer),
+                                         saver_patch3_orig, 38, MEMMEM2_WCARD (0x3f))) != NULL;
+       ptr++)
+    {
+      if (ucon64.quiet < 0)
+        printf ("offset: 0x%08x\n", (int) (ptr - buffer));
+      fseek (destfile, (long) (ptr - buffer), SEEK_SET);
+      fwrite (saver_patch3_repl, 1, 38, destfile);
+      found++;
+    }
+
+  for (ptr = buffer;
+       (ptr = (unsigned char *) memmem2 (ptr, fsize - (ptr - buffer),
+                                         saver_patch4_orig, 40, MEMMEM2_WCARD (0x3f))) != NULL;
+       ptr++)
+    {
+      if (ucon64.quiet < 0)
+        printf ("offset: 0x%08x\n", (int) (ptr - buffer));
+      fseek (destfile, (long) (ptr - buffer), SEEK_SET);
+      fwrite (saver_patch4_repl, 1, 40, destfile);
+      found++;
+    }
+
+  for (ptr = buffer;
+       (ptr = (unsigned char *) memmem2 (ptr, fsize - (ptr - buffer),
+                                         saver_patch5_orig, 42, 0)) != NULL;
+       ptr++)
+    {
+      if (ucon64.quiet < 0)
+        printf ("offset: 0x%08x\n", (int) (ptr - buffer));
+      fseek (destfile, (long) (ptr - buffer), SEEK_SET);
+      fwrite (saver_patch5_repl, 1, 42, destfile);
+      found++;
+    }
+
+  fprintf (stdout, "%d saver patches applied\n", found);
+
+  return 0;
+}
+
+
+// We allocate one buffer for the templates and loader (not for each structure)
+#if     GBA_SCI_TEMPLATE_SIZE < GBA_BLANK_SIZE || \
+        GBA_SCI_TEMPLATE_SIZE < GBA_MENU_SIZE || \
+        GBA_SCI_TEMPLATE_SIZE < GBA_SAV_TEMPLATE_SIZE
+#error GBA_SCI_TEMPLATE_SIZE is too small
+#endif
+
+int
+gba_sc (void)
+/*
+  reverse engineered from SuperCard.exe
+
+  BOND EON only?
+  a0 7f 00 03 00 80 00 03
+  a0 7f 00 03 f0 7f 00 03
+
+  0e 48 39 68  01 60 0e 48 -> 00 48 00 47  01 fb 7f 08
+  0003756c  0a 1c 02 80  0e 48 39 68  01 60 0e 48  79 68 01 60 TONY HAWK 2!
+  0003756c  0a 1c 02 80  00 48 00 47  01 fb 7f 08  79 68 01 60
+
+  d0 -> e0
+  00037842  7f 08 27 e0  d0 20 00 05  01 88 01 22 TONY HAWK 2!
+  00037842  7f 08 27 e0  e0 20 00 05  01 88 01 22
+
+  08 80 -> c0 46
+  00039870  08 88 07 48  08 80 00 f0  e5 f8 01 f0  a9 f8 01 f0 PINOBEE only?
+  00039870  08 88 07 48  c0 46 00 f0  e5 f8 01 f0  a9 f8 01 f0
+
+  08 80 -> c0 46
+  00000238  3a 4c 20 1c  08 80 3a 49  3a 48 08 60  4a 60 3a 48 PINOBEE
+  00000238  3a 4c 20 1c  c0 46 3a 49  3a 48 08 60  4a 60 3a 48
+
+  01 60 -> c0 46
+  000000fa  09 02 14 31  01 60 27 49  28 48 08 60  40 21 09 03 CT SPECIAL F only?
+  000000fa  09 02 14 31  c0 46 27 49  28 48 08 60  40 21 09 03
+*/
+{
+  const unsigned char sc_orig[15][6] =
+    {
+      { 0x04, 0x02, 0x00, 0x04, 0x01, 0x20 },
+      { 0x04, 0x02, 0x00, 0x04, 0x14, 0x40 },
+      { 0x04, 0x02, 0x00, 0x04, 0x14, 0xb2 },
+      { 0x04, 0x02, 0x00, 0x04, 0x38, 0x68 },
+      { 0x04, 0x02, 0x00, 0x04, 0xa0, 0x0f },
+      { 0x04, 0x02, 0x00, 0x04, 0xb4, 0x05 },
+      { 0x04, 0x02, 0x00, 0x04, 0xb4, 0x45 },
+      { 0x04, 0x02, 0x00, 0x04, 0xb6, 0x45 },
+      { 0x04, 0x02, 0x00, 0x04, 0xb7, 0x45 },
+      { 0x04, 0x02, 0x00, 0x04, 0xd4, 0x00 },
+      { 0x04, 0x02, 0x00, 0x04, 0xe3, 0xff },
+      { 0x04, 0x02, 0x00, 0x04, 0xf8, 0x20 },
+      { 0x04, 0x02, 0x00, 0x04, 0xfc, 0xff },
+      { 0x04, 0x02, 0x00, 0x04, 0xfe, 0x2f },
+//      { 0x04, 0x02, 0x00, 0x04, 0xfe, 0xff },   // CASTLEVANIA1 + CROUCHING TI only?
+      { 0x04, 0x02, 0x00, 0x04, 0xff, 0xf8 }    //  they have it but work also without it
+    },
+    sc_repl[4] =
+    {
+      0x00, 0x00, 0x00, 0x00                    // overwrite only the 0x04 0x02 0x00 0x04 part
+    },
+    sc2_orig[4] =
+    {
+      0xfc, 0x7f, 0x00, 0x03                    // replace this everywhere
+    },
+    sc2_repl = 0xa0,
+    sc3_orig[92] =
+    {
+      // replace this at offset 0xee only
+      0x80, 0xe3,
+      0x81, 0x1f, 0xa0, 0xe3,
+      0xb1, 0x00, 0x8c,                         // diff
+      0xe1,
+      0xd4, 0xc0, 0x8c, 0xe2, 0x74, 0x00, 0x9f, 0xe5,
+      0x74, 0x10, 0x9f, 0xe5, 0x74, 0x30, 0x9f, 0xe5,
+      0x01, 0x30, 0x43, 0xe0, 0x21, 0x23, 0xa0, 0xe3,
+      0x43, 0x21, 0x82, 0xe1, 0x07, 0x00, 0x8c, 0xe8,
+      0x03, 0x00, 0x80, 0xe0, 0x60, 0x10, 0x9f, 0xe5,
+      0x60, 0x30, 0x9f, 0xe5, 0x01, 0x30, 0x43, 0xe0,
+      0x21, 0x23, 0xa0, 0xe3, 0x43, 0x21, 0x82, 0xe1,
+      0x07, 0x00, 0x8c, 0xe8, 0x64, 0x30, 0x9f, 0xe5,
+      0x13, 0xff, 0x2f, 0xe1, 0x15, 0x49, 0x13, 0x48,
+      0x16, 0x4b, 0x42, 0x1a, 0xd2, 0x1a, 0x02, 0x20,
+      0xfe, 0x46
+    },
+    sc3_repl[92] =
+    {
+      0x80, 0xe3,
+      0x81, 0x1f, 0xa0, 0xe3,
+      0x00, 0x00, 0xa0,                         // diff
+      0xe1,
+      0xd4, 0xc0, 0x8c, 0xe2, 0x74, 0x00, 0x9f, 0xe5,
+      0x74, 0x10, 0x9f, 0xe5, 0x74, 0x30, 0x9f, 0xe5,
+      0x01, 0x30, 0x43, 0xe0, 0x21, 0x23, 0xa0, 0xe3,
+      0x43, 0x21, 0x82, 0xe1, 0x07, 0x00, 0x8c, 0xe8,
+      0x03, 0x00, 0x80, 0xe0, 0x60, 0x10, 0x9f, 0xe5,
+      0x60, 0x30, 0x9f, 0xe5, 0x01, 0x30, 0x43, 0xe0,
+      0x21, 0x23, 0xa0, 0xe3, 0x43, 0x21, 0x82, 0xe1,
+      0x07, 0x00, 0x8c, 0xe8, 0x64, 0x30, 0x9f, 0xe5,
+      0x13, 0xff, 0x2f, 0xe1, 0x15, 0x49, 0x13, 0x48,
+      0x16, 0x4b, 0x42, 0x1a, 0xd2, 0x1a, 0x02, 0x20,
+      0xfe, 0x46
+    };
+  int x = 0;
+  unsigned int fsize = (unsigned int) ucon64.fsize, padded = 0;
+  uint32_t address = 0, pos = 0;
+  char dest_name[FILENAME_MAX], fname[FILENAME_MAX];
+  unsigned char *buffer, *ptr = NULL;
+  const char *p = NULL;
+  FILE *destfile;
+
+  strcpy (dest_name, ucon64.fname);
+  ucon64_file_handler (dest_name, NULL, 0);
+  fcopy (ucon64.fname, 0, ucon64.fsize, dest_name, "wb");
+
+  if ((destfile = fopen (dest_name, "rb+")) == NULL)
+    {
+      fprintf (stderr, ucon64_msg[OPEN_WRITE_ERROR], dest_name);
+      return -1;
+    }
+  if ((buffer = (unsigned char *) malloc (fsize)) == NULL)
+    {
+      fprintf (stderr, ucon64_msg[ROM_BUFFER_ERROR], fsize);
+      fclose (destfile);
+      exit (1);
+    }
+  if (fread (buffer, 1, fsize, destfile) != fsize)
+    {
+      fprintf (stderr, ucon64_msg[READ_ERROR], dest_name);
+      free (buffer);
+      fclose (destfile);
+      return -1;
+    }
+
+  // saver patch
+  gba_saver_patch (destfile, buffer, fsize);
+
+  // restart
+  for (ptr = buffer;
+       (ptr = (unsigned char *) memmem2 (ptr, fsize - (ptr - buffer),
+                                         sc_orig[0], 4, 0)) != NULL &&
+       (ptr - buffer) < 7445976;                // seems like < 7445976 is far enough
+                                                // SONICPINBALL 64Mb (< 7445976)
+                                                // AGB KIRBY DX (< 10223404) 128Mb
+       ptr++)
+    for (x = 0; x < 15; x++)
+      if (!memcmp (ptr, sc_orig[x], 6))         // Do the last 2 bytes match?
+        {
+          if (ucon64.quiet < 0)
+            printf ("offset: 0x%08x\n", (int) (ptr - buffer));
+          fseek (destfile, (long) (ptr - buffer), SEEK_SET);
+          fwrite (sc_repl, 1, 4, destfile);
+        }
+
+  for (ptr = buffer;
+       (ptr = (unsigned char *) memmem2 (ptr, fsize - (ptr - buffer),
+                                         sc2_orig, 4, 0)) != NULL &&
+       (ptr - buffer) < 7445660;                // seems like < 7445660 is far enough
+                                                // SONICPINBALL 64 Mb (< 7445660)
+                                                // AGB KIRBY DX (< 10223428) 128Mb
+       ptr++)
+    {
+      if (ucon64.quiet < 0)
+        printf ("offset: 0x%08x\n", (int) (ptr - buffer));
+      fseek (destfile, (long) (ptr - buffer), SEEK_SET);
+      fputc (sc2_repl, destfile);
+    }
+
+  if (!memcmp (buffer + 0xee, &sc3_orig, 92))
+    {
+      if (ucon64.quiet < 0)
+        printf ("offset: 0x%08x\n", 0xee);
+      fseek (destfile, 0xee, SEEK_SET);
+      fwrite (sc3_repl, 1, 92, destfile);
+    }
+
+  // write menu (intro) at end of ROM
+  // SuperCard.exe ignores padding with anything else than 0xff Bytes
+  fseek (destfile, -1, SEEK_END);
+  if ((unsigned char) fgetc (destfile) == 0xff)
+    padded = (unsigned int) ucon64_testpad (ucon64.fname);
+  fseek (destfile, fsize - padded, SEEK_SET);
+
+  printf ("Writing restart menu at offset: 0x%08x\n", fsize - padded);
+
+  {
+    // there is a 52232 bytes blank before the actual menu
+    unsigned char *old_buffer = buffer;
+    if ((buffer = (unsigned char *) realloc (old_buffer, GBA_SCI_TEMPLATE_SIZE)) == NULL)
+      {
+        fprintf (stderr, ucon64_msg[ROM_BUFFER_ERROR], GBA_SCI_TEMPLATE_SIZE);
+        free (old_buffer);
+        fclose (destfile);
+        exit (1);
+      }
+  }
+  memset (buffer, 0, GBA_BLANK_SIZE);
+  fwrite (buffer, 1, GBA_BLANK_SIZE, destfile);
+
+  /*
+    menu[0x2f0]:
+    0xe0,                               // some kind of chksum? changes into 0xee, 0xf0, ...
+    0xff, 0x00, 0x0e, 0x01, 0x00, 0x00, 0x00,
+    0xc0, 0x00, 0x00, 0x08,             // the old start address 0xc0 changes into 0xc8 sometimes
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0xf0, 0xfd, 0x00, 0x0e,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x08,             // address in menu
+    0x00, 0x00, 0x00, 0x00,
+  */
+  // write the menu (the formulas will NOT be optimized)
+  p = get_property (ucon64.configfile, "gbaloader_sc", PROPERTY_MODE_FILENAME);
+  if (!p)
+    p = "sc_menu.bin";
+  {
+    size_t len = strnlen (p, sizeof fname - 1);
+
+    strncpy (fname, p, len)[len] = '\0';
+  }
+  if (ucon64_fread (buffer, 0, GBA_MENU_SIZE, fname) == 0)
+    {
+      fprintf (stderr, "ERROR: Could not load Super Card loader (%s)\n", fname);
+      exit (1);                                 // fatal
+    }
+  fwrite (buffer, 1, GBA_MENU_SIZE, destfile);
+  pos = ftell (destfile);                       // truncate() this later
+
+  // calculate and write new start address
+  printf ("New start address: 0x%08x\n", (int) (pos - GBA_MENU_SIZE + 0x8000000));
+  address = ((pos - GBA_MENU_SIZE - 8) >> 2) & 0xffffff;
+  fseek (destfile, 0, SEEK_SET);
+#ifdef  WORDS_BIGENDIAN
+  address = bswap_32 (address);
+#endif
+  fwrite (&address, 1, 3, destfile);
+
+  // calculate and write new address at offset 0x60
+  printf ("New offset 0x60 address: 0x%08x\n", (int) (pos - GBA_MENU_SIZE + 846));
+  address = (pos - GBA_MENU_SIZE + 846 - 2) & 0xffffff;
+  if ((pos - GBA_MENU_SIZE) > 128 * MBIT)
+    address |= 0x09000000;                      // 0x09 == "large" jmp?
+  else
+    address |= 0x08000000;
+  fseek (destfile, 0x60, SEEK_SET);
+#ifdef  WORDS_BIGENDIAN
+  address = bswap_32 (address);
+#endif
+  fwrite (&address, 1, 4, destfile);
+
+  // calculate and write new address in menu
+  printf ("New address in menu: 0x%08x\n", (int) (pos - GBA_MENU_SIZE + 846 - 53076));
+  address = (pos - GBA_MENU_SIZE + 846 - 53076 - 2) & 0xffffff;
+  if ((pos - GBA_MENU_SIZE) > 128 * MBIT)
+    address |= 0x09000000;                      // 0x09 == "large" jmp?
+  else
+    address |= 0x08000000;
+  fseek (destfile, pos - GBA_MENU_SIZE + 784, SEEK_SET);
+#ifdef  WORDS_BIGENDIAN
+  address = bswap_32 (address);
+#endif
+  fwrite (&address, 1, 4, destfile);
+
+  fclose (destfile);
+
+  puts ("Removing padded bytes");
+  truncate2 (dest_name, pos);
+  puts ("Super Card conversion done");
+  printf (ucon64_msg[WROTE], dest_name);
+
+  // write SAV template
+  set_suffix (dest_name, ".sav");
+  if ((destfile = fopen (dest_name, "wb")) == NULL)
+    {
+      fprintf (stderr, ucon64_msg[OPEN_WRITE_ERROR], dest_name);
+      return -1;
+    }
+  memset (buffer, 0, GBA_SAV_TEMPLATE_SIZE);
+  buffer[2] = 0xaa;
+  buffer[3] = 0x55;
+  fwrite (buffer, 1, GBA_SAV_TEMPLATE_SIZE, destfile);
+  fclose (destfile);
+  printf (ucon64_msg[WROTE], dest_name);
+
+  // write SCI template
+  set_suffix (dest_name, ".sci");
+  if ((destfile = fopen (dest_name, "wb")) == NULL)
+    {
+      fprintf (stderr, ucon64_msg[OPEN_WRITE_ERROR], dest_name);
+      return -1;
+    }
+  memset (buffer, 0, GBA_SCI_TEMPLATE_SIZE);
+  buffer[2] = 0xaa;
+  buffer[3] = 0x55;
+  fwrite (buffer, 1, GBA_SCI_TEMPLATE_SIZE, destfile);
+  fclose (destfile);
+  printf (ucon64_msg[WROTE], dest_name);
+
+  free (buffer);
+  return 0;
+}
