@@ -1,7 +1,7 @@
 /*
 ngp.h - NeoGeo Pocket support for uCON64
 
-Copyright (c) 1999 - 2001 NoisyB <noisyb@gmx.net>
+Copyright (c) 1999 - 2001 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,12 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef NGP_H
 #define NGP_H
 
+#include "misc/getopt2.h"                       // st_getopt2_t
+#include "ucon64.h"
+
+
 extern const st_getopt2_t ngp_usage[];
-extern int ngp_init (st_rominfo_t *rominfo);
+
+extern int ngp_init (st_ucon64_nfo_t *rominfo);
+
 #endif
