@@ -1,7 +1,8 @@
 /*
 pce.h - PC-Engine support for uCON64
 
-Copyright (c) 1999 - 2001 NoisyB <noisyb@gmx.net>
+Copyright (c) 1999 - 2001                    NoisyB
+Copyright (c) 2003 - 2004, 2015, 2017 - 2018 dbjh
 
 
 This program is free software; you can redistribute it and/or modify
@@ -20,12 +21,18 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #ifndef PCE_H
 #define PCE_H
-extern const st_getopt2_t pcengine_usage[];
 
-extern int pcengine_init (st_rominfo_t *rominfo);
-extern int pcengine_mgd (st_rominfo_t *rominfo);
-extern int pcengine_msg (st_rominfo_t *rominfo);
-extern int pcengine_swap (st_rominfo_t *rominfo);
-extern int pcengine_f (st_rominfo_t *rominfo);
-extern int pcengine_multi (int truncate_size, char *fname);
+#include "misc/getopt2.h"                       // st_getopt2_t
+#include "ucon64.h"
+
+
+extern const st_getopt2_t pce_usage[];
+
+extern int pce_init (st_ucon64_nfo_t *rominfo);
+extern int pce_mgd (st_ucon64_nfo_t *rominfo);
+extern int pce_msg (st_ucon64_nfo_t *rominfo);
+extern int pce_swap (st_ucon64_nfo_t *rominfo);
+extern int pce_f (st_ucon64_nfo_t *rominfo);
+extern int pce_multi (unsigned int truncate_size);
+
 #endif
