@@ -1,7 +1,7 @@
 /*
 psx.c - Playstation support for uCON64
 
-Copyright (c) 2001 NoisyB <noisyb@gmx.net>
+Copyright (c) 2001 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -18,49 +18,37 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-#ifdef  HAVE_CONFIG_H
-#include "config.h"
-#endif
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#ifdef  HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include "misc/misc.h"
-#include "misc/itypes.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
-#endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "ucon64.h"
-#include "ucon64_misc.h"
-#include "psx.h"
+#include "console/psx.h"
 
 
+static st_ucon64_obj_t psx_obj[] =
+  {
+    {UCON64_PSX, WF_SWITCH}
+  };
+
 const st_getopt2_t psx_usage[] =
   {
     {
       NULL, 0, 0, 0,
-      NULL, "Playstation (One)/Playstation 2 (CD only)"/*"1994/(2000) Sony http://www.playstation.com"*/,
+      NULL, "Playstation (One)"/*"1994/(2000) Sony http://www.playstation.com"*/,
       NULL
     },
     {
-      "psx", 0, 0, UCON64_PSX,
+      UCON64_PSX_S, 0, 0, UCON64_PSX,
       NULL, "force recognition",
-      &ucon64_wf[WF_OBJ_PSX_SWITCH]
+      &psx_obj[0]
     },
     {NULL, 0, 0, 0, NULL, NULL, NULL}
 };
 
 
 int
-psx_init (st_rominfo_t *rominfo)
+psx_init (st_ucon64_nfo_t *rominfo)
 {
   int result = -1;
 
   rominfo->console_usage = psx_usage[0].help;
-//  rominfo->copier_usage = cdrw_usage[0].help;
+//  rominfo->backup_usage = cdrw_usage[0].help;
 
   return result;
 }
