@@ -1,7 +1,7 @@
 /*
 psx.h - Playstation support for uCON64
 
-Copyright (c) 2001 NoisyB <noisyb@gmx.net>
+Copyright (c) 2001 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,13 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #ifndef PSX_H
 #define PSX_H
-extern int psx_init (st_rominfo_t *rominfo);
+
+#include "misc/getopt2.h"                       // st_getopt2_t
+#include "ucon64.h"
+
+
 extern const st_getopt2_t psx_usage[];
+
+extern int psx_init (st_ucon64_nfo_t *rominfo);
+
 #endif
