@@ -1,7 +1,7 @@
 /*
 libdm_misc.c - libdiscmage miscellaneous
 
-Copyright (c) 2002 - 2004 NoisyB (noisyb@gmx.net)
+Copyright (c) 2002 - 2004 NoisyB
 
 
 This library is free software; you can redistribute it and/or
@@ -24,12 +24,26 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #ifdef  HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
+#endif
 #include <ctype.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
 #include <stdio.h>
 #include <stdlib.h>
-#include <time.h>
 #include <string.h>
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4820) // 'bytes' bytes padding added after construct 'member_name'
+#endif
+#include <time.h>
 #include <sys/stat.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
 #ifdef  DEBUG
 #ifdef  __GNUC__
 #warning DEBUG active
@@ -41,7 +55,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "libdiscmage.h"
 #include "libdm_misc.h"
 #include "format/format.h"
-#ifdef  DJGPP                                   // DXE's are specific to DJGPP
+#ifdef  DJGPP                                   // DXEs are specific to DJGPP
 #include "dxedll_priv.h"
 #endif
 #include "misc_wav.h"
@@ -59,10 +73,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 const char pvd_magic[] = {0x01, 'C', 'D', '0', '0', '1', 0x01, 0};
 const char svd_magic[] = {0x02, 'C', 'D', '0', '0', '1', 0x01, 0};
-const char vdt_magic[] = {(const char) 0xff, 'C', 'D', '0', '0', '1', 0x01, 0};
+const char vdt_magic[] = {0xff, 'C', 'D', '0', '0', '1', 0x01, 0};
 
 
-#define ISODCL(from, to) (to - from + 1)
+#define ISODCL(from, to) ((to) - (from) + 1)
 typedef struct
   {
     char type[ISODCL (1, 1)];   /* 711 */
@@ -103,7 +117,7 @@ typedef struct
 
 /*
  * A CD-ROM physical sector size is 2048, 2052, 2056, 2324, 2332, 2336,
- * 2340, or 2352 bytes long.  
+ * 2340, or 2352 bytes long.
 
 *         Sector types of the standard CD-ROM data formats:
  *
@@ -137,7 +151,7 @@ typedef struct
 
 /*
   from CDmage
-  
+
   m1_2048=iso
   m1_2352=tao
   m2_2336=mm2
@@ -196,13 +210,14 @@ const st_track_probe_t track_probe[] =
     {0, 0, 0, 0, 0, NULL}
   };
 
-const char *dm_msg[] = {
-  "ERROR: %s has been deprecated\n",
-  "ERROR: Unknown/unsupported track mode\n",
-  "ERROR: The images track mode is already MODE1/2048\n",
-  "WARNING: This function is still in ALPHA stage. It might not work properly\n",
-  NULL
-};
+const char *dm_msg[] =
+  {
+    "ERROR: %s has been deprecated\n",
+    "ERROR: Unknown/unsupported track mode\n",
+    "ERROR: The images track mode is already MODE1/2048\n",
+    "WARNING: This function is still in ALPHA stage. It might not work properly\n",
+    NULL
+  };
 
 static void (*dm_gauge_ptr) (int pos, int size) = NULL;
 
@@ -229,8 +244,8 @@ dm_get_track_mode_by_id (int id, int8_t *mode, uint16_t *sector_size)
   for (x = 0; track_probe[x].sector_size; x++)
     if (track_probe[x].id == id)
       {
-        *mode = track_probe[x].mode;
-        *sector_size = track_probe[x].sector_size;
+        *mode = (int8_t) track_probe[x].mode;
+        *sector_size = (uint16_t) track_probe[x].sector_size;
         return;
       }
 }
@@ -286,7 +301,7 @@ dm_lba_to_msf (int lba, int *m0, int *s0, int *f0)
       f = (lba + 150 - m * CD_SECS * CD_FRAMES - s * CD_FRAMES);
 
     }
-  else if (lba >= -45150 && lba <= -151)
+  else if (lba >= -45150 /* && lba <= -151 */)
     {
       m = (lba + 450150) / CD_SECS / CD_FRAMES;
       s = (lba + 450150 - m * CD_SECS * CD_FRAMES) / CD_FRAMES;
@@ -409,7 +424,7 @@ dm_fdopen (dm_image_t *image, int track_num, const char *mode)
   dm_track_t *track = (dm_track_t *) &image->track[track_num];
   FILE *fh;
 
-  if (!(fh = fopen (image->fname, mode)))
+  if ((fh = fopen (image->fname, mode)) == NULL)
     return NULL;
 
   if (!fseek (fh, track->track_start, SEEK_SET))
@@ -420,8 +435,6 @@ dm_fdopen (dm_image_t *image, int track_num, const char *mode)
 }
 
 
-
-
 int
 dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
 {
@@ -432,19 +445,21 @@ dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
   char buf[MAXBUFSIZE], buf2[MAXBUFSIZE];
 #endif
   unsigned int x = 0;
-  int result = 0, m = 0, s = 0, f = 0;
+  size_t result = 0;
+  int m = 0, s = 0, f = 0;
   char *p = NULL;
   FILE *fh = NULL, *fh2 = NULL;
 
   if (flags & DM_FIX || flags & DM_2048)
-    fprintf (stderr, dm_msg[ALPHA]);
+    fputs (dm_msg[ALPHA], stderr);
 
 // set dest. name
   strcpy (buf, basename (image->fname));
   p = (char *) get_suffix (buf);
   if (p)
     buf[strlen (buf) - strlen (p)] = 0;
-  sprintf (buf2, "%s_%d", buf, track_num + 1);
+  snprintf (buf2, sizeof buf2, "%s_%d", buf, track_num + 1);
+  buf2[sizeof buf2 - 1] = '\0';
 
   switch (track->mode)
     {
@@ -468,7 +483,7 @@ dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
 #if 0
   if (track->total_len < track->track_len + track->pregap_len)
     {
-      fprintf (stderr, "SKIPPING: track seems truncated\n");
+      fputs ("SKIPPING: track seems truncated\n", stderr);
       return -1;
     }
 #endif
@@ -477,7 +492,7 @@ dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
 // this is not a problem
   if (flags & DM_2048 && track->mode == 1 && track->sector_size == 2048)
     {
-      fprintf (stderr, dm_msg[ALREADY_2048]);
+      fputs (dm_msg[ALREADY_2048], stderr);
       return -1;
     }
 #endif
@@ -485,7 +500,7 @@ dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
   if (track->pregap_len != 150) // 0x96
     fprintf (stderr, "WARNING: track seems to have a non-standard pregap (%d Bytes)\n",
              track->pregap_len);
- 
+
 #if 0
   if (image->pregap && track->mode == 0 && remaining_tracks > 1) // quick hack to save next track pregap (audio tracks only)
     track->track_len += pregap_length;       // if this isn't last track in current session
@@ -493,11 +508,11 @@ dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
 #endif
 
 // open source and check
-  if (!(fh = fopen (image->fname, "rb")))
+  if ((fh = fopen (image->fname, "rb")) == NULL)
     return -1;
 
 // open dest.
-  if (!(fh2 = fopen (buf2, "wb")))
+  if ((fh2 = fopen (buf2, "wb")) == NULL)
     {
       fclose (fh);
       return -1;
@@ -513,17 +528,18 @@ dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
   for (x = 0; x < track->track_len; x++)
     {
       memset (buf, 0, sizeof (buf));
-      fread (&buf, 1, track->sector_size, fh);
+      if (fread_checked2 (&buf, 1, track->sector_size, fh) != 0)
+        return -1;
 
       if (flags & DM_2048)
         result = fwrite (&buf[track->seek_header], 1, 2048, fh2);
       else
         {
-          const char sync_data[] = {0, (const char) 0xff, (const char) 0xff,
-                                       (const char) 0xff, (const char) 0xff,
-                                       (const char) 0xff, (const char) 0xff,
-                                       (const char) 0xff, (const char) 0xff,
-                                       (const char) 0xff, (const char) 0xff, 0};
+          const char sync_data[] = {0, 0xff, 0xff,
+                                       0xff, 0xff,
+                                       0xff, 0xff,
+                                       0xff, 0xff,
+                                       0xff, 0xff, 0};
 //          uint32_t value_32 = 0;
 
           memset (&buf2, 0, sizeof (buf2));
@@ -546,7 +562,7 @@ dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
 
       if (!result)
         {
-          fprintf (stderr, "ERROR: writing sector %d\n", x);
+          fprintf (stderr, "ERROR: writing sector %u\n", x);
           fclose (fh);
           fclose (fh2);
           return -1;
@@ -555,9 +571,9 @@ dm_rip (const dm_image_t *image, int track_num, uint32_t flags)
       if (!(x % 100))
         dm_gauge (x * track->sector_size, track->track_len * track->sector_size);
     }
-                  
+
   dm_gauge (x * track->sector_size, track->track_len * track->sector_size);
-                        
+
 //  fseek (fh, track->total_len * track->sector_size, SEEK_CUR);
 
   fclose (fh);
@@ -580,16 +596,16 @@ dm_isofix (const dm_image_t * image, int start_lba, int track_num)
   FILE *dest = NULL, *src = NULL, *boot = NULL, *header = NULL;
   int mac = FALSE;
   const char sub_header[] = {0, 0, 0x08, 0, 0, 0, 0x08, 0};
-  
+
   if (start_lba <= 0)
     {
-      fprintf (stderr, "ERROR: Bad LBA value");
+      fputs ("ERROR: Bad LBA value", stderr);
       return -1;
     }
 
   if (!track)
     return -1;
-  
+
   mac = (track->sector_size == 2056 ? TRUE : FALSE);
 
   if (!(src = fopen (image->fname, "rb")))
@@ -661,16 +677,16 @@ dm_isofix (const dm_image_t * image, int start_lba, int track_num)
           memcmp (svd_magic, buf, 8) != 0 &&
           memcmp (vdt_magic, buf, 8) != 0)
         {
-          fprintf (stderr, "ERROR: Found unknown Volume Descriptor");
+          fputs ("ERROR: Found unknown Volume Descriptor", stderr);
 // which sector?
           return -1;
         }
 
       printf ("Found %s at sector %d\n",
-        !memcmp (pvd_magic, buf, 8) ? "PVD" :
-        !memcmp (svd_magic, buf, 8) ? "SVD" :
-        !memcmp (vdt_magic, buf, 8) ? "VDT" : "unknown Volume Descriptor",
-        last_pos / track->sector_size);
+              !memcmp (pvd_magic, buf, 8) ? "PVD" :
+              !memcmp (svd_magic, buf, 8) ? "SVD" :
+              !memcmp (vdt_magic, buf, 8) ? "VDT" : "unknown Volume Descriptor",
+              last_pos / track->sector_size);
 
       if (mac)
         fwrite (sub_header, 8, 1, dest);
@@ -702,9 +718,9 @@ dm_isofix (const dm_image_t * image, int start_lba, int track_num)
     }
 
   if (start_lba < 11700)
-    fprintf (stderr,
-             "WARNING: LBA value should be greater or equal to 11700 for multisession\n"
-             "         images\n");
+    fputs ("WARNING: LBA value should be greater or equal to 11700 for multisession\n"
+           "         images\n",
+           stderr);
 
   // adding padding data up to start LBA value...
   size_left = start_lba - (last_pos / track->sector_size);
@@ -746,12 +762,8 @@ void
 dm_nfo (const dm_image_t *image, int verbose, int ansi_color)
 {
 #define COLS 80
-  int t = 0, s = 0, x = 0, min = 0, sec = 0, frames = 0,
-      filesize = q_fsize (image->fname);
+  int t = 0, filesize = q_fsize (image->fname);
   char buf[MAXBUFSIZE];
-  st_iso_header_t iso_header;
-  FILE *fh = NULL;
-  int result = -1;
 
 #ifndef USE_ANSI_COLOR
   (void) ansi_color;                            // warning remover
@@ -761,30 +773,33 @@ dm_nfo (const dm_image_t *image, int verbose, int ansi_color)
   if (image->console_usage != NULL)
     {
       strcpy (buf, rominfo->console_usage[0].desc);
-      puts (to_func (buf, strlen (buf), toprint2));
+      puts (tofunc (buf, strlen (buf), toprint2));
 
 #if 0
       if (image->console_usage[1].desc)
         {
           strcpy (buf, rominfo->console_usage[1].desc);
-          printf ("  %s\n", to_func (buf, strlen (buf), toprint2));
+          printf ("  %s\n", tofunc (buf, strlen (buf), toprint2));
         }
 #endif
     }
 #endif
 
-  printf ("%d Bytes (%.4f MB)\n\n", filesize, (float) filesize / (1024 * 1024));
-  printf ("Type: %s\n", image->desc);
+  printf ("%d Bytes (%.4f MB)\n"
+          "\n"
+          "Type: %s\n", filesize, (float) filesize / (1024 * 1024), image->desc);
 
   if (image->misc[0])
     puts (image->misc);
 
-  printf ("Sessions: %d\n", image->sessions);
-  printf ("Tracks: %d\n", image->tracks);
+  printf ("Sessions: %d\n"
+          "Tracks: %d\n", image->sessions, image->tracks);
 
   if ((COLS / image->tracks) > 1 && image->sessions && image->tracks)
     {
-      printf ("Layout: ");
+      int s = 0, x = 0;
+
+      fputs ("Layout: ", stdout);
 
       for (s = t = 0; s < image->sessions; s++)
         {
@@ -810,7 +825,7 @@ dm_nfo (const dm_image_t *image, int verbose, int ansi_color)
             printf ("\x1b[0m] ");
           else
 #endif
-          printf ("] "); 
+          printf ("] ");
         }
 
       fputc ('\n', stdout);
@@ -818,6 +833,8 @@ dm_nfo (const dm_image_t *image, int verbose, int ansi_color)
 
   for (t = 0; t < image->tracks; t++)
     {
+      int min = 0, sec = 0, frames = 0;
+      st_iso_header_t iso_header;
       const dm_track_t *track = (const dm_track_t *) &image->track[t];
 
       if (!track)
@@ -826,75 +843,83 @@ dm_nfo (const dm_image_t *image, int verbose, int ansi_color)
       if (!track->mode && track->sector_size == 2352)
         strcpy (buf, "AUDIO");
       else
-        sprintf (buf, "MODE%d/%d", track->mode, track->sector_size);
+        sprintf (buf, "MODE%d/%u", track->mode, track->sector_size);
 
       printf ("Track: %d %s", t + 1, buf);
 
       dm_lba_to_msf (track->track_len, &min, &sec, &frames);
       printf ("\n  %d Sectors, %d:%02d/%02d MSF, %d Bytes (%.4f MB)",
-        (int) track->total_len,
-        min, sec, frames,
-        (int) (track->total_len * track->sector_size),
-        (float) (track->total_len * track->sector_size) / (1024 * 1024));
+              (int) track->total_len,
+              min, sec, frames,
+              (int) (track->total_len * track->sector_size),
+              (float) (track->total_len * track->sector_size) / (1024 * 1024));
 
       fputc ('\n', stdout);
 
       if (verbose)
         {
           printf ("  Pregap: %d, Start Sector: %d, End Sector: %d, Postgap: %d\n",
-            track->pregap_len,
-            (int) (track->track_start / track->sector_size),
-            (int) ((track->track_start / track->sector_size) + track->total_len),
-            track->postgap_len);
+                  track->pregap_len,
+                  (int) (track->track_start / track->sector_size),
+                  (int) ((track->track_start / track->sector_size) + track->total_len),
+                  track->postgap_len);
 
           dm_lba_to_msf (track->track_len, &min, &sec, &frames);
           printf ("  Total Time: %d:%02d/%02d MSF, File Start Pos: %d, End Pos: %d\n",
-            min, sec, frames,
-            (int) track->track_start,
-            (int) track->track_end);
+                  min, sec, frames,
+                  (int) track->track_start,
+                  (int) track->track_end);
         }
 
       memset (&iso_header, 0, sizeof (st_iso_header_t));
-      result = -1;
 
       if (track->iso_header_start != -1)
-        if ((fh = fopen (image->fname, "rb")))
-          if (fread (&iso_header, track->iso_header_start, sizeof (st_iso_header_t), fh))
+        {
+          FILE *fh;
+
+          if ((fh = fopen (image->fname, "rb")) != NULL)
             {
-              if (verbose)
-                mem_hexdump (&iso_header, sizeof (st_iso_header_t), track->iso_header_start);
+              fseek (fh, track->iso_header_start, SEEK_SET);
+              if (fread (&iso_header, sizeof (st_iso_header_t), 1, fh))
+                {
+                  if (verbose)
+                    mem_hexdump (&iso_header, sizeof (st_iso_header_t),
+                                 track->iso_header_start);
 
-              // name, maker, country and size
-              // some have a name with control chars in it -> replace control chars
-              strncpy2 (buf, NULL_TO_EMPTY (iso_header.volume_id),
-                sizeof (iso_header.volume_id));
-              to_func (buf, strlen (buf), toprint2);
-              if (*strtrim (buf))
-                printf ("  %s\n", buf);
+                  // name, maker, country and size
+                  // some have a name with control chars in it -> replace control chars
+                  strncpy2 (buf, iso_header.volume_id,
+                            sizeof iso_header.volume_id);
+                  tofunc (buf, strlen (buf), toprint2);
+                  if (*strtrim (buf))
+                    printf ("  %s\n", buf);
 
-              strncpy2 (buf, NULL_TO_EMPTY (iso_header.publisher_id),
-                sizeof (iso_header.publisher_id));
-              to_func (buf, strlen (buf), toprint2);
-              if (*strtrim (buf))
-                printf ("  %s\n", buf);
+                  strncpy2 (buf, iso_header.publisher_id,
+                            sizeof iso_header.publisher_id);
+                  tofunc (buf, strlen (buf), toprint2);
+                  if (*strtrim (buf))
+                    printf ("  %s\n", buf);
 
-              strncpy2 (buf, NULL_TO_EMPTY (iso_header.preparer_id),
-                sizeof (iso_header.preparer_id));
-              to_func (buf, strlen (buf), toprint2);
-              if (*strtrim (buf))
-                printf ("  %s\n", buf);
+                  strncpy2 (buf, iso_header.preparer_id,
+                            sizeof iso_header.preparer_id);
+                  tofunc (buf, strlen (buf), toprint2);
+                  if (*strtrim (buf))
+                    printf ("  %s\n", buf);
 #if 0
-              strncpy2 (buf, NULL_TO_EMPTY (iso_header.logical_block_size),
-                sizeof (iso_header.logical_block_size));
-              to_func (buf, strlen (buf), toprint2);
-              if (*strtrim (buf))
-                printf ("  %s\n", buf);
+                  strncpy2 (buf, iso_header.logical_block_size,
+                            sizeof iso_header.logical_block_size);
+                  tofunc (buf, strlen (buf), toprint2);
+                  if (*strtrim (buf))
+                    printf ("  %s\n", buf);
 #endif
-              strncpy2 (buf, NULL_TO_EMPTY (iso_header.application_id),
-                sizeof (iso_header.application_id));
-              to_func (buf, strlen (buf), toprint2);
-              if (*strtrim (buf))
-                printf ("  %s\n", buf);
+                  strncpy2 (buf, iso_header.application_id,
+                            sizeof iso_header.application_id);
+                  tofunc (buf, strlen (buf), toprint2);
+                  if (*strtrim (buf))
+                    printf ("  %s\n", buf);
+                }
+              fclose (fh);
             }
+        }
     }
 }
