@@ -1,9 +1,9 @@
 /*
 misc.h - miscellaneous functions
 
-Copyright (c) 1999 - 2004 NoisyB <noisyb@gmx.net>
-Copyright (c) 2001 - 2004 dbjh
-Copyright (c) 2002 - 2004 Jan-Erik Karlsson (Amiga code)
+Copyright (c) 1999 - 2004              NoisyB
+Copyright (c) 2001 - 2005, 2015 - 2021 dbjh
+Copyright (c) 2002 - 2004              Jan-Erik Karlsson (Amiga code)
 
 
 This program is free software; you can redistribute it and/or modify
@@ -31,16 +31,28 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 extern "C" {
 #endif
 
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
+#endif
 #include <string.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#pragma warning(push)
+#pragma warning(disable: 4820) // 'bytes' bytes padding added after construct 'member_name'
+#endif
 #include <time.h>                               // gauge() prototype contains time_t
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
 #include <stdio.h>
 #ifdef  USE_ZLIB
 #include "misc_z.h"
 #endif                                          // USE_ZLIB
 
 
-#ifdef __sun
-#ifdef __SVR4
+#ifdef  __sun
+#ifdef  __SVR4
 #define __solaris__
 #endif
 #endif
@@ -51,19 +63,16 @@ extern "C" {
 #ifndef OWN_INTTYPES
 #define OWN_INTTYPES                            // signal that these are defined
 typedef unsigned char uint8_t;
-typedef unsigned short int uint16_t;
-typedef unsigned int uint32_t;
-#ifndef _WIN32
-typedef unsigned long long int uint64_t;
-#else
-typedef unsigned __int64 uint64_t;
-#endif
 typedef signed char int8_t;
+typedef unsigned short int uint16_t;
 typedef signed short int int16_t;
+typedef unsigned int uint32_t;
 typedef signed int int32_t;
 #ifndef _WIN32
+typedef unsigned long long int uint64_t;
 typedef signed long long int int64_t;
 #else
+typedef unsigned __int64 uint64_t;
 typedef signed __int64 int64_t;
 #endif
 #endif                                          // OWN_INTTYPES
@@ -72,11 +81,11 @@ typedef signed __int64 int64_t;
 #if     (defined __unix__ && !defined __MSDOS__) || defined __BEOS__ || \
         defined AMIGA || defined __APPLE__      // Mac OS X actually
 // GNU/Linux, Solaris, FreeBSD, Cygwin, BeOS, Amiga, Mac (OS X)
-#define FILE_SEPARATOR '/'
-#define FILE_SEPARATOR_S "/"
+#define DIR_SEPARATOR '/'
+#define DIR_SEPARATOR_S "/"
 #else // DJGPP, Win32
-#define FILE_SEPARATOR '\\'
-#define FILE_SEPARATOR_S "\\"
+#define DIR_SEPARATOR '\\'
+#define DIR_SEPARATOR_S "\\"
 #endif
 
 #ifndef MAXBUFSIZE
@@ -104,12 +113,16 @@ typedef signed __int64 int64_t;
 #define OFFSET(a, offset) ((((unsigned char *)&(a))+(offset))[0])
 
 #ifdef  WORDS_BIGENDIAN
-#undef WORDS_BIGENDIAN
+#undef  WORDS_BIGENDIAN
 #endif
 
 #if     defined _LIBC || defined __GLIBC__
   #include <endian.h>
-  #if __BYTE_ORDER == __BIG_ENDIAN
+  #if     __BYTE_ORDER == __BIG_ENDIAN
+    #define WORDS_BIGENDIAN 1
+  #endif
+#elif   defined __BYTE_ORDER__ && defined __ORDER_BIG_ENDIAN__
+  #if     __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
     #define WORDS_BIGENDIAN 1
   #endif
 #elif   defined AMIGA || defined __sparc__ || defined __BIG_ENDIAN__ || \
@@ -118,41 +131,55 @@ typedef signed __int64 int64_t;
 #endif
 
 #ifdef  __MSDOS__                               // __MSDOS__ must come before __unix__,
-  #define CURRENT_OS_S "MSDOS"                  //  because DJGPP defines both
+  #define CURRENT_OS_S "MS-DOS"                 //  because DJGPP defines both
 #elif   defined __unix__
   #ifdef  __CYGWIN__
-    #define CURRENT_OS_S "Win32 (Cygwin)"
-  #elif   defined __FreeBSD__
-    #define CURRENT_OS_S "Unix (FreeBSD)"
-  #elif   defined __OpenBSD__
-    #define CURRENT_OS_S "Unix (OpenBSD)"
-  #elif   defined __linux__
-    #define CURRENT_OS_S "Unix (Linux)"
-  #elif   defined __solaris__
-    #ifdef __sparc__
-      #define CURRENT_OS_S "Unix (Solaris/Sparc)"
+    #ifdef  __x86_64__                          // _WIN64 is defined in a header
+      #define CURRENT_OS_S "Win64 (Cygwin)"     //  file (include windows.h)
     #else
-      #define CURRENT_OS_S "Unix (Solaris/i386)"
+      #define CURRENT_OS_S "Win32 (Cygwin)"
+    #endif
+  #elif   defined __FreeBSD__
+    #define CURRENT_OS_S "UNIX (FreeBSD)"
+  #elif   defined __OpenBSD__
+    #define CURRENT_OS_S "UNIX (OpenBSD)"
+  #elif   defined __NetBSD__
+    #define CURRENT_OS_S "UNIX (NetBSD)"
+  #elif   defined __linux__
+    #define CURRENT_OS_S "UNIX (Linux)"
+  #elif   defined __solaris__
+    #ifdef  __sparc__
+      #define CURRENT_OS_S "UNIX (Solaris/Sparc)"
+    #else
+      #define CURRENT_OS_S "UNIX (Solaris/i386)"
     #endif
   #else
-    #define CURRENT_OS_S "Unix"
+    #define CURRENT_OS_S "UNIX"
   #endif
 #elif   defined _WIN32
-  #ifdef  __MINGW32__
+  #ifdef  __MINGW64__
+    #define CURRENT_OS_S "Win64 (MinGW-w64)"
+  #elif   defined __MINGW32__
     #define CURRENT_OS_S "Win32 (MinGW)"
+  #elif   defined _MSC_VER
+    #ifdef  _WIN64
+      #define CURRENT_OS_S "Win64 (Visual C++)"
+    #else
+      #define CURRENT_OS_S "Win32 (Visual C++)"
+    #endif
   #else
-    #define CURRENT_OS_S "Win32 (Visual C++)"
+    #define CURRENT_OS_S "Win32"
   #endif
 #elif   defined __APPLE__
-  #if   defined __POWERPC__ || defined __ppc__
-    #define CURRENT_OS_S "Apple (PPC)"
+  #if     defined __POWERPC__ || defined __ppc__
+    #define CURRENT_OS_S "Mac OS X (PPC)"
   #else
-    #define CURRENT_OS_S "Apple"
-  #endif
+    #define CURRENT_OS_S "macOS"                // first Mac OS X, then OS X,
+  #endif                                        //  now macOS
 #elif   defined __BEOS__
   #define CURRENT_OS_S "BeOS"
 #elif   defined AMIGA
-  #if defined __PPC__
+  #if     defined __PPC__
     #define CURRENT_OS_S "Amiga (PPC)"
   #else
     #define CURRENT_OS_S "Amiga (68K)"
@@ -205,11 +232,11 @@ extern uint64_t bswap_64 (uint64_t x);
 #define le2me_32(x) (x)
 #define le2me_64(x) (x)
 #endif
-extern int memwcmp (const void *buffer, const void *search, uint32_t searchlen, int wildcard);
-extern void *mem_search (const void *buffer, uint32_t buflen, const void *search, uint32_t searchlen);
-extern void *mem_swap_b (void *buffer, uint32_t n);
-extern void *mem_swap_w (void *buffer, uint32_t n);
-extern void mem_hexdump (const void *buffer, uint32_t n, int virtual_start);
+extern int memwcmp (const void *buffer, const void *search, size_t searchlen, int wildcard);
+extern void *mem_search (const void *buffer, size_t buflen, const void *search, size_t searchlen);
+extern void *mem_swap_b (void *buffer, size_t n);
+extern void *mem_swap_w (void *buffer, size_t n);
+extern void mem_hexdump (const void *buffer, size_t n, size_t virtual_start);
 
 
 /*
@@ -220,13 +247,13 @@ extern void mem_hexdump (const void *buffer, uint32_t n, int virtual_start);
   tofname()   replaces chars that can not be used for filenames
   toprint2()  replaces chars that should not be used for stdout
 
-  is_func()   use all is*() functions on an array of char
-  to_func()   use all to*() functions on an array of char
+  isfunc()    use all is*() functions on an array of char
+  tofunc()    use all to*() functions on an array of char
 
   strtrim()   trim isspace()'s from start and end of string
   strncpy2()  a safer strncpy that DOES terminate a string
 
-  is_upper2() wrapper for is_upper() (DAMN stupid programmers!)
+  isupper2()  wrapper for isupper() (DAMN stupid programmers!)
   set_suffix() set/replace suffix of filename with suffix
               suffix means in this case the suffix INCLUDING the dot '.'
   set_suffix_i() like set_suffix(), but doesn't change the case
@@ -250,10 +277,12 @@ extern int isfname (int c);
 extern int isprint2 (int c);
 extern int tofname (int c);
 extern int toprint2 (int c);
-extern int is_func (char *s, int size, int (*func) (int));
-extern char *to_func (char *s, int size, int (*func) (int));
-#define strupr(s) (to_func(s, strlen(s), toupper))
-#define strlwr(s) (to_func(s, strlen(s), tolower))
+extern int isfunc (char *s, size_t size, int (*func) (int));
+extern char *tofunc (char *s, size_t size, int (*func) (int));
+#if     !(defined _MSC_VER || defined __CYGWIN__ || defined __MSDOS__)
+#define strupr(s) (tofunc(s, strlen(s), toupper))
+#define strlwr(s) (tofunc(s, strlen(s), tolower))
+#endif
 //#ifndef HAVE_STRCASESTR
 // strcasestr is GNU only
 extern char *strcasestr2 (const char *str, const char *search);
@@ -264,6 +293,8 @@ extern char *strcasestr2 (const char *str, const char *search);
 #ifndef _WIN32
 #define stricmp strcasecmp
 #define strnicmp strncasecmp
+#else
+#include <sys/types.h>                          // off_t (VC++)
 #endif
 extern char *strtrim (char *str);
 extern char *strncpy2 (char *dest, const char *src, size_t size);
@@ -281,7 +312,7 @@ extern int one_filesystem (const char *filename1, const char *filename2);
 extern char *realpath2 (const char *src, char *full_path);
 extern int mkdir2 (const char *name);
 extern int rename2 (const char *oldname, const char *newname);
-extern int truncate2 (const char *filename, int size);
+extern int truncate2 (const char *filename, off_t new_size);
 extern int strarg (char **argv, char *str, const char *separator_s, int max_args);
 
 
@@ -305,9 +336,13 @@ extern int strarg (char **argv, char *str, const char *separator_s, int max_args
                   it can be used for procedures which take some time to
                   inform the user about the actual progress
   getenv2()       getenv() clone for enviroments w/o HOME, TMP or TEMP variables
-  tmpnam2()       replacement for tmpnam() temp must have the size of FILENAME_MAX
+  tmpnam2()       replacement for tmpnam() first argument must be at least
+                  FILENAME_MAX bytes large
+  fread_checked() calls exit() if fread_checked2() returns -1
+  fread_checked2() returns 0 if fread() read the requested number of bytes,
+                  otherwise it returns -1
   renlwr()        renames all files tolower()
-  drop_privileges() switch to the real user and group id (leave "root mode")
+  drop_privileges() switch to the real user and group ID (leave "root mode")
   register_func() atexit() replacement
                   returns -1 if it fails, 0 if it was successful
   unregister_func() unregisters a previously registered function
@@ -315,33 +350,46 @@ extern int strarg (char **argv, char *str, const char *separator_s, int max_args
   handle_registered_funcs() calls all the registered functions
   wait2           wait (sleep) a specified number of milliseconds
 */
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4820) // 'bytes' bytes padding added after construct 'member_name'
+#endif
 typedef struct st_cm_set
 {
   char *data;
-  int size;
+  unsigned int size;
 } st_cm_set_t;
 
 typedef struct st_cm_pattern
 {
   char *search, wildcard, escape, *replace;
-  int search_size, replace_size, offset, n_sets;
+  size_t search_size, replace_size;
+  unsigned int n_sets;
+  int offset;
   st_cm_set_t *sets;
 } st_cm_pattern_t;
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
 
-extern int change_mem (char *buf, int bufsize, char *searchstr, int strsize,
-                       char wc, char esc, char *newstr, int newsize, int offset, ...);
-extern int change_mem2 (char *buf, int bufsize, char *searchstr, int strsize,
-                        char wc, char esc, char *newstr, int newsize,
-                        int offset, st_cm_set_t *sets);
-extern int build_cm_patterns (st_cm_pattern_t **patterns, const char *filename, int verbose);
+extern int change_mem (char *buf, size_t bufsize, char *searchstr,
+                       size_t strsize, char wc, char esc, char *newstr,
+                       size_t newsize, int offset, ...);
+extern int change_mem2 (char *buf, size_t bufsize, char *searchstr,
+                        size_t strsize, char wc, char esc, char *newstr,
+                        size_t newsize, int offset, st_cm_set_t *sets);
+extern int build_cm_patterns (st_cm_pattern_t **patterns, const char *filename);
 extern void cleanup_cm_patterns (st_cm_pattern_t **patterns, int n_patterns);
+extern int cm_verbose;
 
 extern void clear_line (void);
 extern int ansi_init (void);
 extern char *ansi_strip (char *str);
-extern int gauge (time_t init_time, int pos, int size);
+extern int gauge (time_t init_time, size_t pos, size_t size);
 extern char *getenv2 (const char *variable);
-extern char *tmpnam2 (char *temp);
+extern char *tmpnam2 (char *tmpname, const char *basedir);
+extern void fread_checked (void *buffer, size_t size, size_t number, FILE *file);
+extern int fread_checked2 (void *buffer, size_t size, size_t number, FILE *file);
 //extern int renlwr (const char *path);
 #if     defined __unix__ && !defined __MSDOS__
 extern int drop_privileges (void);
@@ -349,50 +397,7 @@ extern int drop_privileges (void);
 extern int register_func (void (*func) (void));
 extern int unregister_func (void (*func) (void));
 extern void handle_registered_funcs (void);
-extern void wait2 (int nmillis);
-
-
-/*
-  Extended getopt(), usage and workflow handling
-
-  getopt2_usage()      a renderer for a nice usage output
-                         takes an st_getopt2_t array
-  getop2_parse_usage() parse usage output into st_getopt2_t
-                         array (for development)
-  getopt2_long()       turn st_getopt2_t into struct option for getopt1()
-  getopt2_short()      turn st_getopt2_t shortoptions string for getopt1()
-
-  getopt2_get_index_by_val()
-*/
-#include "getopt.h"           // getopt2 needs struct option from getopt1
-
-#define OPTION '-'
-#define OPTION_S "-"
-#define OPTION_LONG_S "--"
-#define OPTARG '='
-#define OPTARG_S "="
-
-typedef struct
-{
-  const char *name;           // see getopt()
-  int has_arg;                // see getopt()
-  int *flag;                  // see getopt()
-  int val;                    // see getopt()
-  const char *arg_name;       // name of the options arg as it should be
-                              // displayed in the --help output
-                              // "--name=arg_name" if has_arg == 1
-                              // "--name[=arg_name]" if has_arg == 2
-  const char *help;           // --help, -h, -? output for the current option
-  void *object;               // could be used for workflow objects
-} st_getopt2_t;
-
-extern void getopt2_usage (const st_getopt2_t *option);
-#ifdef  DEBUG
-extern void getopt2_parse_usage (const char *usage_output);
-#endif
-extern int getopt2_long (struct option *long_option, const st_getopt2_t *option, int n);
-extern int getopt2_short (char *short_option, const st_getopt2_t *option, int n);
-extern const st_getopt2_t *getopt2_get_index_by_val (const st_getopt2_t *option, int val);
+extern void wait2 (unsigned int nmillis);
 
 
 /*
@@ -445,11 +450,11 @@ extern int quick_io_c (int value, size_t start, const char *fname, const char *m
 #define q_fputc(f,s,b,m) (quick_io_c(b,s,f,m))
 typedef enum { SWAP_BYTE, SWAP_WORD } swap_t;
 
-extern int q_fncmp (const char *filename, int start, int len,
-                    const char *search, int searchlen, int wildcard);
-extern int q_fcpy (const char *src, int start, int len, const char *dest, const char *mode);
+extern int q_fncmp (const char *filename, size_t start, size_t len,
+                    const char *search, size_t searchlen, int wildcard);
+extern int q_fcpy (const char *src, size_t start, size_t len, const char *dest, const char *mode);
 extern int q_rfcpy (const char *src, const char *dest);
-extern int q_fswap (const char *filename, int start, int len, swap_t type);
+extern int q_fswap (const char *filename, size_t start, size_t len, swap_t type);
 #define q_fswap_b(f, s, l) q_fswap(f, s, l, SWAP_BYTE)
 #define q_fswap_w(f, s, l) q_fswap(f, s, l, SWAP_WORD)
 #if 1
@@ -460,7 +465,7 @@ extern char *q_fbackup (const char *filename, int mode);
 extern char *q_fbackup (char *move_name, const char *filename);
 #endif
 #ifndef  USE_ZLIB
-extern int q_fsize (const char *filename);
+extern off_t q_fsize (const char *filename);
 #endif
 
 
@@ -473,8 +478,7 @@ extern int q_fsize (const char *filename);
                   if the value of propname was 0, [Nn] or [Nn][Oo] and an
                   integer or at least 1 for every other case
   get_property_fname()  like get_property() but specifically for filenames,
-                  i.e., it runs realpath2() on the filename and fixes the
-                  characters if necessary (Cygwin)
+                  i.e., it runs realpath2() on the filename
   set_property()  set propname with value in filename
   DELETE_PROPERTY() like set_property but when value of propname is NULL the
                   whole property will disappear from filename
@@ -483,23 +487,23 @@ extern int q_fsize (const char *filename);
 #define PROPERTY_SEPARATOR_S "="
 #define PROPERTY_COMMENT '#'
 #define PROPERTY_COMMENT_S "#"
-extern char *get_property (const char *filename, const char *propname, char *value,
-                           const char *def);
+extern char *get_property (const char *filename, const char *propname,
+                           char *buffer, size_t bufsize, const char *def);
 extern int32_t get_property_int (const char *filename, const char *propname);
 extern char *get_property_fname (const char *filename, const char *propname,
                                  char *buffer, const char *def);
-extern int set_property (const char *filename, const char *propname, const char *value, const char *comment);
+extern int set_property (const char *filename, const char *propname,
+                         const char *value, const char *comment);
 #define DELETE_PROPERTY(a, b) (set_property(a, b, NULL, NULL))
 
 
 /*
   Portability (conio.h, etc...)
-  
+
   init_conio()         init console I/O
   deinit_conio()       stop console I/O
   getch()
   kbhit()
-  fix_character_set()  fixes some Cygwin problems with filenames
   truncate()
   sync()
   popen()
@@ -512,7 +516,7 @@ extern int set_property (const char *filename, const char *propname, const char
         defined __APPLE__                       // Mac OS X actually
 extern void init_conio (void);
 extern void deinit_conio (void);
-#define getch           getchar                 // getchar() acts like DOS getch() after init_conio()
+#define getch getchar                           // getchar() acts like DOS getch() after init_conio()
 extern int kbhit (void);                        // may only be used after init_conio()!
 
 #elif   defined __MSDOS__
@@ -524,31 +528,27 @@ extern int kbhit (void);                        // may only be used after init_c
 
 #elif   defined AMIGA
 extern int kbhit (void);
-//#define getch           getchar
+//#define getch getchar
 // Gonna use my (Jan-Erik) fake one. Might work better and more like the real
 //  getch().
 #endif
 
-#ifdef  __CYGWIN__
-extern char *fix_character_set (char *value);
-#endif
-
-
 #ifdef  _WIN32
 // Note that _WIN32 is defined by cl.exe while the other constants (like WIN32)
 //  are defined in header files. MinGW's gcc.exe defines all constants.
-
-#include <sys/types.h>
-
+#ifdef  __MINGW64__
+// MinGW-w64 already has truncate(), but with size of type off32_t...
+#define truncate _truncate
+#endif
 extern int truncate (const char *path, off_t size);
 extern int sync (void);
-// For MinGW popen() and pclose() are unavailable for DLL's. For DLL's _popen()
+// For MinGW popen() and pclose() are unavailable for DLLs. For DLLs _popen()
 //  and _pclose() should be used. Visual C++ only has the latter two.
-#ifndef pclose                                  // misc_z.h's definition gets higher "precedence"
-#define pclose  _pclose
+#ifndef pclose                                  // misc_z.h's definition gets higher precedence
+#define pclose _pclose
 #endif
 #ifndef popen                                   // idem
-#define popen   _popen
+#define popen _popen
 #endif
 
 #ifdef  USE_ANSI_COLOR
@@ -558,15 +558,16 @@ extern int vprintf2 (const char *format, va_list argptr);
 extern int printf2 (const char *format, ...);
 extern int fprintf2 (FILE *file, const char *format, ...);
 #define vprintf vprintf2
-#define printf  printf2
+#define printf printf2
 #define fprintf fprintf2
 #endif // USE_ANSI_COLOR
 
-#ifndef __MINGW32__
-#include <io.h>
-#include <direct.h>
-#include <sys/stat.h>                           // According to MSDN <sys/stat.h> must
-                                                //  come after <sys/types.h>. Yep, that's M$.
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4820) // 'bytes' bytes padding added after construct 'member_name'
+#include <sys/stat.h>
+#pragma warning(pop)
+
 #define S_IWUSR _S_IWRITE
 #define S_IRUSR _S_IREAD
 #define S_ISDIR(mode) ((mode) & _S_IFDIR ? 1 : 0)
@@ -582,30 +583,38 @@ extern int fprintf2 (FILE *file, const char *format, ...);
 #define STDERR_FILENO (fileno (stderr))
 
 #else
-#ifdef  DLL
-#define access  _access
-#define chmod   _chmod
-#define fileno  _fileno
-#define getcwd  _getcwd
-#define isatty  _isatty
-#define rmdir   _rmdir
-#define stat    _stat
-#define strdup  _strdup
+#if     defined DLL && !defined __MINGW64__
+#define access _access
+#define chdir _chdir
+#define chmod _chmod
+#define getcwd _getcwd
+#define isatty _isatty
+#ifndef USE_ZLIB
+#define snprintf _snprintf
+#endif
+#define stat _stat
+#define strdup _strdup
 #define stricmp _stricmp
 #define strnicmp _strnicmp
 #endif // DLL
 
-#endif // !__MINGW32__
+#endif // _MSC_VER
 
 #elif   defined AMIGA                           // _WIN32
+extern int truncate (const char *path, off_t size);
+extern int chmod (const char *path, mode_t mode);
+extern int sync (void);
+extern int readlink (const char *path, char *buf, int bufsize);
+
 // custom _popen() and _pclose(), because the standard ones (named popen() and
 //  pclose()) are buggy
-#ifndef pclose                                  // misc_z.h's definition gets higher "precedence"
-#define pclose  _pclose
+#ifndef pclose                                  // misc_z.h's definition gets higher precedence
+#define pclose _pclose
 #endif
 #ifndef popen                                   // idem
-#define popen   _popen
+#define popen _popen
 #endif
+
 extern FILE *_popen (const char *path, const char *mode);
 extern int _pclose (FILE *stream);
 #endif                                          // AMIGA
