@@ -1,8 +1,8 @@
 /*
 chksum.c - miscellaneous checksum functions
 
-Copyright (c) 1999 - 2004 NoisyB <noisyb@gmx.net>
-Copyright (c) 2001 - 2004 dbjh
+Copyright (c) 1999 - 2004       NoisyB
+Copyright (c) 2001 - 2004, 2021 dbjh
 
 sha1 - Copyright (c) 2002, Dr Brian Gladman <brg@gladman.me.uk>, Worcester, UK.
 
@@ -31,20 +31,17 @@ MD5  - Copyright (C) 1990, RSA Data Security, Inc. All rights reserved.
 #ifdef  HAVE_CONFIG_H
 #include "config.h"                             // USE_ZLIB
 #endif
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-//#ifdef HAVE_BYTESWAP_H
-//#include <byteswap.h>
-//#else
-#include "bswap.h"
-//#endif
-#ifdef  USE_ZLIB
-#include <zlib.h>
-#include "unzip.h"
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
 #endif
-#include "misc.h"
-#include "chksum.h"
+#include <stdlib.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
+#include "misc/bswap.h"
+#include "misc/chksum.h"
+#include "misc/misc.h"
 
 
 #if     (!defined TRUE || !defined FALSE)
@@ -66,7 +63,11 @@ MD5  - Copyright (C) 1990, RSA Data Security, Inc. All rights reserved.
 
 #if     defined _LIBC || defined __GLIBC__
   #include <endian.h>
-  #if __BYTE_ORDER == __BIG_ENDIAN
+  #if     __BYTE_ORDER == __BIG_ENDIAN
+    #define WORDS_BIGENDIAN 1
+  #endif
+#elif   defined __BYTE_ORDER__ && defined __ORDER_BIG_ENDIAN__
+  #if     __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
     #define WORDS_BIGENDIAN 1
   #endif
 #elif   defined AMIGA || defined __sparc__ || defined __BIG_ENDIAN__ || \
@@ -96,7 +97,7 @@ sha1_compile (s_sha1_ctx_t ctx[1])
 #define PARITY(x,y,z) ((x) ^ (y) ^ (z))
 #define MAJ(x,y,z) (((x) & (y)) ^ ((x) & (z)) ^ ((y) & (z)))
 
-  uint32_t w[80], i, a, b, c, d, e, t;
+  uint32_t w[80], i, a, b, c, d, e;
 
   /*
     note that words are compiled from the buffer into 32-bit
@@ -117,7 +118,7 @@ sha1_compile (s_sha1_ctx_t ctx[1])
 
   for (i = 0; i < 80; i++)
     {
-      t = a;
+      uint32_t t = a;
       a = ROTL32 (a, 5) + e + w[i];
       if (i < 20)
         a += CH (b, c, d) + 0x5a827999;
@@ -125,7 +126,7 @@ sha1_compile (s_sha1_ctx_t ctx[1])
         a += PARITY (b, c, d) + 0x6ed9eba1;
       else if (i < 60)
         a += MAJ (b, c, d) + 0x8f1bbcdc;
-      else if (i < 80)
+      else // if (i < 80)
         a += PARITY (b, c, d) + 0xca62c1d6;
       e = d;
       d = c;
@@ -456,24 +457,19 @@ md5_final (s_md5_ctx_t *mdContext)
 }
 
 
-// CRC16
-//#define WITH_CRC16                            // currently not used
-
-#if     !defined USE_ZLIB || defined WITH_CRC16
-
 #define CRC16_POLYNOMIAL 0xa001
 #define CRC32_POLYNOMIAL 0xedb88320
 
 
 void
 init_crc_table (void *table, unsigned int polynomial)
-// works for crc16 and crc32
+// works for CRC16 and CRC32
 {
-  unsigned int crc, i, j;
+  unsigned int i, j;
 
   for (i = 0; i < 256; i++)
     {
-      crc = i;
+      unsigned int crc = i;
       for (j = 8; j > 0; j--)
         if (crc & 1)
           crc = (crc >> 1) ^ polynomial;
@@ -486,10 +482,8 @@ init_crc_table (void *table, unsigned int polynomial)
         ((unsigned short *) table)[i] = (unsigned short) crc;
     }
 }
-#endif
 
 
-#ifdef  WITH_CRC16
 
 static unsigned short *crc16_table = NULL;
 
@@ -519,12 +513,10 @@ chksum_crc16 (unsigned short crc, const void *buffer, unsigned int size)
     crc = (crc >> 8) ^ crc16_table[(crc ^ *p++) & 0xff];
   return ~crc;
 }
-#endif
 
 
 // CRC32
 #ifndef USE_ZLIB
-
 static unsigned int *crc32_table = NULL;
 
 
