@@ -44,34 +44,38 @@
  * or the equivalent for your platform.
  */
 /*
-Portions copyright (c) 2001 - 2002 NoisyB <noisyb@gmx.net>
-Portions copyright (c) 2002        dbjh
+Portions copyright (c) 2001 - 2002                    NoisyB
+Portions copyright (c) 2002, 2016 - 2017, 2020 - 2021 dbjh
 */
-#ifdef  HAVE_CONFIG_H
-#include "config.h"
+#ifdef  _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4668) // 'symbol' is not defined as a preprocessor macro, replacing with '0' for 'directives'
 #endif
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <ctype.h>
+#ifdef  _MSC_VER
+#pragma warning(pop)
+#endif
+#include "misc/archive.h"
 #include "misc/file.h"
 #include "misc/misc.h"
-#ifdef  USE_ZLIB
-#include "misc/archive.h"
-#endif
-#include "misc/getopt2.h"                       // st_getopt2_t
-#include "ucon64.h"
 #include "ucon64_misc.h"
 #include "console/snes.h"
-#include "console/genesis.h"
-#include "console/nes.h"
-#include "console/sms.h"
-#include "console/gb.h"
-#include "gg.h"
+#include "patch/gg.h"
 
 
-#define GAME_GENIE_MAX_STRLEN 12
+#define GAME_GENIE_MAX_STRLEN 12 /* longest string is "XXX-XXX-XXX" */
 
+#define encodeNES(v, n, m, s) (data[n] |= ((v) >> (s)) & (m))
+#define decodeNES(v, n, m, s) (v |= (data[n] & (m)) << (s))
+#define encodeSNES(x, y) (transposed |= (((address & (0xc00000 >> (2*(y)))) << (2*(y))) >> (2*(x))))
+#define decodeSNES(x, y) (address |= (((transposed & (0xc00000 >> (2*(x)))) << (2*(x))) >> (2*(y))))
+
+
+static st_ucon64_obj_t gg_obj[] =
+  {
+    {0, WF_INIT | WF_PROBE},
+    {0, WF_INIT | WF_PROBE | WF_NO_ROM}
+  };
 
 const st_getopt2_t gg_usage[] =
   {
@@ -87,7 +91,7 @@ const st_getopt2_t gg_usage[] =
       "CODE" OPTARG_S "'AAAA:VV' or CODE" OPTARG_S "'AAAA:VV:CC'\n"
       OPTION_LONG_S "gge" OPTARG_S "CODE " OPTION_LONG_S "snes\n"
       "CODE" OPTARG_S "'AAAAAA:VV'",
-      &ucon64_wf[WF_OBJ_ALL_INIT_PROBE_NO_ROM]
+      &gg_obj[1]
     },
     {
       "ggd", 1, 0, UCON64_GGD,
@@ -101,7 +105,7 @@ const st_getopt2_t gg_usage[] =
       "GG_CODE" OPTARG_S "'XXXXXX' or GG_CODE" OPTARG_S "'XXXXXXXX'\n"
       OPTION_LONG_S "ggd" OPTARG_S "GG_CODE " OPTION_LONG_S "snes\n"
       "GG_CODE" OPTARG_S "'XXXX-XXXX'",
-      &ucon64_wf[WF_OBJ_ALL_INIT_PROBE_NO_ROM]
+      &gg_obj[1]
     },
     {
       "gg", 1, 0, UCON64_GG,
@@ -113,73 +117,20 @@ const st_getopt2_t gg_usage[] =
       "Genesis/Sega Mega Drive/Sega CD/32X/Nomad,\n"
       "Nintendo Entertainment System/NES/Famicom/Game Axe (Redant),\n"
       "Super Nintendo Entertainment System/SNES/Super Famicom",
-      &ucon64_wf[WF_OBJ_ALL_INIT_PROBE]
+      &gg_obj[0]
     },
     {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
 
 
-static st_rominfo_t *gg_rominfo;
+static st_ucon64_nfo_t *gg_rominfo;
 static int CPUaddress;
+static int gg_argc;
+static const char *gg_argv[128];
+static const char genesisChars[] = "ABCDEFGHJKLMNPRSTVWXYZ0123456789";
 
 
-/*********************************************************************
- *
- * GAME BOY / GAME GEAR  ROUTINES
- *
- *********************************************************************/
-
-static int gameGenieDecodeGameBoy (const char *in, char *out);
-static int gameGenieEncodeGameBoy (const char *in, char *out);
-
-/*********************************************************************
- *
- * MEGADRIVE  ROUTINES
- *
- *********************************************************************/
-
-static int isGenesisChar (char c);
-static int genesisValue (char c);
-static int gameGenieDecodeMegadrive (const char *in, char *out);
-static int gameGenieEncodeMegadrive (const char *in, char *out);
-
-/*********************************************************************
- *
- * NES ROUTINES
- *
- ********************************************************************/
-
-#define encodeNES(v, n, m, s) data[n] |= (v >> s) & m
-#define decodeNES(v, n, m, s) v |= (data[n] & m) << s
-
-static char mapNesChar (char hex);
-static char unmapNesChar (char genie);
-static int isNesChar (char c);
-static int gameGenieDecodeNES (const char *in, char *out);
-static int gameGenieEncodeNES (const char *in, char *out);
-
-/*********************************************************************
- *
- * SNES ROUTINES
- *
- ********************************************************************/
-
-#define encodeSNES(x, y) transposed |= (((address & (0xc00000 >> (2*y))) << (2*y)) >> (2*x))
-#define decodeSNES(x, y) address |= (((transposed & (0xc00000 >> (2*x))) << (2*x)) >> (2*y))
-
-static char mapSnesChar (char hex);
-static char unmapSnesChar (char genie);
-static int gameGenieDecodeSNES (const char *in, char *out);
-static int gameGenieEncodeSNES (const char *in, char *out);
-
-
-/*********************************************************************
- *
- * UTILITY ROUTINES
- *
- ********************************************************************/
-
-char
+static char
 hexDigit (int value)
 {
   switch (toupper (value))
@@ -220,14 +171,13 @@ hexDigit (int value)
       break;
     }
   return '?';
-
 }
 
 
-int
+static int
 hexValue (char digit)
 {
-  switch (toupper (digit))
+  switch (toupper ((int) digit))
     {
     case '0':
       return 0;
@@ -268,35 +218,29 @@ hexValue (char digit)
 }
 
 
-int
+static int
 hexByteValue (char x, char y)
 {
   return (hexValue (x) << 4) + hexValue (y);
 }
 
 
-/*********************************************************************
- *
- * GAME BOY / GAME GEAR  ROUTINES
- *
- *********************************************************************/
-
-int
+static int
 gameGenieDecodeGameBoy (const char *in, char *out)
 {
   int address = 0;
   int value = 0;
-  int check = 0;
   int haveCheck = 0;
   int i;
+  int in_len = (int) strlen (in);
 
-  if (strlen (in) == 11 && in[3] == '-' && in[7] == '-')
+  if (in_len == 11 && in[3] == '-' && in[7] == '-')
     haveCheck = 1;
-  else if (strlen (in) == 7 && in[3] == '-')
+  else if (in_len == 7 && in[3] == '-')
     haveCheck = 0;
   else
     return -1;
-  for (i = 0; i < (int) strlen (in); ++i)
+  for (i = 0; i < in_len; ++i)
     if (in[i] != '-' && !isxdigit ((int) in[i]))
       return -1;
   if (hexValue (in[6]) < 8)
@@ -310,7 +254,7 @@ gameGenieDecodeGameBoy (const char *in, char *out)
 
   if (haveCheck)
     {
-      check = hexByteValue (in[8], in[10]);
+      int check = hexByteValue (in[8], in[10]);
       check = ~check;
       check = (((check & 0xfc) >> 2) | ((check & 0x03) << 6)) ^ 0x45;
       sprintf (out, "%04X:%02X:%02X", address, value, check);
@@ -322,41 +266,37 @@ gameGenieDecodeGameBoy (const char *in, char *out)
 }
 
 
-int
+static int
 gameGenieEncodeGameBoy (const char *in, char *out)
 {
-  int check = 0;
   int haveCheck = 0;
   int i;
+  int in_len = (int) strlen (in);
 
-  if (strlen (in) == 10 && in[4] == ':' && in[7] == ':')
+  if (in_len == 10 && in[4] == ':' && in[7] == ':')
     haveCheck = 1;
-  else if (strlen (in) == 7 && in[4] == ':')
+  else if (in_len == 7 && in[4] == ':')
     haveCheck = 0;
   else
     return -1;
-  for (i = 0; i < (int) strlen (in); ++i)
+  for (i = 0; i < in_len; ++i)
     if (in[i] != ':' && !isxdigit ((int) in[i]))
       return -1;
   if (hexValue (in[0]) > 7)
     return -1;
 
-  out[0] = toupper (in[5]);
-  out[1] = toupper (in[6]);
-  out[2] = toupper (in[1]);
+  out[0] = (char) toupper ((int) in[5]);
+  out[1] = (char) toupper ((int) in[6]);
+  out[2] = (char) toupper ((int) in[1]);
   out[3] = '-';
-  out[4] = toupper (in[2]);
-  out[5] = toupper (in[3]);
+  out[4] = (char) toupper ((int) in[2]);
+  out[5] = (char) toupper ((int) in[3]);
   out[6] = hexDigit (~hexValue (in[0]) & 0xf);
-  out[7] = 0;
+  out[7] = '\0';
 
   if (haveCheck)
     {
-      check = hexByteValue (in[8], in[10]);
-      check = ~check;
-      check = (((check & 0xfc) >> 2) | ((check & 0x03) << 6)) ^ 0x45;
-
-      check = hexByteValue (in[8], in[9]);
+      int check = hexByteValue (in[8], in[9]);
       check ^= 0x45;
       check = ~(((check & 0xc0) >> 6) | ((check & 0x3f) << 2));
       i = (check & 0xf0) >> 4;
@@ -364,37 +304,28 @@ gameGenieEncodeGameBoy (const char *in, char *out)
       out[8] = hexDigit (i & 0xf);
       out[9] = hexDigit ((i ^ 8) & 0xf);
       out[10] = hexDigit (check & 0xf);
-      out[11] = 0;
+      out[11] = '\0';
     }
 
   return 0;
 }
 
 
-/*********************************************************************
- *
- * MEGADRIVE  ROUTINES
- *
- *********************************************************************/
-
-static const char genesisChars[] = "ABCDEFGHJKLMNPRSTVWXYZ0123456789";
-
-
-int
+static int
 isGenesisChar (char c)
 {
-  return strchr (genesisChars, toupper (c)) != 0;
+  return strchr (genesisChars, toupper ((int) c)) != 0;
 }
 
 
-int
+static int
 genesisValue (char c)
 {
-  return strchr (genesisChars, toupper (c)) - genesisChars;
+  return (int) (strchr (genesisChars, toupper ((int) c)) - genesisChars);
 }
 
 
-int
+static int
 gameGenieDecodeMegadrive (const char *in, char *out)
 {
   int address = 0;
@@ -413,7 +344,6 @@ gameGenieDecodeMegadrive (const char *in, char *out)
   for (i = 5; i < 9; ++i)
     data[i - 1] = genesisValue (in[i]);
 
-  address = 0;
   address |= (data[3] & 0x0f) << 20;
   address |= (data[4] & 0x1e) << 15;
   address |= (data[1] & 0x03) << 14;
@@ -422,13 +352,12 @@ gameGenieDecodeMegadrive (const char *in, char *out)
   address |= (data[6] & 0x07) << 5;
   address |= (data[7] & 0x1f);
 
-  value = 0;
   value |= (data[5] & 0x01) << 15;
   value |= (data[6] & 0x18) << 10;
   value |= (data[4] & 0x01) << 12;
   value |= (data[5] & 0x1e) << 7;
   value |= (data[0] & 0x1f) << 3;
-  value |= (data[1] & 0x16) >> 2;
+  value |= (data[1] & 0x1c) >> 2;
 
   sprintf (out, "%06X:%04X", address, value);
 
@@ -436,11 +365,11 @@ gameGenieDecodeMegadrive (const char *in, char *out)
 }
 
 
-int
+static int
 gameGenieEncodeMegadrive (const char *in, char *out)
 {
-  int address = 0;
-  int value = 0;
+  unsigned int address = 0;
+  unsigned int value = 0;
   int data[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
   int i;
 
@@ -465,29 +394,23 @@ gameGenieEncodeMegadrive (const char *in, char *out)
   data[4] |= (value >> 12) & 0x01;
   data[5] |= (value >> 7) & 0x1e;
   data[0] |= (value >> 3) & 0x1f;
-  data[1] |= (value << 2) & 0x16;
+  data[1] |= (value << 2) & 0x1c;
 
   for (i = 0; i < 4; ++i)
     out[i] = genesisChars[data[i]];
   out[4] = '-';
   for (i = 5; i < 9; ++i)
     out[i] = genesisChars[data[i - 1]];
-  out[9] = 0;
+  out[9] = '\0';
 
   return 0;
 }
 
 
-/*********************************************************************
- *
- * NES ROUTINES
- *
- ********************************************************************/
-
-char
+static char
 mapNesChar (char hex)
 {
-  switch (toupper (hex))
+  switch (toupper ((int) hex))
     {
     case '0':
       return 'A';
@@ -528,10 +451,10 @@ mapNesChar (char hex)
 }
 
 
-char
+static char
 unmapNesChar (char genie)
 {
-  switch (toupper (genie))
+  switch (toupper ((int) genie))
     {
     case 'A':
       return '0';
@@ -572,14 +495,14 @@ unmapNesChar (char genie)
 }
 
 
-int
+static int
 isNesChar (char c)
 {
-  return strchr ("APZLGITYEOXUKSVN-", toupper (c)) != 0;
+  return strchr ("APZLGITYEOXUKSVN-", toupper ((int) c)) != 0;
 }
 
 
-int
+static int
 gameGenieDecodeNES (const char *in, char *out)
 {
   int address = 0;
@@ -588,14 +511,15 @@ gameGenieDecodeNES (const char *in, char *out)
   int haveCheck = 0;
   int data[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
   int i;
+  int in_len = (int) strlen (in);
 
-  if (strlen (in) == 8)
+  if (in_len == 8)
     haveCheck = 1;
-  else if (strlen (in) == 6)
+  else if (in_len == 6)
     haveCheck = 0;
   else
     return -1;
-  for (i = 0; i < (int) strlen (in); ++i)
+  for (i = 0; i < in_len; ++i)
     if (!isNesChar (in[i]))
       return -1;
 
@@ -607,7 +531,6 @@ gameGenieDecodeNES (const char *in, char *out)
       data[7] = hexValue (unmapNesChar (in[7]));
     }
 
-  address = 0x8000;             /* force high bit on */
   decodeNES (address, 1, 8, 4);
   decodeNES (address, 2, 7, 4);
   decodeNES (address, 3, 7, 12);
@@ -622,7 +545,7 @@ gameGenieDecodeNES (const char *in, char *out)
       decodeNES (value, 1, 7, 4);
       decodeNES (value, 7, 8, 0);
       decodeNES (check, 6, 7, 0);
-      decodeNES (check, 6, 8, 0);
+      decodeNES (check, 5, 8, 0);
       decodeNES (check, 6, 8, 4);
       decodeNES (check, 7, 7, 4);
     }
@@ -643,23 +566,24 @@ gameGenieDecodeNES (const char *in, char *out)
 }
 
 
-int
+static int
 gameGenieEncodeNES (const char *in, char *out)
 {
-  int address = 0x8000;
-  int value = 0;
-  int check = 0;
+  unsigned int address = 0;
+  unsigned int value = 0;
+  unsigned int check = 0;
   int haveCheck = 0;
   int data[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
   int i;
+  int in_len = (int) strlen (in);
 
-  if (strlen (in) == 10 && in[4] == ':' && in[7] == ':')
+  if (in_len == 10 && in[4] == ':' && in[7] == ':')
     haveCheck = 1;
-  else if (strlen (in) == 7 && in[4] == ':')
+  else if (in_len == 7 && in[4] == ':')
     haveCheck = 0;
   else
     return -1;
-  for (i = 0; i < (int) strlen (in); ++i)
+  for (i = 0; i < in_len; ++i)
     if (in[i] != ':' && !isxdigit ((int) in[i]))
       return -1;
 
@@ -686,7 +610,7 @@ gameGenieEncodeNES (const char *in, char *out)
       encodeNES (value, 1, 7, 4);
       encodeNES (value, 7, 8, 0);
       encodeNES (check, 6, 7, 0);
-      encodeNES (check, 6, 8, 0);
+      encodeNES (check, 5, 8, 0);
       encodeNES (check, 6, 8, 4);
       encodeNES (check, 7, 7, 4);
     }
@@ -725,16 +649,10 @@ gameGenieEncodeNES (const char *in, char *out)
 }
 
 
-/*********************************************************************
- *
- * SNES ROUTINES
- *
- ********************************************************************/
-
-char
+static char
 mapSnesChar (char hex)
 {
-  switch (toupper (hex))
+  switch (toupper ((int) hex))
     {
     case '0':
       return 'D';
@@ -775,10 +693,10 @@ mapSnesChar (char hex)
 }
 
 
-char
+static char
 unmapSnesChar (char genie)
 {
-  switch (toupper (genie))
+  switch (toupper ((int) genie))
     {
     case 'D':
       return '0';
@@ -819,7 +737,7 @@ unmapSnesChar (char genie)
 }
 
 
-int
+static int
 gameGenieDecodeSNES (const char *in, char *out)
 {
   int value, hirom, address, transposed;
@@ -834,11 +752,11 @@ gameGenieDecodeSNES (const char *in, char *out)
   value = hexByteValue (unmapSnesChar (in[0]), unmapSnesChar (in[1]));
 
   transposed = hexValue (unmapSnesChar (in[8])) +
-    (hexValue (unmapSnesChar (in[7])) << 4) +
-    (hexValue (unmapSnesChar (in[6])) << 8) +
-    (hexValue (unmapSnesChar (in[5])) << 12) +
-    (hexValue (unmapSnesChar (in[3])) << 16) +
-    (hexValue (unmapSnesChar (in[2])) << 20);
+               (hexValue (unmapSnesChar (in[7])) << 4) +
+               (hexValue (unmapSnesChar (in[6])) << 8) +
+               (hexValue (unmapSnesChar (in[5])) << 12) +
+               (hexValue (unmapSnesChar (in[3])) << 16) +
+               (hexValue (unmapSnesChar (in[2])) << 20);
 
   address = 0;
   decodeSNES (0, 4);
@@ -854,24 +772,24 @@ gameGenieDecodeSNES (const char *in, char *out)
   decodeSNES (10, 3);
   decodeSNES (11, 6);
 
-  // if a ROM was specified snes.c will handle ucon64.snes_hirom
-  if (UCON64_ISSET (ucon64.snes_hirom))         // -hi or -nhi option was specified
+  /* if a ROM was specified snes.c will handle ucon64.snes_hirom */
+  if (UCON64_ISSET (ucon64.snes_hirom))         /* -hi or -nhi switch was specified */
     hirom = ucon64.snes_hirom;
-  // if only a ROM was specified (not -hi or -nhi) the next if will fail for a
-  //  handful of ROMs, namely Sufami Turbo ROMs and Extended ROMs
+  /* if only a ROM was specified (not -hi or -nhi) the next if will fail for a
+      handful of ROMs, namely Sufami Turbo ROMs and Extended ROMs */
   else if (gg_rominfo != 0)
     hirom = gg_rominfo->header_start > SNES_HEADER_START ? 1 : 0;
   else
-    hirom = 1;                                  // I am less sure about the LoROM
-                                                //  CPU -> ROM address conversion
+    hirom = 1;                                  /* I am less sure about the LoROM */
+                                                /*  CPU -> ROM address conversion */
   CPUaddress = address;
   if (hirom)
     {
-      if (address >= 0xc00000 && address <= 0xffffff)
+      if (address >= 0xc00000 /* && address <= 0xffffff */)
         address -= 0xc00000;
-      else if (address >= 0x800000 && address <= 0xbfffff)
+      else if (address >= 0x800000 /* && address <= 0xbfffff */)
         address -= 0x800000;
-      else if (address >= 0x400000 && address <= 0x7fffff)
+      else if (address >= 0x400000 /* && address <= 0x7fffff */)
         address -= 0x400000;
     }
   else
@@ -889,10 +807,11 @@ gameGenieDecodeSNES (const char *in, char *out)
 }
 
 
-int
+static int
 gameGenieEncodeSNES (const char *in, char *out)
 {
-  int value, address, transposed;
+  int value, transposed;
+  unsigned int address;
 
   if (!isxdigit ((int) in[0]) || !isxdigit ((int) in[1]) ||
       !isxdigit ((int) in[2]) || !isxdigit ((int) in[3]) ||
@@ -945,7 +864,7 @@ usage (void)
 #endif
 
 
-int
+static int
 gg_main (int argc, const char **argv)
 {
   char buffer[GAME_GENIE_MAX_STRLEN];
@@ -998,25 +917,34 @@ gg_main (int argc, const char **argv)
             }
         }
       if (result != 0)
-        printf ("%-12s is badly formed\n", argv[i]);
+        printf ("%s is badly formed\n", argv[i]);
       else
         {
-          if (CPUaddress != -1)                 // SNES decode specific
-            printf ("%-12s = %s (CPU address: %06X)\n", argv[i], buffer, CPUaddress);
+          if (CPUaddress != -1)                 /* SNES decode specific */
+            printf ("%s = %s (CPU address: %06X)\n", argv[i], buffer, CPUaddress);
           else
-            printf ("%-12s = %s\n", argv[i], buffer);
+            printf ("%s = %s\n", argv[i], buffer);
         }
     }
   return 0;
 }
 
 
-int gg_argc;
-const char *gg_argv[128];
+static void
+apply_code (FILE *destfile, int offset, char *bufold, char *bufnew, int size)
+{
+  fputc ('\n', stdout);
+  dumper (stdout, bufold, size, offset, DUMPER_HEX);
+
+  fseek (destfile, offset, SEEK_SET);
+  fwrite (bufnew, 1, size, destfile);
+
+  dumper (stdout, bufnew, size, offset, DUMPER_HEX);
+}
 
 
 int
-gg_display (st_rominfo_t *rominfo, const char *code)
+gg_display (st_ucon64_nfo_t *rominfo, const char *code)
 {
   gg_argv[0] = "uggconv";
 
@@ -1041,14 +969,14 @@ gg_display (st_rominfo_t *rominfo, const char *code)
 
     default:
       fprintf (stderr,
-           "ERROR: You must specify a ROM or force the console type\n"
-           "       The force recognition option for SNES would be " OPTION_LONG_S "snes\n");
+               "ERROR: You must specify a ROM or force the console type\n"
+               "       For example, the force recognition switch for SNES is " OPTION_LONG_S "snes\n");
       return -1;
     }
   gg_argv[2] = code;
   gg_argc = 3;
 
-  if (ucon64.file_size > 0)                     // check if rominfo contains valid ROM info
+  if (rominfo && ucon64.fsize > 0)              /* check if rominfo contains valid ROM info */
     gg_rominfo = rominfo;
   else
     gg_rominfo = 0;
@@ -1060,13 +988,17 @@ gg_display (st_rominfo_t *rominfo, const char *code)
 
 
 int
-gg_apply (st_rominfo_t *rominfo, const char *code)
+gg_apply (st_ucon64_nfo_t *rominfo, const char *code)
 {
-  int size = ucon64.file_size - rominfo->buheader_len, address, value,
-      result = -1;
-  char buf[MAXBUFSIZE], dest_name[FILENAME_MAX];
+  unsigned int size = (unsigned int) ucon64.fsize -
+                         (rominfo ? rominfo->backup_header_len : 0),
+               offset, address, value, writefile;
+  int check = -1, result = -1;
+  char buf[GAME_GENIE_MAX_STRLEN], buf2[GAME_GENIE_MAX_STRLEN],
+       dest_name[FILENAME_MAX];
+  FILE *destfile;
 
-  if (ucon64.file_size > 0)                     // check if rominfo contains valid ROM info
+  if (rominfo && ucon64.fsize > 0)              /* check if rominfo contains valid ROM info */
     gg_rominfo = rominfo;
   else
     {
@@ -1077,8 +1009,6 @@ gg_apply (st_rominfo_t *rominfo, const char *code)
   CPUaddress = -1;
   switch (ucon64.console)
     {
-      // interleaved images (GD3 (SNES) & SMD (Genesis)) should be allowed to
-      //  be patched
       case UCON64_GB:
       case UCON64_SMS:
         result = gameGenieDecodeGameBoy (code, buf);
@@ -1102,32 +1032,104 @@ gg_apply (st_rominfo_t *rominfo, const char *code)
       return -1;
     }
 
-  if (CPUaddress != -1)                         // SNES decode specific
-    printf ("%-12s = %s (CPU address: %06X)\n", code, buf, CPUaddress);
+  if (CPUaddress != -1)                         /* SNES decode specific */
+    printf ("%s = %s (CPU address: %06X)\n", code, buf, CPUaddress);
   else
-    printf ("%-12s = %s\n", code, buf);
-  sscanf (buf, "%x:%x:*", &address, &value);
+    printf ("%s = %s\n", code, buf);
+  sscanf (buf, "%x:%x:%x", &address, &value, (unsigned int *) &check);
 
   if (address >= size)
     {
-      fprintf (stderr, "ERROR: Address is too high for this ROM (%d)\n", address);
+      fprintf (stderr, "ERROR: Address is too high for this ROM (%u)\n", address);
       return -1;
     }
 
-  strcpy (dest_name, ucon64.rom);
-  ucon64_file_handler (dest_name, NULL, 0);
-  fcopy (ucon64.rom, 0, ucon64.file_size, dest_name, "wb"); // no copy if one file
+  strcpy (dest_name, ucon64.fname);
+  writefile = !(ucon64.console == UCON64_NES && check == -1);
 
-  fputc ('\n', stdout);
-  buf[0] = ucon64_fgetc (dest_name, address + rominfo->buheader_len);
-  dumper (stdout, buf, 1, address + rominfo->buheader_len, DUMPER_HEX);
+  if (writefile)
+    {
+      ucon64_file_handler (dest_name, NULL, 0);
+      fcopy (ucon64.fname, 0, ucon64.fsize, dest_name, "wb"); /* no copy if one file */
 
-  ucon64_fputc (dest_name, address + rominfo->buheader_len, value, "r+b");
+      if ((destfile = fopen (dest_name, "r+b")) == NULL)
+        {
+          fprintf (stderr, ucon64_msg[OPEN_WRITE_ERROR], dest_name);
+          return -1;
+        }
+    }
+  else
+    {
+      puts ("NOTE: A 6 digit code is too generic. File will not be modified.\n"
+            "      Listing possible 8 digit codes");
+      if ((destfile = fopen (dest_name, "rb")) == NULL)
+        {
+          fprintf (stderr, ucon64_msg[OPEN_READ_ERROR], dest_name);
+          return -1;
+        }
+    }
 
-  buf[0] = value;
-  dumper (stdout, buf, 1, address + rominfo->buheader_len, DUMPER_HEX);
-  fputc ('\n', stdout);
+  offset = address + rominfo->backup_header_len;
+  if (ucon64.console == UCON64_NES)
+    {
+      while (offset < ucon64.fsize)
+        {
+          fseek (destfile, offset, SEEK_SET);
+          buf[0] = (char) fgetc (destfile);
 
-  printf (ucon64_msg[WROTE], dest_name);
+          if (check != -1)
+            { /* 8 digit code */
+              if (buf[0] == (char) check)
+                {
+                  buf2[0] = (char) value;
+                  apply_code (destfile, offset, buf, buf2, 1);
+                }
+            }
+          else
+            { /* 6 digit code => display the corresponding 8 digit code(s) */
+              char longcode[9];
+
+              fputc ('\n', stdout);
+              dumper (stdout, buf, 1, offset, DUMPER_HEX);
+
+              address = offset - rominfo->backup_header_len;
+              sprintf (buf, "%04X:%02X:%02X", address & 0xffff, value,
+                       (unsigned char) buf[0]);
+              gameGenieEncodeNES (buf, longcode);
+              printf ("%s = %s\n", buf, longcode);
+            }
+
+          offset += 8 * 1024;
+        }
+    }
+  else if (ucon64.console == UCON64_GEN)
+    {
+      fseek (destfile, offset, SEEK_SET);
+      buf[0] = (char) fgetc (destfile);
+      buf[1] = (char) fgetc (destfile);
+
+      buf2[0] = (char) (value >> 8);
+      buf2[1] = (char) value;
+      apply_code (destfile, offset, buf, buf2, 2);
+    }
+  else /* UCON64_SNES || UCON64_SMS || UCON64_GB */
+    {
+      fseek (destfile, offset, SEEK_SET);
+      buf[0] = (char) fgetc (destfile);
+
+      /* for SNES check is always -1 */
+      if (check == -1 || buf[0] == (char) check)
+        {
+          buf2[0] = (char) value;
+          apply_code (destfile, offset, buf, buf2, 1);
+        }
+    }
+
+  fclose (destfile);
+  if (writefile)
+    {
+      fputc ('\n', stdout);
+      printf (ucon64_msg[WROTE], dest_name);
+    }
   return 0;
 }
