@@ -2,7 +2,7 @@
 gg.h - GameGenie support for uCON64
 
 Copyright (c) 2001 WyrmCorp <http://wyrmcorp.com>.
-Copyright (c) 2002 NoisyB <noisyb@gmx.net>
+Copyright (c) 2002 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,13 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #ifndef GG_H
 #define GG_H
+
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
 extern const st_getopt2_t gg_usage[];
-extern int gg_apply (st_rominfo_t *rominfo, const char *code);
-extern int gg_display (st_rominfo_t *rominfo, const char *code);
+
+extern int gg_apply (st_ucon64_nfo_t *rominfo, const char *code);
+extern int gg_display (st_ucon64_nfo_t *rominfo, const char *code);
+
 #endif
