@@ -1,7 +1,7 @@
 /*
-patch.h - single header file for all patch functions
+patch.h - patch support for uCON64
 
-Copyright (c) 2003 NoisyB <noisyb@gmx.net>
+Copyright (c) 2003 - 2006 NoisyB
 
 
 This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,9 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef PATCH_H
 #define PATCH_H
 
-#include "aps.h"
-#include "bsl.h"
-#include "gg.h"
-#include "ips.h"
-#include "pal4u.h"
-#include "ppf.h"
-#include "xps.h"
+#include "misc/getopt2.h"                       // st_getopt2_t
+
+
+extern const st_getopt2_t patch_usage[];
 
 #endif
