@@ -1,7 +1,7 @@
 /*
-ucon64_opts.h - switch()'es for all uCON64 options
+ucon64_opts.h - switches for all uCON64 options
 
-Copyright (c) 2002 - 2004 NoisyB <noisyb@gmx.net>
+Copyright (c) 2002 - 2004 NoisyB
 Copyright (c) 2002 - 2003 dbjh
 
 
@@ -21,6 +21,11 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #ifndef UCON64_OPTS_H
 #define UCON64_OPTS_H
+
+#include "ucon64.h"
+
+
 extern int ucon64_switches (st_ucon64_t *p);
 extern int ucon64_options (st_ucon64_t *p);
-#endif // UCON64_OPTS_H
+
+#endif
